<?php
/**
 * Projekt: WS-Reporter
 *          Report-Definition in Text-Datei exportieren
 *
 * Autor:    Wilfried Schollenberger
 *           WS Unternehmensberatung und Controlling-Systeme GmbH
 * Kontakt:  wisch@ws-unternehmensberatung.de
 * Lizenz:   GPL 3.0
 *
 * Stand:    23.2.2023
 *
 */
include 'includes/defender.inc';
require_once './includes/funktionen.inc';
require_once './includes/classes.inc';
global  $query;
require_once './phpsec/createDBH.inc';

/**
 * 
 * @var Query $query
 */
$query = null;
if (session_status() == PHP_SESSION_NONE)  session_start();
anmeldungPruefen();

if (array_key_exists('WSReportQuery', $_SESSION)){
    $query = $_SESSION['WSReportQuery'];
}
session_abort();

if (array_key_exists('reportID', $_REQUEST)){
    if ($query == null || $query->reportID != $_REQUEST['reportID']){
        require_once './includes/loadReportDef.inc';
        loadReportID($_REQUEST['reportID']);
    }
    if ($query == null || $query->reportID != $_REQUEST['reportID']){
        echo "FEHLER:<br>Die Report-Defintion Nr. ".$_REQUEST['reportID']." konnte nicht geladen werden.";
        die();
    }
}
else
if (array_key_exists('report', $_REQUEST)){
    if ($query == null || $query->title != $_REQUEST['report']){
        require_once './includes/loadReportDef.inc';
        loadReport($_REQUEST['report']);
    }
    if ($query == null || $query->title != $_REQUEST['report']){
        echo "FEHLER:<br>Die Report-Defintion ".$_REQUEST['report']." konnte nicht geladen werden.";
        die();
    }
}
if ($query == null
    && !array_key_exists('reportID', $_REQUEST)
    && !array_key_exists('report', $_REQUEST)){
    echo "FEHLER:<br>Keine Report-Anforderung gefunden.";
    die();
}
rechteZuruecksetzen();

/* Export erlauben, damit Admin neuen Report importieren kann
if (!dqAdminRechte()){
    http_response_code(425);
    echo "Für diese Operation sind die Administrator-Rechte \nfür die Datenquelle erforderlich";
    die;
}
*/



header("Content-Type: text/plain; charset=UTF-8");
header("Content-Disposition: attachment; filename=\"".$query->title.'-'.date_create('now')->format('Y-m-d').".wsr\"");

// var_dump($_REQUEST);
if (false) $query = new Query("dummy");

echo "<WS-Report Report Version 1.0>\n";
echo "<class>com.ws.servlets.GenericWSReportJoinQuery\n";
echo "<classTable>".$query->class_table."\n";
echo "<reportTitle>".$query->title."\n";
if ($query->reportID > 0){
    $logSql = new SqlAbfrage('WSR');
    $logSql->sql = 'select userStored, timeStored, userReleased, timeReleased from wsrAdminReport';
    $logSql->addSel('reportID = ', $query->reportID);
    $stmt = $logSql->executeE(false);
    $z = $stmt->fetchObject();
    if ($z !== false){
        if (isValid($z->userStored)) echo '<userStored>'.$z->userStored."\n";
        if (isValid($z->timeStored)) echo '<timeStored>'.$z->timeStored."\n";
        if (isValid($z->userReleased)) echo '<userReleased>'.$z->userReleased."\n";
        if (isValid($z->timeReleased)) echo '<timeReleased>'.$z->timeReleased."\n";
        
    }
}
echo "<sortMode>".$query->sort_mode."\n";
echo "<chartSize>".$query->chart_size."\n";
echo "<displayCharts>".$query->display_charts."\n";
if ($query->fix_labelColumns) echo "<fixierte Label>\n";
if ($query->dropSummen) echo "<drop Summen>\n";
if ($query->values_in_columns) echo "<Werte in Spalten>\n";
if ($query->repGeldFaktor != 1) echo "<repGeldFaktor>".$query->repGeldFaktor."\n";
if ($query->folderMode >= 0) echo "<repFolderMode>".$query->folderMode."\n";
if (isValid($query->valueColumnFolderMode)) echo "<repValueColumnFolderMode>".$query->valueColumnFolderMode."\n";

if (isValid($query->rLanguage)) echo "<rLanguage>".$query->rLanguage."\n";
if (count($query->rTranslations) > 0){
    echo "<rTranslations>".$query->rTranslations[0];
    for ($i = 1; $i < count($query->rTranslations); $i ++) echo ' '.$query->rTranslations[$i];
    echo "\n";
}

if ($query->repWerteBasis != null) echo "<repWerteBasis>".$query->repWerteBasis."\n";
if ($query->repWerteAuswahlTitel != null) echo "<repWerteAuswahlTitel>".$query->repWerteAuswahlTitel."\n";
if ($query->repWerteAuswahlLabel) echo "<repWerteAuswahlLabel>\n";
if ($query->repWerteAuswahlSelector) echo "<repWerteAuswahlSelector>\n";




if ($query->histType){
    echo "<histOption>".$query->historyOption."\n";
    if (isValid($query->historyDate)) echo "<histDate>".$query->historyDate."\n";
}

foreach($query->dimensions as $dimNr => $dim){
    if (false) $dim = new Dimension(null);
    if (isValid($dim->historyOption) &&  $query->reportHasDim('base', $dimNr)){
        echo "<dimHistOption>".$dimNr.'.'.$dim->historyOption."\n";
        if (isValid($dim->historyDate)) 
            echo "<dimHistDate>".$dimNr.'.'.$dim->historyDate."\n";        
    }
}

if ($query->repTables != null)
    for($i=0; $i < $query->repTables->size(); $i++){
        $jt = $query->repTables->getPos($i);
        echo "<repTable>".$jt->kennzeichen."\n";
        echo "<repTableClass>".$jt->query->class_table."\n";
        echo "<repTablePrefix>".$jt->prefix."\n";
        echo "<repTableJoinType>".$jt->joinType."\n";
        foreach($jt->dimHistoryOption as $dimNr => $opt){
            if (isValid($opt) &&  $query->reportHasDim($jt->kennzeichen, $dimNr)){
                echo "<repTableDimHistOption>".$dimNr.'.'.$opt."\n";
                if (isValid($jt->dimHistoryDate[$dimNr]))
                    echo "<repTableDimHistDate>".$dimNr.'.'.$jt->dimHistoryDate[$dimNr]."\n";
            }
        }
        echo "</repTable>\n";
        
}

// $row = new RepRow("XX");
for ($i=0; $i < $query->repRows->size(); $i++){
    $row = $query->repRows->getPos($i);
    echo '<repRow>'.$row->getName()."\n";
    echo '<repRowSort>'.$row->sort."\n";
    echo '<repRowWebLink>'.$row->web_link."\n";
    if (isValid($row->web_title)) echo '<repRowWebLinkTitle>'.$row->web_title."\n";
    if (isValid($row->web_target)) echo '<repRowWebLinkTarget>'.$row->web_target."\n";
    if ($row->webLinkColumn){
        echo "<repRowWebLinkColumn>\n";
        if (isValid($row->web_link_title)) echo '<repRowWebLinkTitle>'.$row->web_link_title."\n";
        if (isValid($row->web_link_value)) echo '<repRowWebLinkValue>'.$row->web_link_value."\n";
    }    
    if ($row->summenZeilen) echo "<repRowSummenzeile>\n";
    if ($row->webCopyFilter) echo "<repRowCopyFilter>\n";
    if ($row->fixedTableColumns) echo "<repRowFixierteLabelspalte>\n";
    echo "</repRow>\n";
}

for ($i=0; $i < $query->repValues->size(); $i++){
    /**
     * @var RepValue $row
     */
    $row = $query->repValues->getPos($i);
    echo '<repValue>'.$row->getName()."\n";
    if ($row->label != null) echo '<repValueLabel>'.$row->label."\n";
    echo '<repValueDisp>'.$row->dispVal."\n";
    if (isValid($row->backgroundColor)) echo '<repValueBackgroundColor>'.$row->backgroundColor."\n";
    echo '<repValueChartScale>'.$row->skalierung."\n";
    if ($row->keineSummen)  echo "<repValueKeineSummen>\n";
    if (isValid($row->base) && $row->base != 'base') echo "<repValueCalcBase>".$row->base."\n"; 
    if ($row->function > 0){
        echo "<repValueFunction>".$row->function."\n";
        // Nur zum Verständnis
        echo "<repValueFunctionText>".RepValue::getFunctionText($row->function)."\n";
        echo "<repValueFunctionValue>".$row->value."\n";
        echo "<repValueFunctionAbSpalte>".$row->abSpalte ."\n";
        echo "<repValueFunctionNachSpalten>".$row->nachSpalten."\n";
    }
    
    echo "</repValue>\n";
}

for ($i=0; $i < $query->repFilter->size(); $i++){
    $row = $query->repFilter->getPos($i);
    echo '<repFilter>'.$row->getName()."\n";
    echo '<repFilterVon>'.$row->fs_von."\n";
    echo '<repFilterBis>'.$row->fs_bis."\n";
    echo '<repFilterAbSpalte>'.$row->abSpalte."\n";
    echo '<repFilterInTitelZeile>'.$row->inTitelZeile."\n";
    if ($row->inTitelMitLabel) echo "<repFilterInTitelMitLabel>\n";
    if ($row->selectorImOutput) echo "<repFilterSelectorImOutput>\n";
    if ($row->reportCopyFilter) echo "<repFilterReportCopyFilter>\n";
    if ($row->doNotTake) echo "<repFilterDoNotTake>\n";
    if ($row->fs_list != null && count($row->fs_list) > 0){
        if ($row->ausschlussListe)  echo '<repFilterListe>ausschlussListe'."\n";
        else echo '<repFilterListe>auswahlListe'."\n";
        for ($il=0; $il < count($row->fs_list);$il++){
            echo '<repFilterListElement>'.$row->fs_list[$il]."\n";
        }
        echo "</repFilterListe>\n";
    }
    echo "</repFilter>\n";
}

for ($i=0; $i < $query->addFilter->size(); $i++){
    $row = $query->addFilter->getPos($i);
    echo '<addFilter>'.$row->getName()."\n";
    echo '<addFilterVon>'.$row->fs_von."\n";
    echo '<addFilterBis>'.$row->fs_bis."\n";
    echo '<addFilterAbSpalte>'.$row->abSpalte."\n";
    echo '<addFilterInTitelZeile>'.$row->inTitelZeile."\n";
    if ($row->selectorImOutput) echo "<addFilterSelectorImOutput>\n";
    if ($row->fs_list != null && count($row->fs_list) > 0){
        if ($row->auschlussListe)  echo '<addFilterListe>ausschlussListe'."\n";
        else echo '<addFilterListe>auswahlListe'."\n";
        for ($il=0; $il < count($row->fs_list);$il++){
            echo '<addFilterListElement>'.$row->fs_list[$il]."\n";
        }
        echo "</addFilterListe>\n";
    }
    echo "</addFilter>\n";
}

for ($i=0; $i < $query->repColumns->size(); $i++){
    /**
     * @var RepColumn $row;
     */
    $row = $query->repColumns->getPos($i);
    echo '<repColumn>'.$row->getName()."\n";
    echo '<repColumnLabel>'.$row->label."\n";
    echo '<repColumnValName>'.$row->val_name."\n";
    echo '<repColumnDispVal>'.$row->displayOption."\n";
    echo '<repColumnChartScale>'.$row->skalierung."\n";
    echo '<repColumnFilter>'.$row->filter."\n";
    echo '<repColumnVon>'.$row->fs_von."\n";
    echo '<repColumnBis>'.$row->fs_bis."\n";
    if (isValid($row->web_link)) echo '<repColumnLink>'.$row->web_link."\n";
    if (isValid($row->web_title)) echo '<repColumnLinkTitle>'.$row->web_title."\n";
    if (isValid($row->web_target)) echo '<repColumnLinkTarget>'.$row->web_target."\n";
    if ($row->keineSummen)  echo "<repColumnKeineSummen>\n";
    if ($row->sort > 0) echo "<repColumnSort>".$row->sort."\n";
    if ($row->reportCopyFilter) echo "<repColumnCopyFilter>\n";
    if ($row->reportLinkInValues) echo "<repColumnLinkInValues>\n";

    if ($row->function > 0){
        echo "<repColumnFunction>".$row->function."\n";
        // Nur zum Verständnis
        echo "<repColumnFunctionText>".RepValue::getFunctionText($row->function)."\n";
        echo "<repColumnFunctionAbSpalte>".$row->abSpalte ."\n";
        echo "<repColumnFunctionNachSpalten>".$row->nachSpalten."\n";
    }
    
    
    echo "</repColumn>\n";
}

for ($i=0; $i < $query->repReports->size(); $i++){
    $rep = $query->repReports->getPos($i);
    echo '<repReport>'.$rep->getName()."\n";
    echo '<repReportKommentar>'.$rep->kommentar."\n";
    if ($rep->copyFilter) echo "<repReportCopyFilter>\n";
    echo "</repReport>\n";
}


if (isValid($query->kommentar)){
    echo "<kommentar>\n";
    echo trim(str_replace("\r",'',$query->kommentar));
    echo "\n</kommentar>\n";
}

if (isValid($query->report_hints)){
    echo "<hints>\n";
    echo trim(str_replace("\r",'',$query->report_hints));
    echo "\n</hints>\n";
}


// Mehrsprachige Title
$sql = new SqlAbfrage('WSR');
$sql->sql = 'select reportTitle_de,reportTitle_en, reportTitle_fr from reportTitle';
$sql->addSel('reportID = ', $query->reportID);
$stmt = $sql->executeE(true);
if ($stmt !== false){
    if($z = $stmt->fetchObject()){
        if ($query->rLanguage != 'de') echo '<reportTitle_de>'.$z->reportTitle_de."\n";
        if ($query->rLanguage != 'en') echo '<reportTitle_en>'.$z->reportTitle_en."\n";
        if ($query->rLanguage != 'fr') echo '<reportTitle_fr>'.$z->reportTitle_fr."\n";
        echo "</reportTitle>\n";
    }
}
// Übersetzte Kommentare
foreach ($query->rTranslations as $tr){
    if (isValid($tr)  && $tr != $query->rLanguage){
        $sql = new SqlAbfrage('WSR');
        $sql->sql = 'select kommentar from reportComment';
        $sql->addSel('reportID = ',$query->reportID);
        $sql->addSel('rLanguage = ',$tr);
        $sql->sql .= ' order by SortNr';
        $stmt = $sql->executeE(true);
        if ($stmt !== false){
            $rcCount = 0;
            while ($z = $stmt->fetchObject()){
                if ($rcCount == 0) echo '<kommentarTranslated>'.$tr."\n";
                $rcCount ++;
                echo $z->kommentar;
            }
            echo "\n";
            if ($rcCount > 0) echo "</kommentarTranslated>\n";
        }
    }
}

// Übersetzte Technische Hinweise
foreach ($query->rTranslations as $tr){
    if (isValid($tr)  && $tr != $query->rLanguage){
        $sql = new SqlAbfrage('WSR');
        $sql->sql = 'select kommentar from reportHints';
        $sql->addSel('reportID = ',$query->reportID);
        $sql->addSel('rLanguage = ',$tr);
        $sql->sql .= ' order by SortNr';
        $stmt = $sql->executeE(true);
        if ($stmt !== false){
            $rcCount = 0;
            while ($z = $stmt->fetchObject()){
                if ($rcCount == 0) echo '<hintsTranslated>'.$tr."\n";
                $rcCount ++;
                echo $z->kommentar;
            }
            echo "\n";
            if ($rcCount > 0) echo "</hintsTranslated>\n";
        }
    }
}


echo "</WS-Report>\n";

if (! isValid($query->rLanguage) || count($query->rTranslations) <= 0) return;
echo "\n<WS-Report Translations Version 1.0>\n";


error_log('exportiere Sprachen: '.$query->rLanguage.' -> '.print_r($query->rTranslations,true));

$reportID = $query->reportID; 

$sql = new SqlAbfrage('WSR');

$sql->sql = "select distinct q.type, q.text, t.*  from (";

$sql->sql .= "select 1 as sort, ".qTranslate::QT_LABEL." as type, prefix as text from reportJoinedTables";
$sql->addSel('reportID = ', $reportID);

$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_LABEL." as type, web_title as text from reportRows";
$sql->firstWhere = true;
$sql->addSel('reportID = ', $reportID);
$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_LABEL." as type, web_link_title as text from reportRows";
$sql->firstWhere = true;
$sql->addSel('reportID = ', $reportID);
$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_WEBLINK." as type, web_link_value as text from reportRows";
$sql->firstWhere = true;
$sql->addSel('reportID = ', $reportID);
$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_WEBLINK." as type, web_link as text from reportRows";
$sql->firstWhere = true;
$sql->addSel('reportID = ', $reportID);
$sql->addWhere('web_link like "#%"');

$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_LABEL." as type, label as text from reportValues";
$sql->firstWhere = true;
$sql->addSel('reportID = ', $reportID);

$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_LABEL." as type, column_label as text from reportColumns";
$sql->firstWhere = true;
$sql->addSel('reportID = ', $reportID);

$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_WEBLINK." as type, web_link as text from reportColumns";
$sql->firstWhere = true;
$sql->addSel('reportID = ', $reportID);
$sql->addWhere('web_link like "#%"');

$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_LABEL." as type, web_title as text from reportColumns";
$sql->firstWhere = true;
$sql->addSel('reportID = ', $reportID);


$sql->sql .="\n) q join queryTranslations t on t.".$query->rLanguage." = trim(q.text) "; // and q.type = t.qLanguageType";
$sql->sql .="\n and (q.type != ".qTranslate::QT_SQLTABLE.' or t.qLanguageType = '.qTranslate::QT_SQLTABLE.')';

$sql->sql .="\nwhere q.text is not null and q.text != ''";
$sql->sql .="\norder by q.text";

$stmt = $sql->execute();
if ($stmt === false){
    echo '<pre>';
    var_dump($sql);
    echo $sql->errorInfo()[2];
    die();
}
echo '<translateFrom>'.$query->rLanguage."\n";
while ($z = $stmt->fetch(PDO::FETCH_ASSOC)){
    echo '<'.$query->rLanguage.'>'.$z['qLanguageType']."\n";
    echo $z[$query->rLanguage]."\n";
    echo '</'.$query->qLanguage.">\n";
    foreach ($query->rTranslations as $t)
        if (isValid($t))
            if (isValid($z[$t])){
                echo '<'.$t.">\n";
                echo $z[$t]."\n";
                echo '</'.$t.">\n";
        }
        echo "<tReady>\n";
}


/* Die Übersetzung von reportTitle und Komentar ist hier überflüssig,
 * weil die schon beim Import in allen Sprachen importiert und dann 
 * aufgrund des Kennzeichens reportImported auch gespeichert werden.


if ($query->rTranslations != null && count($query->rTranslations) > 0){
    // Die Report-ID funktioniert hier nicht, weil sie sich nach dem Import ändert.
    echo '<report>'.$query->rLanguage."\n";
    echo '<reportTitle>'.$query->title."\n";
    foreach ($query->rTranslations as $tr){
        if (isValid($tr)  && $tr != $query->rLanguage){
            $sql = new SqlAbfrage('WSR');
            $sql->sql = 'select kommentar from reportComment';
            $sql->addSel('reportID = ',$query->reportID);
            $sql->addSel('rLanguage = ',$tr);
            $sql->sql .= ' order by SortNr';
            $stmt = $sql->executeE(true);
            if ($stmt !== false){
                $rcCount = 0;
                while ($z = $stmt->fetchObject()){
                    if ($rcCount == 0) echo '<kommentarTranslated>'.$tr."\n";
                    $rcCount ++;
                    echo $z->kommentar;
                }
                if ($rcCount > 0) echo "\n</kommentarTranslated>\n";
            }
        }
    }
    $sql = new SqlAbfrage('WSR');
    $sql->sql = 'select reportTitle_de,reportTitle_en, reportTitle_fr from reportTitle';
    $sql->addSel('reportID = ', $query->reportID);
    $stmt = $sql->executeE(true);
    if ($stmt !== false){
        if($z = $stmt->fetchObject()){
            if ($query->rLanguage != 'de') echo '<reportTitle_de>'.$z->reportTitle_de."\n";
            if ($query->rLanguage != 'en') echo '<reportTitle_en>'.$z->reportTitle_en."\n";
            if ($query->rLanguage != 'fr') echo '<reportTitle_fr>'.$z->reportTitle_fr."\n";
            echo "</reportTitle>\n";
        }
    }        
    echo "</report>\n";
}
    */



echo "\n</WS-Report Translations>\n";

