<?php 
/**
 * Projekt: WS-Reporter
 *          Übersetzungen zu den Datenquellen und/oder Reports importieren
 *
 * Autor:    Wilfried Schollenberger
 *           WS Unternehmensberatung und Controlling-Systeme GmbH
 * Kontakt:  wisch@ws-unternehmensberatung.de
 * Lizenz:   GPL 3.0
 *
 * Stand:    19.09.2022
 * 
 * Hinweise:
 * Übersetzungen können mit Datenquellen und Reports zusammen 
 * exportiert werden, müssen aber aus diesen dateien separat importiert werden,
 * weil die Behandlung bei bereits anders übersetzten Texten komplex ist.
 * 
 * Die Übersetzungen beginnen mit der Zeile
 *      <WS-Report Translations Version 1.0>
 * und enden mit
 *      </WS-Report Translations> 
 * als erstes muss die Sprache der Definition z.B.mit
 *      <translateFrom>'de;
 * geliefert werden.
 * 
 * Danach folgen die Übersetzungen, wobei jeder Eintrag mit
 *      <lang>nr, z.B. <de>2
 * eingeleitet und mit         
 *      <tReady>
 * beendet wird.
 * 
 * Report-Title und die Kommentare werden beim Import der jeweiligen Definition behandelt. 
 * Dazu gibt es die Optionen queryImported und reportImported
 * 
 */

include 'includes/defender.inc';
global $dbh;
require_once './phpsec/createDBH.inc';
require_once './includes/funktionen.inc';
require_once './includes/classes.inc';
if (session_status() == PHP_SESSION_NONE)  session_start();
anmeldungPruefen();
uebersetzerRechteVerlangen();


function showTextFrom($text,$c){
    global $tFrom;    
    $rows = 0;
    $t = explode("\n",$text);
    foreach ($t as $tz)
        if (strlen($tz) > 150) $rows += 4;
        elseif (strlen($tz) > 100) $rows += 3;
        elseif (strlen($tz) > 50) $rows += 2;
        else $rows++;
?> 
<textarea rows="<?=$rows?>" cols="50" lang="<?=$tFrom?>" id='<?='T'.$c.$tFrom?>' readonly >
<?=htmlspecialchars($text)?>
</textarea>        
<?php         
}

function showTranslation($text,$c,$lang,$type){
    $rows = 0;
    $t = explode("\n",$text);
    foreach ($t as $tz)
        if (strlen($tz) > 150) $rows += 4;
        elseif (strlen($tz) > 100) $rows += 3;
        elseif (strlen($tz) > 50) $rows += 2;
        else $rows++;
        ?>
<textarea rows="<?=$rows?>" cols="50" id='<?='T'.$c.$lang?>' 
     lang="<?=$lang ?>" onchange='enter(this,<?=$type?>)'      
     ><?= htmlspecialchars($text)?>
</textarea>
<?php         
}




$fehlermeldungen = '';
$tFrom = null;
$importe = array();


for ($fcount=0; $fcount < count($_FILES['importFile']['name']); $fcount ++){
    
    $importFileName = $_FILES['importFile']['name'][$fcount];
    $datei = fopen($_FILES['importFile']['tmp_name'][$fcount],'r');
    error_log ('Importiere '.$importFileName);
$start = 0;
$fertig = false;
$import = null;
$sprachen = array('de','en','fr');

while (($zeile = fgets($datei)) !== false && ! $fertig) {
    $zeile = trim($zeile);
//    if ($start > 0) error_log($start.': '.$zeile);
    if ($zeile == "<WS-Report Translations Version 1.0>") $start = 1;
    //    else if (strpos($zeile,"<class>") === 0) ; // nichts machen
    else if ($start > 0 && strpos($zeile, '<translateFrom>') === 0){
        if ($tFrom != null && $tFrom != substr($zeile, strlen("<translateFrom>"))){
            $fehlermeldungen .= '<br>FEHLER: Quellsprache '.substr($zeile, strlen("<translateFrom>"))
                .' aus Datei '.$importFileName
                .' passt nicht zur vorhergehenden Quellsprache '.$tFrom."\n";
                $fertig = true;           
        }else{
            if ($tFrom == null)
                $tFrom = substr($zeile, strlen("<translateFrom>"));
        if (isValid($tFrom)) $start = 2;
        error_log('Translate from '.$tFrom);
        }
    }
    else if ($start >1 && strpos($zeile,"</WS-Report Translations>") === 0) {
        $fertig = true;
        $start = 0;
    }
    else if ($start == 2
        && strpos($zeile,"<tReady>") === 0
        && $import != null){
            if (! array_key_exists($tFrom, $import)){
                echo (WSRTranslate::translate('FEHLER in Import-Eintrag: Die Sprache').' '.$tFrom
                    .' '.WSRTranslate::translate('fehlt. Der Import wird abgebrochen.').'<br><pre>');
                var_dump($import);
                die();
            }
            $importe[$import[$tFrom]] = $import;
            error_log('Importiere '.$import['de']);
            $import = null; 
//            $start = 1;
    }elseif ($start == 2){
        foreach ($sprachen as $sprache)
            if (strpos($zeile,'<'.$sprache.'>') === 0){
                if ($import == null) {
                    $import = array();
                    $import['qLanguageType'] = (int) substr($zeile, strlen('<de>'));
                }
            $import[$sprache] = '';
            while (($zeile = fgets($datei)) !== false 
                && strpos($zeile,'</'.$sprache.'>') === false ){
                    $import[$sprache] .= $zeile;
    //                error_log($zeile);
            }
            
            if (substr($import[$sprache],strlen($import[$sprache])-1) == "\n")
                $import[$sprache] = substr($import[$sprache],0,strlen($import[$sprache])-1);
            
        }
    }
}
}
// error_log(print_r($importe,true));

$selectStmt = array();
$updateStmt = array();
$insertStmt = array();

if (count($importe) < 1){
    echo 'Keine Übersetzungen gefunden.';
    die;
}

if (!isValid($tFrom)){
    echo 'Sprache der Definitionen (tFrom) fehlt.';
    die;                
}
foreach ($sprachen as $sprache){
    if (!isValid($sprache)) continue;
    $selectStmt[$sprache] = $dbh->prepare('select * from queryTranslations where '.$sprache.' = :text');
    if ($selectStmt[$sprache] === false){
        echo 'FEHLER bei create selectStmt für '.$sprache."\n".$dbh->errorInfo()[2];
        die();
    }
    if ($sprache != $tFrom){
        $updateStmt[$sprache] = $dbh->prepare('update queryTranslations set '.$sprache.' = :text where '.$tFrom.' = :tFrom');
        if ($updateStmt[$sprache] === false){
            echo 'FEHLER bei create updateStmt für '.$sprache."\n".$dbh->errorInfo()[2];
            echo '<br>update queryTranslations set '.$sprache.' = :text where '.$tFrom.' = :tFrom';
            die();
        }
        $insertStmt[$sprache] = $dbh->prepare('insert into queryTranslations set '.$sprache.' = :text, '.$tFrom.' = :tFrom'
            .', qLanguageType = :qLanguageType');
        if ($insertStmt[$sprache] === false){
            echo 'FEHLER bei create insertStmt für '.$sprache."\n".$dbh->errorInfo()[2];
            echo '<br>insert into queryTranslations set '.$sprache.' = :text, '.$tFrom.' = :tFrom'
            .', qLanguageType = :qLanguageType';
            die();
        }
    }
}

foreach ($importe as $key => $import){
    if (false === $selectStmt[$tFrom]->execute(array ('text' => $import[$tFrom]))){
        echo "<pre>Fehler beim Lesen von queryTranslation:\n";
        echo $selectStmt[$tFrom]->errorInfo()[2]."\nFür:\n";
        var_dump($selectStmt[$tFrom]->errorInfo());
        echo $import[$tFrom];
        echo "\n tFrom = ".$tFrom."\n";
        var_dump($importe);
        die();
    }
    $z = $selectStmt[$tFrom]->fetch(PDO::FETCH_ASSOC);
    if ($z === false){
       $importe[$key]['status'] = 'neu';
       foreach ($sprachen as $sprache)
       if ($sprache != $tFrom 
           && array_key_exists($sprache, $import)
           && isValid($import[$sprache])){
           $selectStmt[$sprache]->execute(array('text' => $import[$sprache]));
           if ($zs = $selectStmt[$sprache]->fetch(PDO::FETCH_ASSOC)){               
               $importe[$key]['conflict'.$sprache] = $zs[$tFrom];
           }
       }
    }else{
        $importe[$key]['status'] = 'alt';
        foreach ($sprachen as $sprache)
            if ($sprache != $tFrom
                && array_key_exists($sprache, $import)
                && isValid($import[$sprache])
                && $z[$sprache] != $import[$sprache]){
                    $selectStmt[$sprache]->execute(array('text' => $import[$sprache]));
                    if ($zs = $selectStmt[$sprache]->fetch(PDO::FETCH_ASSOC)){
                        $importe[$key]['conflict'.$sprache] = $zs[$tFrom];
                    }
                    if (isValid($z[$sprache])) $importe[$key]['differ'.$sprache] = $z[$sprache];
                    else $importe[$key]['new'.$sprache] = $z[$sprache];
        }
        
    }
}

?>
<!DOCTYPE html>
<html>
<head>
<meta charset="UTF-8">
<title><?=WSRTranslate::translate('Importierte Übersetzungen')?></title>
<meta name="GENERATOR" content="WS-PHP: importTranslation.php">
<meta name="viewport" content="width=device-width, initial-scale=1.0">
<link rel="stylesheet" href="./jscss/standard2.css">
<link rel="stylesheet" href="./jscss/tableFolder.css">
<script type="text/javascript" src="./jscss/translate.js">
	</script>
<script type="text/javascript" src="./jscss/tableHeader.js">
	</script>
<script type="text/javascript" src="./jscss/tableFolder.js">
	</script>
<script type="text/javascript">
 language = "<?=$tFrom?>";
	</script>
	
</head>
<?php 
if ($_SERVER["HTTP_HOST"] == "localhost:8898") echo '<body class="entwicklung">';
else echo '<body>';

if (count($_FILES['importFile']['name']) == 1){
    echo '<h1>'.WSRTranslate::translate('Übersetzungen aus Datei').' '.$importFileName.'</h1>';
}else{
    echo '<h1>'.WSRTranslate::translate('Übersetzungen aus Dateien').'</h1>';
    echo "<div ><b>".WSRTranslate::translateH('Quellen').':</b>';
    foreach ($_FILES['importFile']['name'] as $fname) echo '<br>'.$fname;
    echo '</div>';
}
echo $fehlermeldungen;
?>
<div id="Editor" class = "editorBox">
 <div class = "xButton" onclick="closeEditor()" title="schließen">&otimes;</div>
 <h3><?=WSRTranslate::translateH('Verwendungen des Textes')?></h3>
<pre id="VerwText"></pre>
<?=WSRTranslate::translateH('wird verwendet von:')?><br>
<div id = "VerwAnzeige">
</div>
<br>  <button onclick="closeEditor()">Schließen</button> 
</div>

<table id='Tabelle'>
<thead>
<tr><th class="WSFHeader">&nbsp;</th><th class='fts'>Text (<?=$tFrom ?>)</th>
<th>Sprache</th><th>Übersetzung</th><th>Status</th>
</thead>
<tbody>
<?php
$neueEinträge = 1;
$neueEinträgeMitFehler = 1;
$alteEinträge = 1;
$alteEinträgeMitFehler = 1;
$tc = 0;
?>

<tr><th class="WSFHeader" id='neueEinträgeMitFehler'>&nbsp;</th><th colspan='4'
><?= WSRTranslate::translate('Neue Einträge mit Konflikten')?></th></tr>
<?php
foreach ($importe as $import)
    if ($import['status'] == 'neu'){
    $rows = 0;
    foreach ($sprachen as $sprache)
        if ($sprache != $tFrom 
            && array_key_exists($sprache, $import)
            && isValid($import[$sprache])
            && array_key_exists('conflict'.$sprache, $import)) {
                $rows ++;
                $neueEinträgeMitFehler ++;
    }
    if ($rows > 0){
        $tc ++;
    
?>
<tr><td rowspan = "<?=$rows ?>">
<?=showTextFrom($import[$tFrom],$tc) ?>
</td>
<?php 
$alt = false;
    foreach ($sprachen as $sprache)
        if ($sprache != $tFrom 
            && array_key_exists($sprache, $import)
            && isValid($import[$sprache])
            && array_key_exists('conflict'.$sprache, $import)) {
                
        if ($alt) echo '</tr><tr>';
        else $alt = true;
        
?>
<th><?=$sprache ?></th><td><?=showTranslation($import[$sprache],$tc,$sprache,$import['qLanguageType']) ?>
<span class="gButton"  title='<?=WSRTranslate::translate('Verwendungen anzeigen')?>' onclick="showVarsById('<?='T'.$tc.$sprache?>')">V</span>
<span class="gButton"  title='<?=WSRTranslate::translate('Übersetzungen anzeigen')?>' onclick="showTranslationById('<?='T'.$tc.$sprache?>')">T</span>
</td><td id='<?='R'.$tc.$sprache ?>'>
<?php     
    echo WSRTranslate::translate('Verwendet in Übersetzung von').':<br>'.quoteForHTML($import['conflict'.$sprache]);
    echo '</td>';

    }
?>
</tr>
<?php
} // if Rows
} // Schleife
?>

<tr><th class="WSFHeader" id='alteEinträgeMitFehler' >&nbsp;</th><th colspan='5'>Alte Einträge mit Problemen</th></tr>
<?php
foreach ($importe as $import)
    if ($import['status'] == 'alt'){
    $rows = 0;
    foreach ($sprachen as $sprache)
        if ($sprache != $tFrom 
            && array_key_exists($sprache, $import)
            && isValid($import[$sprache])
            && ( array_key_exists('conflict'.$sprache, $import)
                 || array_key_exists('differ'.$sprache, $import)
                )            
            ) {
                $rows ++;
                $alteEinträgeMitFehler ++;
    }
//    error_log($rows.$import[$tFrom]);
    if ($rows > 0){
        $tc++;
?>
<tr><td rowspan = "<?=$rows ?>"><?=showTextFrom($import[$tFrom],$tc) ?>
<span class="gButton"  title='<?=WSRTranslate::translate('Verwendungen anzeigen')?>' onclick="showVarsById('<?='T'.$tc.$tFrom?>')">V</span>
<span class="gButton"  title='<?=WSRTranslate::translate('Übersetzungen anzeigen')?>' onclick="showTranslationById('<?='T'.$tc.$tFrom?>')">T</span>
</td>
<?php 
$alt = false;
    foreach ($sprachen as $sprache)
        if ($sprache != $tFrom 
            && array_key_exists($sprache, $import)
            && isValid($import[$sprache])
            && ( array_key_exists('conflict'.$sprache, $import)
                || array_key_exists('differ'.$sprache, $import)
                )
            ){
                ?>
<?php     
    if ($alt) echo '</tr><tr>';
    else $alt = true;
?>    
<th><?=$sprache?></th><td><?=showTranslation($import[$sprache],$tc,$sprache,$import['qLanguageType']) ?>
<span class="gButton"  title='<?=WSRTranslate::translate('Verwendungen anzeigen')?>' onclick="showVarsById('<?='T'.$tc.$sprache?>')">V</span>
<span class="gButton"  title='<?=WSRTranslate::translate('Übersetzungen anzeigen')?>' onclick="showTranslationById('<?='T'.$tc.$sprache?>')">T</span>
</td><td id='<?='R'.$tc.$sprache ?>'>
<?php 
if (array_key_exists('conflict'.$sprache, $import)){
    echo WSRTranslate::translate('Verwendet in Übersetzung von').':<br>'.quoteForHTML($import['conflict'.$sprache]);
}
    

if (array_key_exists('differ'.$sprache, $import)){
    if (array_key_exists('conflict'.$sprache, $import)) echo '<br>';
    echo WSRTranslate::translate('Abweichende Übersetzung - bisher').':<br>'.quoteForHTML($import['differ'.$sprache]);
    echo '<br><button onclick="ignoreNewImport(\'T'.$tc.$sprache.'\')">'
        .WSRTranslate::translate('Import ignorieren').'</button>';
    echo '  <button onclick="enterNewImport(\'T'.$tc.$sprache.'\','.$import['qLanguageType'].')">'
     .WSRTranslate::translate('Import eintragen').'</button>';
}
echo '</td>';

}
    
?>
</tr>
<?php
    }
} // Schleife
?>





<tr><th class="WSFHeader" id='neueEinträge'>&nbsp;</th><th colspan='5'><?=WSRTranslate::translate('Neue Einträge')?></th></tr>
<?php
foreach ($importe as $import)
    if ($import['status'] == 'neu'){
    $rows = 0;
    foreach ($sprachen as $sprache)
        if ($sprache != $tFrom 
            && array_key_exists($sprache, $import)
            && isValid($import[$sprache])
            && ! array_key_exists('conflict'.$sprache, $import)) {
                $rows ++;
                $neueEinträge ++;
    }
    if ($rows > 0){
?>
<tr><td rowspan = "<?=$rows ?>"><?=quoteForHTML($import[$tFrom]) ?>
</td>
<?php 
$alt = false;
    foreach ($sprachen as $sprache)
        if ($sprache != $tFrom 
            && array_key_exists($sprache, $import)
            && isValid($import[$sprache])
            && ! array_key_exists('conflict'.$sprache, $import)){
        if ($alt) echo '</tr><tr>';
        else $alt = true;
?>
<th><?=$sprache ?></th><td><?=quoteForHTML($import[$sprache]) ?></td><td>
<?php     
    if ($import['status'] == 'neu'){
        if (false === $insertStmt[$sprache]->execute(array('qLanguageType' => $import['qLanguageType'],
            'tFrom' => $import[$tFrom], 'text' => $import[$sprache] ))){
            echo WSRTranslate::translate('FEHLER beim Eintragen').'<br>'.$insertStmt[$sprache]->errorInfo()[2];
        }else{
            echo WSRTranslate::translate('eingetragen');
            $import['status'] = 'eingetragen';
        }
    }else{
        if (false === $updateStmt[$sprache]->execute(array(
            'tFrom' => $import[$tFrom], 'text' => $import[$sprache] ))){
            echo WSRTranslate::translate('FEHLER beim Eintragen').'<br>'.$updateStmt[$sprache]->errorInfo()[2];
        }else{
            echo WSRTranslate::translate('eingetragen');
        }
    }
    echo '</td><td>&nbsp;</td>';

    }
?>
</tr>
<?php

    } // if Rows
}// Schleife
?>
<tr><th class="WSFHeader" id='alteEinträge' >&nbsp;</th><th colspan='5'>Alte Einträge ergänzt</th></tr>
<?php
foreach ($importe as $import)
    if ($import['status'] == 'alt'){
    $rows = 0;
    foreach ($sprachen as $sprache)
        if ($sprache != $tFrom 
            && array_key_exists($sprache, $import)
            && isValid($import[$sprache])
            &&  array_key_exists('new'.$sprache, $import)
            && ! ( array_key_exists('conflict'.$sprache, $import)
                || array_key_exists('differ'.$sprache, $import)
                )
            
            ) {
                $rows ++;
                $alteEinträge ++;
    }
//    error_log($rows.$import[$tFrom]);
    if ($rows > 0){
?>
<tr><td rowspan = "<?=$rows ?>"><?=quoteForHTML($import[$tFrom]) ?></td>
<?php 
$alt = false;
    foreach ($sprachen as $sprache)
        if ($sprache != $tFrom 
            && array_key_exists($sprache, $import)
            && isValid($import[$sprache])
            &&  array_key_exists('new'.$sprache, $import)
            && ! ( array_key_exists('conflict'.$sprache, $import)
                || array_key_exists('differ'.$sprache, $import)
                )
            ){
                ?>
<?php     

if (array_key_exists('new'.$sprache, $import)){
    if ($alt) echo '</tr><tr>';
    else $alt = true;
    echo '<th>'.$sprache.'</th><td>'.quoteForHTML($import[$sprache]).'</td><td>';
    if (false === $updateStmt[$sprache]->execute(array(
        'tFrom' => $import[$tFrom], 'text' => $import[$sprache] ))){
        echo WSRTranslate::translate('FEHLER beim Eintragen').'<br>'.$updateStmt[$sprache]->errorInfo()[2];
    }else{
        echo WSRTranslate::translate('eingetragen');
    }
    echo '</td>';
}
}
    
?>
</tr>
<?php
    }
}
?>


</tbody>
</table>
<pre><?php
// var_dump($importe)?></pre>
 <script>
    window.addEventListener('load', function() {
<?php    
        echo 'document.getElementById("neueEinträge").rowSpan='.$neueEinträge.";\n";
        echo 'document.getElementById("neueEinträgeMitFehler").rowSpan='.$neueEinträgeMitFehler.";\n";
        echo 'document.getElementById("alteEinträge").rowSpan='.$alteEinträge.";\n";
        echo 'document.getElementById("alteEinträgeMitFehler").rowSpan='.$alteEinträgeMitFehler.";\n";
        ?>    
    var tf = new TableFolder('Tabelle',TableFolder.NachKlassen,TableFolder.Ausgeklappt,true);
    }
   ); 
   </script>
<?php    

?>

</body>
</html>
