<?php
/**
 * Projekt: WS-Reporter
 *          Eingaben bei Query-Spezifikation testen 
 *
 * Autor:    Wilfried Schollenberger
 *           WS Unternehmensberatung und Controlling-Systeme GmbH
 * Kontakt:  wisch@ws-unternehmensberatung.de
 * Lizenz:   GPL 3.0
 *
 * Stand:    05.06.2022
 *
 */


function testSelect($select){
    global $query;
    if (!isValid($select)) return true;
    $sql = new SqlAbfrage('DWH');
    $sql->sql = 'select '.$select.' as V from '.$query->phys_table;
    if ($query->queryType == Query::DIMENSION) $sql->sql .= ' dim';
    else{
        $sql->sql .= ' base';
        foreach ($query->dimensions as $dimnr => $dim){
            if (false) $dim = new Dimension('');
            $sql->sql .= "\n join ".$dim->phys_table.' d'.$dimnr;
        }
    }
    return $sql->test();
}

function testWhere($where){
    global $query;
    if (!isValid($where)) return true;
    $sql = new SqlAbfrage('DWH');
    $sql->sql = 'select * from '.$query->phys_table;
    if ($query->queryType == Query::DIMENSION) $sql->sql .= ' dim';
    else{
        $sql->sql .= ' base';
        foreach ($query->dimensions as $dimnr => $dim){
            if (false) $dim = new Dimension('');
            $sql->sql .= "\n join ".$dim->phys_table.' d'.$dimnr;
        }
    }
    $sql->addWhere($where);
    return $sql->test();
}

function testFehler($wo){
    global $meldungen, $sql;
    if (strlen($meldungen) > 0){
        http_response_code(425);
        echo $wo."\n".
            str_replace('<b>',"",str_replace('</b>',"",str_replace('<br>',"\n",str_replace('</br>',"\n", $meldungen))))."\n";
            echo $sql->sql;
            return false;
    }
    return true;
}

function testCreate($name, $sql){
    global $dbDWH;
    //    require_once './phpsec/createDBH.inc';    
    if ($dbDWH == null) setdbDWH();
    $dbDWH->exec('drop temporary table if exists WSR_'.$name);
    $sql->sql = 'create temporary table WSR_'.$name." as \n".trim($sql->sql);
    $stmt = $sql->execute();
    if ($stmt === false){
        return false;        
    }
    return true;    
}


function testJoinSelect($name, $addQuery){
    global $query, $testMode, $sql, $meldungen;
    require_once './includes/createReport.inc';
    $runningQuery = $query;
    $query = $query->cloneSimpleQuery();
    $query->newReport();
    $join = new JoinedTable($addQuery);
    $join->joinType = JoinedTable::JOINWERTE;
    $k = $query->getKennzeichen();
    $join->kennzeichen = $k;
    $query->repTables->put($join);
    $rv = new RepValue($k.'='.$name);
    $query->repValues->put($rv);
    
    createHistory();
    $meldungen='';
    $sql = new SqlAbfrage('DWH');
    createBasicQuery($sql,'base', -1, -1);
    if(!testFehler("Erstellen der SQL für Basic Query")) return false;
    if(!testCreate('base', $sql)) return false;
    
    $sql = new SqlAbfrage('DWH');
    createBasicQuery($sql,$k,-1, -1);
    if(!testFehler("Erstellen der SQL für $k ")) return false;
    if(!testCreate($k, $sql)) return false;
        
    $sql = new SqlAbfrage('DWH');
    createLeadTable($sql);
    if(!testFehler("Erstellen der SQL für Zeilen und Spalten")) return false;
    if(!testCreate('lead', $sql)) return false;
        
    $sql = new SqlAbfrage('DWH');
    createJoinedTable($sql);
    if(!testFehler("Erstellen der SQL für die Ergebnis-Tabelle")) return false;
    if(!testCreate('ergebnis', $sql)) return false;
 error_log($sql->sql);
    $query = $runningQuery;
    return true;    
    
}