<?php 
/**
 * Projekt: WS-Reporter
 *          Anzeige der Variablen einer Dimension zum Übernehmen
 *          in die Datenquelle
 *
 * Autor:    Wilfried Schollenberger
 *           WS Unternehmensberatung und Controlling-Systeme GmbH
 * Kontakt:  wisch@ws-unternehmensberatung.de
 * Lizenz:   GPL 3.0
 *
 * Stand:    22.02.2023 MS
 *
 */
global $query;
require_once './includes/classes.inc';
require_once './includes/funktionen.inc';
require_once './includes/wsrTranslate.inc';

$dim = $query->dimensions[$query->selectedDimension];
if (false) $dim = new Dimension('N');
$sql = new SqlAbfrage('WSR');
$sql->sql = 'select column_name, SpaltenTyp, SortNr, qFilterTyp, column_label, column_desc from queryColumnMetadata';
$sql->addSel('table_name = ', $dim->dimName);
$sql->sql .= ' order by SpaltenTyp, SortNr';
$stmt = $sql->execute();
$spaltenTyp = 0;
$st = array(1=>'Zeilen',2=>'Werte',3=>'Filter');
?>
<div class="xButton" onclick="closeDimVars()" title="<?=WSRTranslate::translate('schließen')?>">&otimes;</div>
<h3><?=WSRTranslate::translateH('Auswahl aus Dimension')?> <?=$dim->dimNr ?>: 
<?=$dim->dimName?></h3>
<table>
<tr><th><?=WSRTranslate::translate('Name')?></th><th><?=WSRTranslate::translate('Label')?></th>
<th><?=WSRTranslate::translate('Filter-Typ')?></th><th><?=WSRTranslate::translate('eintragen')?></th></tr>
<?php
while ($z = $stmt->fetchObject()){
    if ($spaltenTyp != $z->SpaltenTyp){
        $spaltenTyp = $z->SpaltenTyp;
?>
<tr><th class="left" colspan="4"><?=$st[$spaltenTyp]?>        
<?php
    }
?>
<tr><th class="left"><?=$z->column_name ?></th>
<td <?=makeTitle($z->column_desc) ?>><?=quoteForHTML($z->column_label)?></td>
<td><?php if ($z->qFilterTyp > 0) echo MetaRecord::getKlartextFilter($z->qFilterTyp); else echo '&nbsp;'?></td>
<td class="center"><button onclick="selectDimensionVar(<?=$z->SpaltenTyp ?>,<?=$z->SortNr ?>)"
	><?=WSRTranslate::translate('eintragen')?></button> </td>
</tr>
<?php
}
?>
	 </table>	 
<?php 
$sql = new SqlAbfrage('WSR');
$sql->sql = 'select SatzTyp, attribute from queryAttribute';
$sql->addSel('table_name = ', $dim->dimName);
$sql->addWhere('SatzTyp = 11');
$stmt = $sql->execute();
if (($z = $stmt->fetchObject()) && $z->attribute > 0) {
    require_once './includes/loadClassDef.inc';
    $dimQuery = new Query($dim->dimName);
    loadSimpleClassDef($dimQuery);
?> 
<h3><?=WSRTranslate::translateH('Historisierung')?></h3>
<table><thead>
<tr><th>&nbsp;</th><th><?=WSRTranslate::translateH('aktuell')?></th>
<th><?=WSRTranslate::translateH('in Quelle der Dimension')?></th></tr>
</thead>
<tbody>
<tr>
<th class="left"><?=WSRTranslate::translateH('Historisierung')?></th>
<td><?= Query::getHistText($dim->histType) ?></td>
<td><?= Query::getHistText($dimQuery->histType) ?></td>
</tr>
<tr>
<th class="left"><?=WSRTranslate::translateH('Ermittlung Stichtag')?></th>
<td><?= HistoryOptions::getKlartext($dim->historyOption) ?></td>
<td><?= HistoryOptions::getKlartext($dimQuery->historyOption) ?></td>
</tr>
<?php 
if ($dim->histType == Query::VONBIS 
    || $dimQuery->histType == Query::VONBIS) {?>
<tr>
<th class="left"><?=WSRTranslate::translateH('Select für gültig ab')?></th>
<td><?= $dim->histAbSelect ?></td>
<td><?php if (isValid($dimQuery->histAb))
    echo $dimQuery->getReportMetaRecord($dimQuery->histAb)->getSelectVar(); ?></td>
</tr>
<tr>
<th class="left"><?=WSRTranslate::translateH('Select für gültig bis')?></th>
<td><?= $dim->histBisSelect ?></td>
<td><?php if (isValid($dimQuery->histBis))
    echo $dimQuery->getReportMetaRecord($dimQuery->histBis)->getSelectVar(); ?></td>
</tr>

<?php } ?>
<?php 
if ($dim->histType == Query::STICHTAG 
    || $dimQuery->histType == Query::STICHTAG
    || $dim->histType == Query::STICHTAGE
    || $dimQuery->histType == Query::STICHTAGE) {?>
<tr>
<th class="left"><?=WSRTranslate::translateH('Select für Stichtag')?></th>
<td><?= $dim->histStichtagSelect ?></td>
<td><?php if (isValid($dimQuery->histStichtag))
    echo $dimQuery->getReportMetaRecord($dimQuery->histStichtag)->getSelectVar(); ?></td>
</tr>

<?php } ?>
</tbody>
</table>
	<br> <div class="center">
<button onclick="updateDimHist(<?=$dim->dimNr ?>)"><?=WSRTranslate::translate('aus der Datenquelle aktualisieren')?></button>	 
</div>
<?php     
}else if ($dim->histType > 0){
?>	 
<div class="center">
<button onclick="updateDimHist(<?=$dim->dimNr ?>)"><?=WSRTranslate::translate('Historisierung der Dimension löschen')?></button>	 
</div>	 
<?php 
}
?>	 
	<br> <div class="center">
<button onclick="closeDimVars()"><?=WSRTranslate::translate('schließen')?></button>	 
</div><br><br>
