<?php
/**
 * Projekt: WS-Reporter
 *          Neue Datenquelle gemäß physischer Quelle initialisieren
 *
 * Autor:    Wilfried Schollenberger
 *           WS Unternehmensberatung und Controlling-Systeme GmbH
 * Kontakt:  wisch@ws-unternehmensberatung.de
 * Lizenz:   GPL 3.0
 *
 * Stand:    7.10.2021
 *           17.01.2023 (Umstellung auf SHOW FULL COLUMNS)
 *
 */
require_once './phpsec/createDBH.inc';
require_once './includes/classes.inc';
global $query, $dbDWH;
if ($dbDWH == null) setdbDWH();


function newMeta($name){
    global $query, $mData, $defaultGueltigAb, $defaultGueltigBis, $defaultStichtag;
    $mr = new MetaRecord();
    error_log(print_r($mData,true));
    $mr->init($query->class_table, $name);
    $mr->description = $mData->Comment."\n".$mData->Type;
    $query->metaData[$name] = $mr;
    return $mr;
}


// if (false === ($stmt = $dbDWH->query("select * from ".$query->phys_table.' LIMIT 1'))){
if (false === ($stmt = $dbDWH->query("show full columns from ".$query->phys_table))){
    http_response_code(425);
    echo "Kann Abfrage für DB-Tabelle ".$query->phys_table." nicht erstellen\n";
    var_dump($dbDWH->errorInfo());
    die();
}
/*
if (false === ($res = $stmt->fetch(PDO::FETCH_NUM))){
    http_response_code(425);
    echo "Fehler beim SELECT auf DB-Tabelle ".$query->phys_table."\n";
    var_dump($dbDWH->errorInfo());
    die();
}
if (count($res) < 1){
    http_response_code(425);
    echo "Fehler beim SELECT auf DB-Tabelle ".$query->phys_table.": Keine Spalten gefunden\n";
    var_dump($dbDWH->errorInfo());
    die();
}
*/
// http_response_code(201);
// for ($i = 0; $i < count($res); $i++){
while ($mData = $stmt->fetchObject()){
    // var_dump($stmt->getColumnMeta($i));
//    $metaData = $stmt->getColumnMeta($i);
//    $name = $metaData['name'];
    $name = $mData->Field;
    if ($query->queryType == Query::DIMENSION) $selectName = 'dim.'.$name;
    else $selectName = 'base.'.$name;
//    echo $name.' '.$metaData['native_type']."\n";

    $columnType = explode(' ', str_replace('(', ' ',str_replace(')', ' ', $mData->Type)));
    error_log(print_r($columnType,true));
    switch ($columnType[0]){

        case 'char':
        case 'varchar':
        case 'text':
        case 'tinytext':
        case 'longtext':
        case 'mediumtext':
        case 'enum':
        case 'set':
            $mr = newMeta($name);
            $mr->type = MetaRecord::STRING;
            $mr->filterType = MetaRecord::TEXT;
            $mr->align = 'left';
            $mr->selectVar = $selectName;
            if (isValid($mData->Comment)) $mr->label = $mData->Comment;
            if ($columnType[0] == 'enum') $mr->label = $mData->Type;
            if ($columnType[0] == 'set') $mr->label = $mData->Type;
            $query->rowVars[]= $name;
            break;
        case 'int':
        case 'mediumint':
        case 'smallint':
        case 'tinyint':
            $mr = newMeta($name);
            $mr->type = MetaRecord::INT;
            $mr->filterType = MetaRecord::TEXT;
            $mr->align = 'center';
            $mr->selectVar = $selectName;
            if (isValid($mData->Comment)) $mr->label = $mData->Comment;
            $query->rowVars[]= $name;
        case 'bigint':
        case 'decimal':
        case 'double':
        case 'float':
            $mr = newMeta($name.'_S');
            if (isValid($mData->Comment)) $mr->label = $mData->Comment.' &sum;';
            else $mr->label = $name.' &sum;';
            if ($columnType[0] == 'int'
                || strpos($columnType[0], 'int') > 0){
                    $type = MetaRecord::INT;
                    $precision = 0;
            }else{
                $type = MetaRecord::DECIMAL;
                $precision = (int) explode(',',$columnType[1])[1];
            }
                
                $mr->type = $type;
                $mr->filterType = MetaRecord::NONE;
                $mr->align = 'right';
                $mr->htmlData = 'td';
                $mr->decimals = $precision;
                $mr->selectVar = 'sum('.$name.')';
                $query->valueVars[] = $name.'_S';
                
                $mr = newMeta($name.'_M');
                if (isValid($mData->Comment)) $mr->label = $mData->Comment.' &Oslash;';
                else $mr->label = $name.' &Oslash;';
                $mr->type = MetaRecord::DECIMAL;
                $mr->filterType = MetaRecord::NONE;
                $mr->align = 'right';
                $mr->htmlData = 'td';
                $mr->decimals = $precision;
                $mr->selectVar = 'avg('.$name.')';
                $query->valueVars[] = $name.'_M';
                
                
                $mr = newMeta($name.'_MAX');
                if (isValid($mData->Comment)) $mr->label = $mData->Comment.' &uArr;';
                else $mr->label = $name.' &uArr;';
                $mr->type = $type;
                $mr->filterType = MetaRecord::NONE;
                $mr->align = 'right';
                $mr->htmlData = 'td';
                $mr->decimals = $precision;
                $mr->selectVar = 'max('.$name.')';
                $query->valueVars[] = $name.'_MAX';
                
                $mr = newMeta($name.'_MIN');
                if (isValid($mData->Comment)) $mr->label = $mData->Comment.' &dArr;';
                else $mr->label = $name.' &dArr;';
                $mr->type = $type;
                $mr->filterType = MetaRecord::NONE;
                $mr->align = 'right';
                $mr->htmlData = 'td';
                $mr->decimals = $precision;
                $mr->selectVar = 'min('.$name.')';
                $query->valueVars[] = $name.'_MIN';
                break;
        
        case 'date':
            $mr = newMeta($name);
            if ($name == $defaultGueltigAb){
                $mr->type = MetaRecord::HISTAB;
                $mr->filterType = MetaRecord::NONE;                            
                $mr->label = 'Gültig ab für Historisierung';
                $query->histType = Query::VONBIS;
            }else
                if ($name == $defaultGueltigBis){
                    $mr->type = MetaRecord::HISTBIS;
                    $mr->filterType = MetaRecord::NONE;
                    $mr->label = 'Gültig bis für Historisierung';
            }else
                if ($name == $defaultGueltigStichtag){
                    $mr->type = MetaRecord::HISTSTICHTAG;
                    $mr->filterType = MetaRecord::NONE;
                    $mr->label = 'Stichtag für Historisierung';
                    $query->histType = Query::STICHTAG;
            }else{
                $mr->type = MetaRecord::DATE;
                $mr->filterType = MetaRecord::TEXTEXAKT;
                if (isValid($mData->Comment)) $mr->label = $mData->Comment;                
            }
            $mr->align = 'center';
            $mr->selectVar = $selectName;
            $query->rowVars[]= $name;
                break;
        /*        
        case 'binary':
        case 'blob':
        case 'longblob':
        case 'mediumblob':
        */
        case 'datetime':
        case 'timestamp':
            $mr = newMeta($name);
            if (isValid($mData->Comment)) $mr->label = $mData->Comment;
            else $mr->label = $name;
            $mr->type = MetaRecord::DATETIME;
            $mr->align = 'center';
            $mr->selectVar = $selectName;
            $query->rowVars[]= $name;
            break;
        case 'time':
            $mr = newMeta($name);
            $mr->type = MetaRecord::TIME;
            if (isValid($mData->Comment)) $mr->label = $mData->Comment;
            else $mr->label = $name;
            $mr->align = 'center';
            $mr->selectVar = $selectName;
            $query->rowVars[]= $name;
            break;
                
        default:    $mr = newMeta($name);
                    $mr->label = '? '.$mData->Type.' ';
                    if (isValid($mData->Comment)) $mr->label .= $mData->Comment;
                    else $mr->label .= $name;
                    
                    $mr->type = MetaRecord::STRING;
                    $mr->filterType = MetaRecord::NONE;
                    $mr->htmlData = 'td';
                    $mr->align = 'right';
                    $mr->selectVar = $selectName;
                    $query->rowVars[] = $name;
                    break;
        
    }


  

}

return;









http_response_code(425);
echo "Unbekannter Fehler beim Initialisieren der Datenquelle";
die();