<?php
$wsReportVersion = "0.142";  
/**
 * Projekt: WS-Reporter
 *          Sammlung von Funktionen
 *
 * Autor:    Wilfried Schollenberger
 *           WS Unternehmensberatung und Controlling-Systeme GmbH
 * Kontakt:  wisch@ws-unternehmensberatung.de
 * Lizenz:   GPL 3.0
 *
 * Stand:    6.3.2023
 * 
 * Fortsetzung Mehrsprachigkeit
 *
 */

if (file_exists('./phpsec/createDBH.inc')){
    require_once './includes/wsrTranslate.inc';
}

function anmeldungPruefen(){
    global $user, $rechte;
    if (array_key_exists('Token', $_SESSION)){
        $token = $_SESSION['Token'];
        if (array_key_exists('WS-Report-Token', $_COOKIE)){
            if ($token != $_COOKIE['WS-Report-Token']){
                http_response_code(425);
                echo "FEHLER: Das Anmelde-Token passt nicht.<br>Die Anwendung wird gesperrt.";
                die();
            }
        }
    }
    if (array_key_exists('User', $_SESSION)){
        $user = $_SESSION['User'];
        if ($_SESSION['globalAdmin']){
            $rechte = "admin";
        }else{
            $rechte = "keine";
            if (array_key_exists('WSReportQuery', $_SESSION) && array_key_exists('Rechte', $_SESSION)){
                $query = $_SESSION['WSReportQuery'];
                $dqrechte = $_SESSION['Rechte'];
                if ($query != null && $dqrechte != null 
                    && array_key_exists($query->class_table, $dqrechte)){
                    $dqrechte = $dqrechte[$query->class_table];
                    if ($dqrechte != null){
                        if ($dqrechte['admin'])  $rechte = "dqAdmin";
                        else $rechte = "lesen";                        
                    }
                }
            }
        }
        if (array_key_exists('uLanguage', $_REQUEST) && isValid($_REQUEST['uLanguage'])) {
            $_SESSION['Sprache'] = $_REQUEST['uLanguage'];
            session_write_close();
            session_start();
        }
        error_log('Anmeldung prüfen Rechte='.$rechte.' Sprache='.$_SESSION['Sprache']);    
        return true;
    }
    http_response_code(425);
    echo "Anmeldung fehlt oder ist abgelaufen.<br>";
    echo '<a href="./index.php">zur Anmeldung</a>';
    die();
}

function adminRechteVerlangen(){
    global $rechte;
    if (! $_SESSION['globalAdmin']){
        http_response_code(425);
        echo "Für diese Operation sind \ndie globalen Admininistrator-Rechte erforderlich";
        die();
    }
}

function uebersetzerRechteVerlangen(){
    global $rechte;
    if (! $_SESSION['globalAdmin'] && ! $_SESSION['uebersetzer']){
        http_response_code(425);
        echo "Für diese Operation sind \ndie Übersetzer-Rechte erforderlich";
        die();
    }
}


function dqAdminRechte(){
    global $query, $entwicklung;
    if (array_key_exists('globalAdmin',$_SESSION) && $_SESSION['globalAdmin']) {
        return true;
    }
    if (array_key_exists("Rechte", $_SESSION)
        && $_SESSION['Rechte'] != NULL
        && array_key_exists($query->class_table, $_SESSION['Rechte'])
        && $_SESSION['Rechte'][$query->class_table]['admin']
        ) {
            return true;   
    }
    if (isset($entwicklung)) return $entwicklung;
    return false;
}

function leseRechteVerlangen(){
    global $query, $dbh;
    if (array_key_exists('globalAdmin',$_SESSION) && $_SESSION['globalAdmin']) {
        return true;
    }
    if (array_key_exists("Rechte", $_SESSION)
        && $_SESSION['Rechte'] != NULL
        && array_key_exists($query->class_table, $_SESSION['Rechte'])
        ){
            return true;
        }        
        // Prüfen ob Lese-Recht ohne Anmeldung
        if (!array_key_exists('User', $_SESSION) && $query != null){
            $stmt = $dbh->query('select * from wsrRechte where user = "" and table_name = "'.$query->class_table.'"');
            if( $stmt->fetch()) return true;
        }
        if ($query == null){
            error_log('FEHLER: Query ist NULL');
//            error_log($ex->getTraceAsString());            
        }
     http_response_code(404);
     echo "Für diese Datenquelle sind Sie nicht berechtigt.";
     die();
    return false;
}

function rechtAnwendenI($filter,$von,$bis){
    global $query;
    if (!isValid($filter)) return;
    $filter = trim($filter);
    $mr = $query->getReportMetaRecord($filter);    
    if ($mr == null) {
//        error_log($query->class_table.': kein Filter >'.$filter.'<');
        return;
    }
    if (false) $mr = new MetaRecord();
    if ($mr->filterType == MetaRecord::NONE) return;
    $filter = new RepFilter($filter);
    $filter->fs_von = $von;
    $filter->fs_bis = $bis;
    $query->repFilter->put($filter);
}
 

function rechteAnwenden(){
    global $query;
    if (array_key_exists('globalAdmin', $_SESSION) && $_SESSION['globalAdmin']) return;
    if (!array_key_exists('Rechte', $_SESSION)){
        echo 'FEHLER: Anmeldeinformationen sind ungültig';
        die();
    }
    $rechte = null;
    if (array_key_exists('RechteG',$_SESSION)) $rechte = $_SESSION['RechteG'];    
    if ($rechte != null && array_key_exists('filter', $rechte) && isValid($rechte['filter'])) 
        rechtAnwendenI($rechte['filter'], $rechte['filterVon'], $rechte['filterBis']);
    if ($rechte != null && array_key_exists('filter2', $rechte) && isValid($rechte['filter2'])) 
        rechtAnwendenI($rechte['filter2'], $rechte['filter2Von'], $rechte['filter2Bis']);
    $rechte = null;
    if (array_key_exists($query->class_table,$_SESSION['Rechte'])) 
        $rechte = $_SESSION['Rechte'][$query->class_table];
    if ($rechte != null && array_key_exists('filter', $rechte) && isValid($rechte['filter'])) 
        rechtAnwendenI($rechte['filter'], $rechte['filterVon'], $rechte['filterBis']);
    if ($rechte != null && array_key_exists('filter2', $rechte) && isValid($rechte['filter2'])) 
        rechtAnwendenI($rechte['filter2'], $rechte['filter2Von'], $rechte['filter2Bis']);
}

/**
 * Vor dem Speichern und exportieren werden Filter, die über
 * die Benutzer-Rechte gesetzt wurden wieder zurück gesetzt.
 */
function rechteZuruecksetzen(){
    global $query;
    if ($_SESSION['globalAdmin']) return;
    $rechte = $_SESSION['RechteG'];
    if (array_key_exists('filter', $rechte) && isValid($rechte['filter'])){
        $filter = trim($rechte['filter']);
        if ($query->repFilter->has($filter)){
            $f = $query->repFilter->get($filter);
            error_log("Entferne Filter $filter mit {$f->fs_von} - {$f->fs_bis}" );
            $f->fs_von = null;
            $f->fs_bis = null;
        }
    }
    if (array_key_exists('filter2', $rechte) && isValid($rechte['filter2'])){
        $filter = trim($rechte['filter2']);
        if ($query->repFilter->has($filter)){
            $f = $query->repFilter->get($filter);
            error_log("Entferne Filter $filter mit {$f->fs_von} - {$f->fs_bis}" );
            $f->fs_von = null;
            $f->fs_bis = null;
        }
    }
    $rechte = $_SESSION['Rechte'][$query->class_table];
    if (array_key_exists('filter', $rechte) && isValid($rechte['filter'])){
        $filter = trim($rechte['filter']);
        if ($query->repFilter->has($filter)){
            $f = $query->repFilter->get($filter);
            error_log("Entferne Filter $filter mit {$f->fs_von} - {$f->fs_bis}" );
            $f->fs_von = null;
            $f->fs_bis = null;
        }
    }
    if (array_key_exists('filter2', $rechte) && isValid($rechte['filter2'])){
        $filter = trim($rechte['filter2']);
        if ($query->repFilter->has($filter)){
            $f = $query->repFilter->get($filter);
            error_log("Entferne Filter $filter mit {$f->fs_von} - {$f->fs_bis}" );
            $f->fs_von = null;
            $f->fs_bis = null;
        }
    }
}



function rechteFilter(){
    global $query;
    $liste = array();
    if ($_SESSION['globalAdmin']) return $liste;
    $rechte = $_SESSION['RechteG'];
    if (array_key_exists('filter', $rechte) && isValid($rechte['filter'])){
        if (isValid($rechte['filterVon']) || isValid($rechte['filterBis'])){
            $liste[] = $rechte['filter'];
        }else{
            $mr = $query->getReportMetaRecord($rechte['filter']);
            if ($mr->filterType == MetaRecord::SONDERFILTER) $liste[] = $rechte['filter'];
        }
    }
    if (array_key_exists('filter2', $rechte) && isValid($rechte['filter2'])){
        if (isValid($rechte['filter2Von']) || isValid($rechte['filter2Bis'])){
            $liste[] = $rechte['filter2'];
        }else{
            $mr = $query->getReportMetaRecord($rechte['filter']);
            if ($mr->filterType == MetaRecord::SONDERFILTER) $liste[] = $rechte['filter'];
        }
    }
    $rechte = $_SESSION['Rechte'][$query->class_table];
    if (array_key_exists('filter', $rechte) && isValid($rechte['filter'])){
        if (isValid($rechte['filterVon']) || isValid($rechte['filterBis'])){
            $liste[] = $rechte['filter'];
        }else{
            $mr = $query->getReportMetaRecord($rechte['filter']);
            if ($mr != null && $mr->filterType == MetaRecord::SONDERFILTER) $liste[] = $rechte['filter'];
        }
    }
    if (array_key_exists('filter2', $rechte) && isValid($rechte['filter2'])){
        if (isValid($rechte['filter2Von']) || isValid($rechte['filter2Bis'])){
            $liste[] = $rechte['filter2'];
        }else{
            $mr = $query->getReportMetaRecord($rechte['filter']);
            if ($mr->filterType == MetaRecord::SONDERFILTER) $liste[] = $rechte['filter'];
        }
    }
    return $liste;
}

function isValid($s){
    if ($s == null) return false;
    if (strlen(trim($s)) <= 0) return false;
    return true;
}

/**
 * Verwandelt $val in einen DB-Bool-Wert mit default 0;
 * @param mixed $val
 * @return boolean
 */
function bool($val){
    if ($val) return 1;
    return 0;
}

function dragNDdrop($wo,$pos,$dropAction){
    $act = <<<EOS
draggable="true" 
ondragstart="dragStart(event,{$wo},{$pos})"
ondragover="dragOver(event,{$wo},this)"
ondrop="{$dropAction}(event,{$wo},{$pos},this)"

ondragenter="dragEnter(this)"
ondragleave="dragLeave(this)"
EOS;
    echo $act;
}


function drag($wo,$pos,$dropAction){
    $act = <<<EOS
class = "draggable"
draggable="true"
ondragstart="dragStart(event,{$wo},{$pos})"
EOS;
    echo $act;
}

function drop($wo,$pos,$dropAction){
    $act = <<<EOS
ondragover="dragOver(event,{$wo},this)"
ondrop="{$dropAction}(event,{$wo},{$pos},this)"
ondragenter="dragEnter(this)"
ondragleave="dragLeave(this)"
EOS;
    echo $act;
}


function makeTitle($s){
    if (isValid($s)) return ' title="'.str_replace('"','&quot;', $s).'" ';
    return '';
}
function makeTitleClass($s,$c){
    if (isValid($s)) return ' class="'.$c.' help" title="'.str_replace('"','&quot;', $s).'" ';
    return ' class="'.$c.'" ';
}
/**
 * Packt descriptions und hints für Pop-Up-Title zusammen
 * @param MetaRecord $mr
 * @return string
 */

/**
 *
 * @param MetaRecord $mr
 */
function makeDescription($mr) {
    if ($mr === null) return WSRTranslate::translate('FEHLER bei makeDescription: MetaRecord ist NULL');
    $t = '';
    if (isValid($mr->description)) $t .= quoteForHTML($mr->description);
    if (isValid($mr->hints)){
        if (isValid($t)) $t.="<br>\n";
        $t .= "---<br>\n";
        $t .= quoteForHTML($mr->hints);
    }
    return $t;
}

/**
 *
 * @param MetaRecord $mr
 */
function makeTitleDescription($mr) {
    if ($mr === null) return WSRTranslate::translate('FEHLER bei makeTitleDescription: MetaRecord ist NULL');
    $t = '';
    if (isValid($mr->description)) $t .= str_replace('"','&quot;', $mr->description);
    if (isValid($mr->hints)){
        if (isValid($t)) $t.="\n";
        $t .= "---\n";
        $t .= str_replace('"','&quot;', $mr->hints);
    }
    return $t;
}

/**
 *
 * @param MetaRecord $mr
 */
function makeFilterTitleDescription($mr) {
    if ($mr === null) return WSRTranslate::translate('FEHLER bei makeTitleDescription: MetaRecord ist NULL');
    $t = '';
    if ($mr->filterType == MetaRecord::TEXTLIKE)
        $t = WSRTranslate::translate('Textmuster mit % und _ als Platzhalter')."\n".
        WSRTranslate::translate('Ausschluss mit ! als erstem Zeichen')."\n";
    elseif ($mr->filterType == MetaRecord::TEXT)
        $t = WSRTranslate::translate('Suche nach Text im Wert')."\n".
        WSRTranslate::translate('Ausschluss mit ! als erstem Zeichen')."\n";
    elseif ($mr->filterType == MetaRecord::TEXTEXAKT)
        $t = WSRTranslate::translate('Suche nach diesem Wert Wert')."\n".
        WSRTranslate::translate('Ausschluss mit ! als erstem Zeichen')."\n";
    $t .= makeTitleDescription($mr);    
    return $t;
}

/**
 *
 * @param MetaRecord $mr
 */
function makeOutputFilterTitleDescription($mr) {
    if ($mr === null) return WSRTranslate::translate('FEHLER bei makeTitleDescription: MetaRecord ist NULL');
    $t = '';
    if ($mr->filterType == MetaRecord::TEXTLIKE)
        $t = WSRTranslate::translate('Textmuster mit % und _ als Platzhalter')."\n".
        WSRTranslate::translate('Ausschluss mit ! als erstem Zeichen')."\n";
        elseif ($mr->filterType == MetaRecord::TEXT)
        $t = WSRTranslate::translate('Suche nach Text im Wert')."\n".
        WSRTranslate::translate('Ausschluss mit ! als erstem Zeichen')."\n";
        elseif ($mr->filterType == MetaRecord::TEXTEXAKT)
        $t = WSRTranslate::translate('Suche nach diesem Wert Wert')."\n".
        WSRTranslate::translate('Ausschluss mit ! als erstem Zeichen')."\n";
        if (isValid($mr->description)) $t .= str_replace('"','&quot;', $mr->description);
        return $t;
}


function quoteForHTML($s){
    if (isValid($s)) return 
                str_replace("\r\n\r\n",'<br>',
                    str_replace("\n\n",'<br>',
                        str_replace("\r\n\r\n\r\n",'<br><br>',   
                            str_replace("\n\n\n",'<br><br>',
                                str_replace("<s",'&lt;s',
                                    str_replace("<S",'&lt;S',
                                        str_replace("<?",'&lt;?',
                                             $s)))))));
//                                            str_replace('"', '&quot;', $s))))))));
    return '';    
}

function quoteForText($s){
    if (isValid($s)) 
        return str_replace('"', '&quot;', trim($s));
//        return str_replace('<','&lt;',str_replace('"', '&quot;', $s));
   return '';
}

function wsrDecode($text){
    $text = html_entity_decode($text);
    $text = trim(str_replace("\r", '',$text));
    return $text;
}

function formatWert($mr,$wert){
    global $query;
    if (false) $mr = new MetaRecord();    
    if (!isValid($wert)) return '&nbsp;';
    if ($mr->hasDispVar()) return $wert;
    switch ($mr->type){
        case MetaRecord::INT:
//            error_log($wert);
            return number_format($wert,0,'','');
        case MetaRecord::GELD:
            $wert *= $query->repGeldFaktor;
        case MetaRecord::DECIMAL: 
            return WSRFormats::formatDecimal($wert,$mr->decimals);
        case MetaRecord::DATE:
            $d = DateTime::createFromFormat('Y-m-d',$wert);
            if ($d === false) return '?'.$wert;
            return WSRFormats::formatDate($d);
        case MetaRecord::DATETIME: 
            $d = DateTime::createFromFormat('Y-m-d H:i:s',$wert);
            if ($d === false) return '?'.$wert;
            return WSRFormats::formatDateTime($d);
        case MetaRecord::DATETIMES:
            $d = DateTime::createFromFormat('Y-m-d H:i:s',$wert);
            if ($d === false) return '?'.$wert;
            return WSRFormats::formatDateTimeS($d);
            // vor Translation:  case MetaRecord::TIME: return $wert; //DateTime::createFromFormat('Y-m-d H:i:s',$wert)->format('H:i:s'); ?? Zeit in STd./min statt Uhrzeit ??
        case MetaRecord::TIME: 
            $d = DateTime::createFromFormat('H:i:s',$wert);
            if ($d === false) return '?'.$wert;
            return WSRFormats::formatTime($d);
        case MetaRecord::TIMES:
            $d = DateTime::createFromFormat('H:i:s',$wert);
            if ($d === false) return '?'.$wert;
            return WSRFormats::formatTimeS($d);
    }
    return $wert;
}

function insertReportComment($reportID,$language,$comment){
    insertReportCommentTable('reportComment',$reportID,$language,$comment);
}
function insertReportHints($reportID,$language,$comment){
    insertReportCommentTable('reportHints',$reportID,$language,$comment);
}

function insertReportCommentTable($table,$reportID,$language,$comment){
    global $dbh;
    $sql = new SqlAbfrage('WSR');
    $sql->sql = 'delete from '.$table;
    $sql->addSel('reportID = ',$reportID);
    $sql->addSel('rLanguage = ',$language);
    
    if ($sql->execute() === false){
        http_response_code(425);
        echo "\nFehler bei Delete Kommentar";
        echo "\n".$sql->errorInfo()[2];
        die();
    }
    if (isValid($comment)){
        $sortNr = 0;
        $stmt = $dbh->prepare('insert into '.$table.' set reportID = :reportID,'
            .' SortNr = :SortNr, rLanguage = :rLanguage, kommentar = :kommentar');
        if ($stmt === false){
            http_response_code(425);
            echo "\nFehler bei prepare Insert Kommentar\n";
            echo "\n".$dbh->errorInfo()[2];
            error_log("Fehler bei prepare von Insert Kommentar\n");
            error_log($dbh->errorInfo()[2]);
            die();
        }
        
        if (strlen($comment) < 250){
            $pars = array(
                'reportID' => $reportID,
                'SortNr' => $sortNr,
                'rLanguage' => $language,
                'kommentar' => str_replace("\r", '', $comment)
            );
            if (false === $stmt->execute($pars)){
                http_response_code(425);
                echo "\nFehler bei execute von Insert Kommentar\n";
                echo "\n".$stmt->errorInfo()[2];
                error_log("Fehler bei execute von Insert Kommentar\n");
                error_log($stmt->errorInfo()[2]);
                error_log(print_r($pars,true));
                die();
            }
        }else{
            $k = explode("\n",str_replace("\r", '', $comment));
            foreach ($k as $kom) {
                $sortNr ++;
                while (strlen($kom) > 248){
                    $pars = array(
                        'reportID' => $reportID,
                        'SortNr' => $sortNr,
                        'rLanguage' => $language,
                        'kommentar' => substr($kom,0,248)
                    );
                    $kom = substr($kom,248);
                    if (false === $stmt->execute($pars)){
                        http_response_code(425);
                        echo "\nFehler bei execute von Insert Kommentar\n";
                        echo "\n".$stmt->errorInfo()[2];
                        error_log("Fehler bei execute von Insert Kommentar\n");
                        error_log($stmt->errorInfo()[2]);
                        error_log(print_r($pars,true));
                        die();
                    }
                    $sortNr ++;
                }
                $pars = array(
                    'reportID' => $reportID,
                    'SortNr' => $sortNr,
                    'rLanguage' => $language,
                    'kommentar' => $kom."\n"
                );
                if (false === $stmt->execute($pars)){
                    http_response_code(425);
                    echo "\nFehler bei execute von Insert Kommentar\n";
                    echo "\n".$stmt->errorInfo()[2];
                    error_log("Fehler bei execute von Insert Kommentar\n");
                    error_log($stmt->errorInfo()[2]);
                    error_log(print_r($pars,true));
                    die();
                }
            }
        }
    }
    
}



function showFilter($filter,$mitLabel){
    global $query;
    if ($filter == null) return null;
    $erg = '';
    if (array_key_exists('spreadSheet', $_REQUEST)) $erg='<b>';
    if ($filter->isWindowFilter()) {
        $mr = $filter->getMetaRecord();
//        $mitLabel = true;
    }else {
        $mr = $query->getReportMetaRecord($filter->getName());
    }
    
    if ($mr == null) return "Fehler (showFilter): Filter für {$filter->getName()} nicht gefunden.";
    if ($mitLabel){
        $n = $filter->getName();
        if (substr($n,1,1) == ':'){
            $join = $query->repTables->get(substr($n,0,1));
            if ($join != null) $erg.= $join->prefix.' ';
        }
        $erg .= '<b>'.$mr->label.'</b>';
        if ($mr->filterType == MetaRecord::SONDERFILTER) return $erg;
        
        // WINDOW-Filter mit Label
        else if ($filter->isWindowFilter()){
            if (false) $filter = new WindowFilter('');
            $wert = $filter->getWert();
            $mrw = $query->getReportMetaRecord($wert);
            if ($mrw != null){
                if (substr($wert,1,1) == ':' || substr($wert,1,1) == '='){
                    $join = $query->repTables->get(substr($wert,0,1));
                    if ($join != null) $erg.= ': aus '.$join->prefix.' '.$mrw->label;
                }
                else $erg .= ': aus '.$mrw->label;
            }
            if (! $filter->getDesc()) $erg .= ' (aufsteigend)';
            if ($filter->getType() < 10){
                $sel_von = $filter->fs_von;
                if (isValid($sel_von) && substr($sel_von,0,1) == '<')
                    $sel_von = '< '.$mrw->formatValue(substr($sel_von, 1));
                    else $sel_von = $mrw->formatValue($sel_von);
                $erg .= ' '.$sel_von;
            }
            else $erg .= ' '.$filter->fs_von;
            return $erg;
        }
        else $erg.=': ';
//        if (array_key_exists('spreadSheet', $_REQUEST)) $erg.="\t";
    }
    else if ($mr->filterType == MetaRecord::SONDERFILTER) return $mr->label;
    
    // WINDOW-Filter ohne Label
    else if ($filter->isWindowFilter()) {
        if (false) $filter = new WindowFilter('');
        $wert = $filter->getWert();
        $mrw = $query->getReportMetaRecord($wert);
        if ($mrw != null){
            if (substr($wert,1,1) == ':' || substr($wert,1,1) == '='){
                $join = $query->repTables->get(substr($wert,0,1));
                if ($join != null) $erg = 'aus '.$join->prefix.' '.$mrw->label;
            }
            else $erg = 'aus '.$mrw->label;
        }
        if (! $filter->getDesc()) $erg .= ' (aufsteigend)';
        if ($filter->getType() < 10){
            $sel_von = $filter->fs_von;
            if (isValid($sel_von) && substr($sel_von,0,1) == '<')
                $sel_von = '< '.$mrw->formatValue(substr($sel_von, 1));
                else $sel_von = $mrw->formatValue($sel_von);
             $erg .= ' '.$sel_von;
        }
            else $erg .= ' '.$filter->fs_von;
        return $erg;
    }
    
    if ($mr->filterType == MetaRecord::LIST
        && $filter->fs_list != NULL
        && count($filter->fs_list) >0
        ) {
            if ($filter->ausschlussListe){
                if ($mitLabel) return $erg.'mit Ausschlussliste';
                else return 'Ausschlussliste für '.$mr->label;                
            }else{
                if ($mitLabel) return $erg.'nur Auswahl';
                else return 'Auswahl aus '.$mr->label;                
            }
    }
    
    if (!$filter->isSet()) return null;
    
    switch ($mr->filterType){
        case MetaRecord::VON_BIS:
            if ($filter->fs_von == $filter->fs_bis) $erg .= $filter->fs_von;
            else{
                if (isValid($filter->fs_von)) $erg .= 'von/ab '.$filter->fs_von.' ';
                if (isValid($filter->fs_bis)) $erg .= 'bis '.$filter->fs_bis;
            }
            break;
            
        case MetaRecord::VON_BIS_SELECT:
            if ($filter->fs_von == $filter->fs_bis) $erg .= $mr->findSelectOption($filter->fs_von);
            else{
                if (isValid($filter->fs_von)) $erg .= 'von/ab '.$mr->findSelectOption($filter->fs_von).' ';
                if (isValid($filter->fs_bis)) $erg .= 'bis '.$mr->findSelectOption($filter->fs_bis);
            }
            break;
            
        case MetaRecord::HISTDEL:
            if (!isValid($filter->fs_von)){
                $filter->fs_bis = null;
                return null;
            }
            $opts = $mr->getSelectOptions();
            foreach ($opts as $o){
                if ($o[0] == $filter->fs_von) $erg .= $o[1];
            }
            break;
        case MetaRecord::SELECT:
            if (!isValid($filter->fs_von)){
                $filter->fs_bis = null;
                return null;
            }
            if (isValid($filter->fs_von)) $ergz = $mr->findSelectOption($filter->fs_von);
            if (!isValid($ergz)) $ergz = $filter->fs_von;
            $erg .= $ergz;
            break;
            
        case MetaRecord::SONDERFILTERSEL:
            if (!isValid($filter->fs_von)){
                $filter->fs_bis = null;
                return null;
            }
            if (isValid($filter->fs_von)) $erg .= $filter->fs_von;
            break;
            
        case MetaRecord::TEXTMITLABEL:
            if (!isValid($filter->fs_von)){
                $filter->fs_bis = null;
                return null;
            }
            if (isValid($filter->fs_von)) $ergz = $mr->findSelectLabel($filter->fs_von);
            if (!isValid($ergz)) $ergz = $filter->fs_von;
            $erg .= $ergz;
            break;
            
            
        default:
            if (!isValid($filter->fs_von)){
                $filter->fs_bis = null;
                return null;
            }
            if (isValid($filter->fs_von)) $erg .= $filter->fs_von;
            break;
    }
    return trim($erg);
}


function iconUp() {
    return '<img src="./icons/go-up.png" alias="auf" title="nach oben verschieben" class="icon"  />';
}
function iconDown() {
    return '<img src="./icons/go-down.png" alias="ab" title="nach unten verschieben" class="icon"  />';
}
function iconDelete() {
    return '<img src="./icons/delete.png" alias="X" title="Zeile löschen" class="icon"  />';
}
function iconSelect() {
    return '<img src="./icons/object-select.png" alias="Sel." title="auswählen" class="icon"  />';
}

function iconOutput() {
    return '<img src="./icons/output_win.svg" alias="Output" title="Ausgabe" class="icon"  />';
}


function testQueryTable($query){
    if (!isset($_REQUEST['table'])) return true;
    if ($query == null){
        http_response_code(425);
        echo "Query ist null\nSession abgelaufen?";
        die();
    }
    if (0 == strcmp($_REQUEST['table'],$query->class_table)) return true;
    http_response_code(425);
    echo "Der Aufruf ist nur für die aktive Tabelle $query->class_table möglich\n";
    echo "Der Editor für ".$_REQUEST['table']." ist nicht mehr aktiv.";
    die();
}

function checkSel($s1,$s2){
    if (0==strcmp($s1,$s2)) return " selected ";
    return "";
}

function checkSelN($s1,$s2){
    if ($s1 == $s2) return " selected ";
    return "";
}

function checked($s){
    if (isset($s) && $s) return " checked ";
    return "";
}


function blankAsNull($s){
    if (isValid($s)) return $s;
    return '';
}

function anleitung($level,$part){
    return '&nbsp;&nbsp;<span class="anleitung" 
              title="Anleitung" onclick="anleitung(\''.$level.'\',\''.$part.'\')">Anl.</span>';
}
