<?php
/**
 * Projekt: WS-Reporter
 *          Report ausgeben
 *
 * Autor:    Wilfried Schollenberger
 *           WS Unternehmensberatung und Controlling-Systeme GmbH
 * Kontakt:  wisch@ws-unternehmensberatung.de
 * Lizenz:   GPL 3.0
 *
 * Stand:    04.03.2023
 *
 */
include 'includes/defender.inc';
// Ausgabe des Reports
global $query, $dbDWH, $meldungen, $daten,  $filterVector, $spreadsheet, $entwicklung, 
    $moves, $aktiverEditor, $mysql, $dbDWH;
$aufrufZeit = hrtime(true);
require_once './phpsec/createDBH.inc';
require_once './includes/funktionen.inc';
require_once './includes/classes.inc';
require_once './includes/createReport.inc';

// Anmeldung über Report-Link für Demo und Präsentationen
if (array_key_exists('user', $_REQUEST)){
    include './includes/anmelden.inc';
    error_log('Neue Anmeldung '.$_REQUEST['user'].' ergibt '.$_SESSION['User']);
}
error_log('Output Report '.print_r($_REQUEST,true));


/**
 * Zuerst DragANDDrop-Aufruf für Report-Editor verarbeiten
 */
if (session_status() == PHP_SESSION_NONE)  session_start();

if (!array_key_exists('Sprache', $_SESSION)) $_SESSION['Sprache'] = 'de';


if (array_key_exists('set', $_REQUEST)
    && $_REQUEST['set'] = 'ActionPos'
    && array_key_exists('action', $_REQUEST)
    && $_REQUEST['action'] = 'moveTo'
    && array_key_exists('id', $_REQUEST)
    && array_key_exists('table', $_REQUEST)
    && array_key_exists('tab', $_REQUEST)
    && array_key_exists('WSReportQuery', $_SESSION)
    ){
        $query = $_SESSION['WSReportQuery'];
        if ($query->object_id != $_REQUEST['id']
            || $query->class_table != $_REQUEST['table']){
            http_response_code(425);
            echo WSRTranslate::translate("Zu dieser Reportausgabe\ngibt es keinen aktiven Report-Editor");
            die();
        }
        
        switch ($_REQUEST['tab']){
            case 'Zeilen':
                $vVector = $query->repRows;
                break;
            case 'Spalten':
                $vVector = $query->repColumns;
                break;
            case 'Werte':
                $vVector = $query->repValues;
                break;
            default:                
                http_response_code(425);
                echo "Ungültiger Parameter tab=".$_REQUEST['tab'];
                die();
        }
        
        $vonPos = $_REQUEST['pos'];
        $nachPos = $_REQUEST['nachPos'];
        $vVector->movePos($vonPos,$nachPos);
        session_write_close();
        return;
}

//   echo'<pre>';var_dump($_REQUEST);echo'</pre>';
$spreadsheet = array_key_exists('spreadSheet', $_REQUEST); //Ausgabe für Tabellenkalkulation
$aktiverEditor = false;
$moves = new Moves();
// error_log(print_r($_REQUEST,true));

if (array_key_exists('action', $_REQUEST)
    && array_key_exists('id', $_REQUEST)
    && array_key_exists('table', $_REQUEST)
//    && session_status() == PHP_SESSION_NONE
    && array_key_exists('WSReportQuery', $_SESSION)
    ){
        $query = $_SESSION['WSReportQuery'];
        if ($query->object_id == $_REQUEST['id']
            || $query->class_table == $_REQUEST['table']){
            error_log('Aktiver Editor');
            $aktiverEditor = true;
        }else{
            error_log('PRÜFEN: Editor nicht mehr aktiv?');            
        }
}

function setChart($name,$mr,$id,$wert,$chartGroup) {
    global $chartValues,$query;
    if ($wert == null) $wert = 0;
//    $chartGroup = $mr->chartGroup;
    if (!isValid($chartGroup)) $chartGroup = $name;
    if (!array_key_exists($chartGroup,$chartValues))
        $chartValues[$chartGroup] = new ChartData($chartGroup, $mr->chartColor, $mr->chartNegColor);
//        error_log( $mr->getName().' Pos:'.$mr->chartColor.' Neg:'.$mr->chartNegColor.' Wert:'.$wert);
        if (isValid($mr->chartMin)){
            $grenze = str_replace(',', '.',str_replace('.', '', $mr->chartMin));
            if ($wert < $grenze) $wert = $grenze;
        }
        if (isValid($mr->chartMax)){
            $grenze = str_replace(',', '.',str_replace('.', '', $mr->chartMax));
            if ($wert > $grenze) $wert = $grenze;
        }
        $chartValues[$chartGroup]->add($id,$wert);        
}

/**
 * Behandlung der Aufruf-Filter:
 * Wer meint das Filter-Handling sei hier kompliziert irrt:
 * Es ist SEHR kompliziert.
 * 
 * Die in der Session gespeicherte Query mit der eingelesenen Report-
 * Definition (WSReportQuery) bleibt immer unverändert. Die zusätzlichen
 * Filter werden jedesmal vor dem Aufruf von createReport() neu eingetragen.
 * Lediglich die Sort-Parameter werden von createReportQuery direkt ausgewertet.
 * 
 * Bei Eingabemasken wird eine Kopie des Query-Objekts mit allen Filter-
 * Einträgen in WSReportErfassung gespeichert, damit die Eingaben
 * korrekt eingetragen werden.
 * 
 * Zusätzliche Filter werden im Aufruf auf zwei Wegen übergeben:
 * * addFilter-Parameter mit der Anzahl der Filter in addFilter
 *   und den Filternamen in filter0 bis filterN kommen aus dem Link,
 *   bzw. der Form, über die outputReport.php aufgerufen wurde.
 * * setFilter n den Parametern setFilter1 und setFilter2 werden
 *   mit dem Parameter setFilterNames=X angekündigt und referenzieren
 *   auf die Position des Filters in $query-repFilter.
 *   
 * Dabei überschreiben setFilter eventuell gesetzte addFilter.    
 *   
 * Unabhängig von der Art der Übergabe werden diese Filter im filterVector
 * eingetragen. Bei der Option reportCopyFilter werden diese Filter auch
 * in den Link-Aufrufen der Tabelle und Report-Verweisen 
 * und in den Form-Einträgen als addFilter übernommen.
 * 
 * Für die Form-Einträge (zum Sortieren, den Report-Editor, die 
 * Spreadsheet-Ausgabe und die einstellbaren Filter) reicht das.
 * 
 * Für die Links in der Tabelle müssen zusätzlich die
 * die Filter aus repFilter stehen, die das reportCopyAttribut haben.
 * Sie dürfen aber nur übernommen werden, wenn der Filter nicht auch im
 * filterVector steht.
 * 
 * Bei den Links in der Tabelle kommen dann aber auch noch die Werte
 * aus der Zeile und ggf. aus der Spalte hinzu.
 * 
 * Damit ergeben sich folgende Optionen:
 * 
 * * Übergabemethode: POST oder GET
 *   ergibt die Funktionen makeGetFilter() und makePostFilter()
 *   
 * Nur makeGetFilter benötigt die Optionen:  
 * * Nur addFilter, setFilter und reportCopyFilter aus repFilter.
 *   Letztere sind unschädlich, wenn sie nicht benötigt werden.  
 * * Nur Filter aus der Zeile wenn reportCopyFilter nicht gesetzt ist.
 * * Nur Filter aus der Spalte bei wenn reportCopyFilter und reportLinkInValues nicht gesetzt sind.
 * * Filter aus der Zeile und der Spalte bei wenn reportCopyFilter nicht gesetzt ist
 *   und reportLinkInValues gesetzt ist.
 * * Die letzten drei Optionen mit den Filter aus filterVector   
 *      
 * Daraus ergeben sich die Optionen:
 * * takeFilterVector: filter aus filterVector weil reportCopyFilter gesetzt ist (TRUE / FALSE)
 * * rowValues: Werte aus der Report-Zeile übernehmen: ($zeile mit den Daten oder NULL)
 * * column: RepColumn-Eintrag oder NULL. Der Eintrag enthält die Spalten-Variable und ggf. 
 *   den Zusatzfilter     
 * * columnValue: SpaltenWert, d.h. der Wert der Spalte, auf den geklickt wurde.     
 * 
 * @return string
 */


/**
 * Diese Funktion erzeugt HIDDEN INPUT-Tags für addFilter
 * Es werden nur gesetzte Filter ausgegeben.
 * @return NULL
 */
function makePostFilter(){
    global $filterVector, $query;
    $afc = 0;
    if ($query->repWerteAuswahl != null) hi('Werte',$query->repWerteAuswahl);
    if ($filterVector->size() < 1) return;
    for ($i=0; $i < $filterVector->size(); $i++){
        $filter = $filterVector->getPos($i);
        if (false) $filter = new RepFilter("Name");
        if ($filter->isSet()){
            hi('filter'.$afc,$filter->getName());
            hi('filterValue'.$afc.'V1',$filter->fs_von);
            hi('filterValue'.$afc.'V2',$filter->fs_bis);
            $afc ++;
        }
    }
    hi('addFilter',$afc);
}

function hi($name,$value){
    if (isValid($value))
        echo '<input type="hidden" name="'.$name.'" value="'.$value.'"/>';
        else echo '<input type="hidden" name="'.$name.'" value=""/>';
}


function makeGetFilter($takeFilterVector,$values,$rvName,$column,$columnValue){
    global $filterVector, $query;
    $afc = 0;
    $addFilter = '';
//    error_log('makeGetFilter tv='.$takeFilterVector.' rvN='.$rvName.' col='.$column.' cv='.$columnValue);
    if ($takeFilterVector){
        for ($i=0; $i < $filterVector->size(); $i++){
            $filter = $filterVector->getPos($i);
            if ($filter != null)
            if ($filter->isSet()){
                $addFilter.='&filter'.$afc.'='.urlencode($filter->getName());
                if (isValid($filter->fs_von))
                    $addFilter.='&filterValue'.$afc.'V1='.urlencode($filter->fs_von);
                else
                    $addFilter.='&filterValue'.$afc.'V1=';
                if (isValid($filter->fs_bis))
                    $addFilter.='&filterValue'.$afc.'V2='.urlencode($filter->fs_bis);
                else 
                    $addFilter.='&filterValue'.$afc.'V2=';                
                $afc ++;
            }
        }
        for ($i=0; $i < $query->repFilter->size(); $i++){
            $filter = $query->repFilter->getPos($i);
            if ($filter->reportCopyFilter
                && $filter->isSet()
                && ! $filterVector->has($filter->getName())){
                $addFilter.='&filter'.$afc.'='.urlencode($filter->getName());
                if (isValid($filter->fs_von))
                    $addFilter.='&filterValue'.$afc.'V1='.urlencode($filter->fs_von);
                else
                    $addFilter.='&filterValue'.$afc.'V1=';
                if (isValid($filter->fs_bis))
                    $addFilter.='&filterValue'.$afc.'V2='.urlencode($filter->fs_bis);
                else
                    $addFilter.='&filterValue'.$afc.'V2=';
                $afc ++;
            }
        }
    }
    if ($values != null){
        for ($i=0; $i < $query->repRows->size(); $i++){
            $vnr = $query->repRows->getPos($i)->getName();
            if (!$query->repRows->getPos($i)->webLinkColumn){
                $mrr = $query->getReportMetaRecord($vnr);
                $vd = $mrr->getSelName();
                if (substr($vnr,1,1) == ':' || substr($vnr,1,1) == '=' ) $vd = substr($vnr,0,1).'__'.$vd;
                if ($values[$vd] != null){
                    $addFilter .='&filter'.$afc.'='.urlencode($vnr)
                            .'&filterValue'.$afc.'V1='.urlencode($values[$vd])
                            .'&filterValue'.$afc.'V2='.urlencode($values[$vd]);
                    $afc ++;
                }
            }
    //            error_log("Teste ".$vnr.' == '.$rvName.' ergibt '.($vnr == $rvName));
            if ($vnr == $rvName) $i = $query->repRows->size()+1;
        }
    }
    
    if ($column != null){
        $addFilter.='&filter'.$afc.'='.$column->getName();
        $addFilter.='&filterValue'.$afc.'V1='.urlencode($columnValue);
        $addFilter.='&filterValue'.$afc.'V2='.urlencode($columnValue);        
        $afc++;
        if (isValid($column->filter)){
            $addFilter.='&filter'.$afc.'='.$column->filter;
            if (isValid($column->fs_von))
                $addFilter.='&filterValue'.$afc.'V1='.urlencode($column->fs_von);
                else
                    $addFilter.='&filterValue'.$afc.'V1=';
            if (isValid($column->fs_bis))
                $addFilter.='&filterValue'.$afc.'V2='.urlencode($column->fs_bis);
                else
                    $addFilter.='&filterValue'.$afc.'V2=';
            $afc++;
        }
    }
    $addFilter.='&addFilter='.$afc;
    if ($query->repWerteAuswahl != null) $addFilter.= '&Werte='.urlencode($query->repWerteAuswahl);
    return $addFilter;
}

/**
 * HTML-Zelle TH oder TD für Zeilen-Label erzeugen. 
 * @param array $zeile
 * @param String $rv
 * @param int $rowspan
 * @return string
 */
function createRowCell($zeile, $rv, $rowspan){
    global $query, $daten, $spreadsheet;
    $hasLink = false;
    if ($rv->webLinkColumn){
        if ($spreadsheet){
            $cell = '';
        }else{
            $class = 'center';
            if ($rv->fixedTableColumns || $query->fix_labelColumns){
                $class = 'fts center';
            }
            if ($rowspan > 2) $class .=' top';
            
            $cell = '<td class="'.$class.'" rowspan="'.$rowspan.'" ';
            if ($spreadsheet) $cell .= ' valign="top"';
            $cell.='>';
        }
    }else{
        $mr = $query->getReportMetaRecord($rv->getName());
        $class = $mr->align;
        if ($rv->fixedTableColumns || $query->fix_labelColumns){
            $class = 'fts '.$mr->align;
        }
        if ($rowspan > 2) $class .=' top';
        
        if ($rowspan > 1){
            $wpos = $query->repRows->findPos($rv->getName())+1;
            while ($wpos < $query->repRows->size() && $query->repRows->getPos($wpos)->webLinkColumn) $wpos++;
            if ($query->repRows->size() > $wpos &&
                $query->repRows->getPos($wpos)->summenZeilen) $class.=' WSFSummen';
            else
                if ($query->repRows->size() == $wpos &&
                    $query->values_in_columns) $class.=' '.$query->valueColumnFolderMode;
        }
        
        $cell = '<'.$mr->htmlData.' class="'.$class.'" rowspan="'.$rowspan.'" ';
        if ($spreadsheet) $cell .= ' valign="top"';
        $cell.='>';
        
    }
    
    if (isValid($rv->web_link) && ! $spreadsheet){
        if (substr($rv->web_link,0,1) == '#'){
            $mrl = $query->getReportMetaRecord(trim(substr($rv->web_link,1)));
            if ($mrl != null){
                if (false) $mrl = new MetaRecord();
                $link = $zeile[$mrl->getSelName()];
                if (isValid($link)){
                    $hasLink = true;
                    $web_title = "";
                    if (isValid($rv->web_title)) $web_title = quoteForText($rv->web_title);
                    else if (isValid($mrl->displayVar))  $web_title = quoteForText($zeile[$mrl->getDispName()]);
                        else
                          if (isValid($mrl->description))  $web_title = quoteForText($mrl->description);
                           else
                            if (isValid($mrl->label))  $web_title = quoteForText($mrl->label);
//                            else if (isValid($row->web_title)) $web_title = quoteForText($row->web_title);
                    
                    if (substr($link,0,1) == '.') $cell .='<a href="'. $link;
                    else $cell .='<a href="http://'.$link; 
                    if (isValid($rv->web_target)) $cell .='" target="'.quoteForText($rv->web_target);
                    if (isValid($web_title)) $cell .='" title="'.quoteForText($web_title);
                    $cell .= '" >';
                }
            }
        }
        else{
            if (isValid($rv->web_title)) 
            $cell .='<a title="'.quoteForText($rv->web_title).'" href="./outputReport.php?report='.$rv->web_link;
            else
            $cell .='<a title="Aufruf des Reports '.quoteForText($rv->web_link).'" href="./outputReport.php?report='.$rv->web_link;     
//                $vname = $mr->getName();
//                if (substr($rv->getName(), 1,1) == ':') $vname = substr($rv->getName(), 0,1).'__'.$vname;
                $cell.= makeGetFilter($rv->webCopyFilter, $zeile, $rv->getName(), null, null);
            if (isValid($rv->web_target))$cell .='" target="'.quoteForText($rv->web_target);
                
            $cell .= '" >';
            $hasLink = true;
        }
    }
    if ($rv->webLinkColumn){
        if (! $spreadsheet) {
            if ($hasLink) $cell .= $rv->web_link_value;
            else $cell .= '&nbsp;';
        }
    }else{
        $vd = $mr->getDispName();
//    if (substr($vnr,1,1) == ':' || substr($vnr,1,1) == '=' ) $vd = substr($vnr,0,1).'__'.$vd;
        $cell .= formatWert($mr, $zeile[$vd]);
    }
    if (isValid($rv->web_link) && ! $spreadsheet)$cell .= '</a>';
    if ($rv->webLinkColumn){
        if (! $spreadsheet) $cell.='</td>';
    }
    else $cell .= "</".$mr->htmlData.">\n";
    return $cell;
}

function startRequiredFilter(){
    global $query, $filterVector, $aktiverEditor, $moves ;
    /****
     * Ausgabe aller Filter für diese Abfrage
     */
    ?>
<div class="inlineBox ">
<?php 
if (array_key_exists('Name', $_SESSION))
    echo '<b>'.WSRTranslate::translate('Anmeldung').':</b> '.$_SESSION['Name']."\n";
?>
<h3><?=WSRTranslate::translate('Zusätzliche Auswahl erforderlich').anleitung('1','outputReqFilter') ?> </h3>
<p><?=WSRTranslate::translate('Die Datenquelle benötigt einen zustätzlichen Filter.')?>
<br>(<?=WSRTranslate::translate('Aus konzeptionellen Gründen oder weil sonst die abgefragte Datenmenge zu groß wäre.')?>)</p> 
 <?php 
 //  (<?=number_format($fertig,0,',','.'))
 for ($i=0; $i < $query->repFilter->size(); $i++){
     $filter =  $query->repFilter->getPos($i);
     $titel = showFilter($filter,true);
     if (isValid($titel)){
         echo "<div class=\"nowrap\">".$titel; 
         if ($filter->abSpalte > 0) echo ' ab '
            .$query->getReportMetaRecord($query->repRows->getPos($filter->abSpalte-1)->getName())->label;
         echo "</div>\n";
     }
 }
 
/****
 * Zusätzliche Filter-Optionen, die der Benutzer aufrufen kann
 */
 
// if ($selFilters && ! array_key_exists('spreadSheet', $_REQUEST)){
     require_once './includes/enterFilterSpec.inc';
?>
    <form id="FilterForm2" method="post" action="./outputReport.php" class="noprint">
    <input type="hidden" name="report" value="<?=$query->title ?>"> 
    <input type="hidden" name="setFilterNames" value="X"> 
    <input type="hidden" name="setFilter" value=""> 
    <input type="hidden" name="setFilterValue1" value=""> 
    <input type="hidden" name="setFilterValue2" value=""> 
    <input type="hidden" name="setFilter2" value=""> 
    <input type="hidden" name="setFilter2Value1" value=""> 
    <input type="hidden" name="setFilter2Value2" value=""> 
<?php 
if ($aktiverEditor){
    ?>
	<input type="hidden" name="action" value="reportBearbeiten"/>
	<input type="hidden" name="id" value="<?=$_REQUEST['id']?>"/>
	<input type="hidden" name="table" value="<?=$_REQUEST['table']?>"/>
 <?php 
}else{
    $moves->outputParams();
}

/****
 * Existierende Filter-Optionen aus $filterVector
*/ 
    makePostFilter();
   
}

function endRequiredFilter(){
 
 ?>
 <input type="submit" value="<?=WSRTranslate::translate('Anzeigen')?>">
    </form>
</div>     
<?php 
} // Ende function endRequiredFilter



function checkRequiredFilter(){
    global $query, $filterVector, $dqFilter ;
    $filterRequired = false;    
    $needTestGroup = array();
    $needTestGroup['base'] = array();
    foreach ($query->metaData as $name => $mr){
        if ($mr->filterRequired) $needTestGroup['base'][$mr->filterRequiredGroup] = true;
    }
    for ($i = 0; $i < $query->repTables->size(); $i++){
        $r = $query->repTables->getPos($i);
        $needTestGroup[$r->kennzeichen] = array();
        foreach ($r->query->metaData as $name => $mr){
            if ($mr->filterRequired) $needTestGroup[$r->kennzeichen][$mr->filterRequiredGroup] = true;
        }
    }
    
    
    
    
    for ($i = 0; $i < $query->repFilter->size(); $i++){
        $f = $query->repFilter->getPos($i);
        $n = $f->getName();
        if ($f->isWindowFilter()) $mr = $f->getMetaRecord();
        else $mr = $query->getReportMetaRecord($n);
// Hier entsteht ein Problem, wenn der Filter nur in Verknüpften Tabellen erforderlich ist.

        if ($f->isSet()){
            if (substr($n, 1,1) == ':' && $mr->filterRequired) {
                $needTestGroup[substr($n,0,1)][$mr->filterRequiredGroup] = false;
            }
            else {
                if ($mr->filterRequired)  $needTestGroup['base'][$mr->filterRequiredGroup] = false;
                foreach ($needTestGroup as $table => $nt){
                    if ($table != 'base'){
                        $jt = $query->repTables->get($table);
                        $tQuery = $jt->query;
                        if (array_key_exists($n,$tQuery->metaData)
                            && $tQuery->metaData[$n]->filterRequired)
                            $needTestGroup[$table][$tQuery->metaData[$n]->filterRequiredGroup] = false;
                    }
                }
            }
        }
    }
    
    foreach (rechteFilter() as $n){
        $mr = $query->getReportMetaRecord($n);
        if ($mr == null) continue;
        // Hier entsteht ein Problem, wenn der Filter nur in Verknüpften Tabellen erforderlich ist.
        if (substr($n, 1,1) == ':' && $mr->filterRequired) {
            $needTestGroup[substr($n,0,1)][$mr->filterRequiredGroup] = false;
        }
        else {
            if ($mr->filterRequired)  $needTestGroup['base'][$mr->filterRequiredGroup] = false;
            foreach ($needTestGroup as $table => $nt){
                if ($table != 'base'){
                    $jt = $query->repTables->get($table);
                    $tQuery = $jt->query;
                    if (array_key_exists($n,$tQuery->metaData)
                        && $tQuery->metaData[$n]->filterRequired)
                        $needTestGroup[$table][$tQuery->metaData[$n]->filterRequiredGroup] = false;
                }
            }
        }
    }
    
/**
 * Das Thema 'notwendige Filter' ist noch nicht wirklich ausgereift.
 * Der Benutzerfilter aus der Berechtigung wird übernommen, wenn die Datenquelle
 * den Filter kennt. In der Report-Definition ist er dann nicht erforderlich.
 * Wenn aber der Administrator einen solchen Report bearbeiten will fehlt
 * der Filter und wird auch nicht über die Links im Report übergeben (bzw. nicht übernommen).
 * 
 * Im "Normalbetrieb" sollten nur die notwendigen Filter angezeigt werden,
 * die in der Report-Definition vorhanden sind.
 * Aber in der Entwicklung sollten alle Filter angezeigt werden, damit sie ggf.
 * in der Report-Definition aufgenommen werden können.
 * 
 * Aktuelle Lösung:
 * Notwendige Filter, die im Report nicht vorhanden sind, werden nur angezeigt,
 * wenn der Report aus der Report-Definition aufgerufen wurde.
 * 
 *           NEU MACHEN !!
*/         
    foreach ($needTestGroup as $table => $ntG){
        foreach ($ntG as $testGroup => $nt) {
            if (!$needTestGroup[$table][$testGroup]) continue;
            $ftg = true;
            
        if ($table == 'base'){
            if (isset($dqFilter) && isValid($dqFilter)
                && array_key_exists($dqFilter, $query->metaData)
                && $query->metaData[$dqFilter]->filterRequired
                && $query->metaData[$dqFilter]->filterRequiredGroup == $testGroup){                
            }else { 
               
            foreach ($query->metaData as $name => $metaRecord){
                if ($metaRecord->filterRequired && $metaRecord->filterRequiredGroup == $testGroup){
                        if (! $filterRequired){
                        $filterRequired = true;
                        startRequiredFilter();
                    }
                    if ($ftg){echo '<hr>'; $ftg = false;}
                    if ($metaRecord->filterType == MetaRecord::SONDERFILTER) continue;
                    if ($metaRecord->filterType == MetaRecord::WINDOWFILTER) continue;
                    $label = $metaRecord->label;
                    $filter = new RepFilter($metaRecord->getName());
                ?>
                    <div class="nowrap" >
                    <?php 
                    if ( ! $query->repFilter->has($metaRecord->getName()) ) 
                        if (array_key_exists('action', $_REQUEST))
                            echo '<span class="help" title="'
                                .WSRTranslate::translate('Der Filter ist NICHT in der Report-Definition').'">?</span> ';
                            else {
                                echo '</div>';
                                continue;
                            }
                    // echo $testGroup.' ';
                    ?>
                    <b <?=makeTitleClass(makeFilterTitleDescription($metaRecord),'') ?>><?=$label?></b>
                    <?php
//                    $metaRecord->resetSelectOptions('base');    
                enterFilterSpec('Check','\''.trim($filter->getName()).'\'',$filter->getName(), $filter->fs_von, $filter->fs_bis);
                    echo '</div>';
                    foreach ($needTestGroup as $tableI => $ntGI){
                        if ($tableI != 'base' && $ntGI){
                            $jMetaData = $query->repTables->get($tableI)->query->metaData;
                            if (array_key_exists(trim($filter->getName()), $jMetaData)
                                && $jMetaData[trim($filter->getName())]->filterRequired){
                                    $needTestGroup[$tableI][$jMetaData[trim($filter->getName())]->filterRequiredGroup] = false;
                            }
                        }
                    }
                }
            }}
        }else{
            $jt = $query->repTables->get($table);
            $tQuery = $jt->query;
            if (isset($dqFilter) && isValid($dqFilter)
                && array_key_exists($dqFilter, $tQuery->metaData)
                && $tQuery->metaData[$dqFilter]->filterRequired
                && $tQuery->metaData[$dqFilter]->filterRequiredGroup == $testGroup){
            }else
                foreach ($tQuery->metaData as $name => $metaRecord){
                if ($metaRecord->filterRequired
                    && $metaRecord->filterRequiredGroup == $testGroup
                    &&( (!array_key_exists($name, $query->metaData))
                        || (!$query->metaData[$name]->filterRequired))){
                        
                    if (! $filterRequired){
                        $filterRequired = true;
                        startRequiredFilter();
                    }
                    if ($ftg){echo '<hr>'; $ftg = false;}
                    if ($metaRecord->filterType == MetaRecord::SONDERFILTER) continue;
                    if ($metaRecord->filterType == MetaRecord::WINDOWFILTER) continue;                    
                    if (isValid($jt->prefix)) $label = $jt->prefix.' '.$metaRecord->label;
                    else $label = $table.': '.$metaRecord->label;
                    $filter = new RepFilter($metaRecord->getName());
                    ?>
                    <div class="nowrap" ></div>
                    <?php 
                    if ( ! $query->repFilter->has($metaRecord->getName()) )                        
                        if (array_key_exists('action', $_REQUEST))
                            echo '<span class="help" title="'
                                .WSRTranslate::translate('Der filter ist NICHT in der Report-Definition').'">?</span> ';
                        else {
                            echo '</div>';
                            continue;
                        }
                        
                    ?>
                    <b <?=makeTitleClass(makeFilterTitleDescription($metaRecord),'') ?>><?=$label?></b>
                    <?php
//                    $metaRecord->resetSelectOptions($jt->kennzeichen);   
                    enterFilterSpec('Check','\''.$table.':'.$filter->getName().'\'',$table.':'.$filter->getName(), $filter->fs_von, $filter->fs_bis);
                    echo '</div>';
                }
            }
        }
        }
    }
    if ($filterRequired){
        echo '<hr>';
        endRequiredFilter();
        
        if (! array_key_exists('action', $_REQUEST)){
?>
  <div class="inlineBox noprint">
     <form method="get" action = "./editReport.php" target="ReportEditor">
<?php 
makePostFilter();
?>     <input type="hidden" name="report" value="<?=$query->title?>">
     <input type="submit" name="editor" value="<?=WSRTranslate::translate('Report bearbeiten')?>" />
     </form>
     </div>
<?php 
/****
 * Kommentar ausgeben
 */
if (isValid($query->kommentar)) echo'<br><div class="kommentar">'.quoteForHTML($query->kommentar).'</div><br>';
        }
//        echo'<pre>filterVector:';var_dump($filterVector);echo'</pre>';
//        echo'<pre>repFilter:';var_dump($query->repFilter);echo'</pre>';
        echo '</body></html>';
        die();
    }
}
// ENDE checkRequiredFilter

function showFilterValue($name){
    global $query;    
   $mr = $query->getReportMetaRecord($name);
   if ($mr == null) return '';   
   if (!isValid($mr->displayVar)) return'';
   if (! $query->repFilter->get($name)->isSet()) return '';
   if ($mr->filterType == MetaRecord::SONDERFILTER || $mr->filterType == MetaRecord::SONDERFILTERSEL){
      $sql = new SqlAbfrage('DWH');
      $vn = 'FD_'.str_replace(':', '_', $name);
      $sql->sql = 'select '.$vn.' as Wert from WSR_ergebnis where '.$vn.' is not null';
      $stmt = $sql->execute();
      if ($z = $stmt->fetchObject()) return $z->Wert;
   }
   return '';
}

function werteErsetzen(){
    global $query;
    $von = $query->repWerteBasis;
    $nach = $query->repWerteAuswahl;
    for ($i=0; $i < $query->repValues->size(); $i++){
        /**
         * @var RepValue $repVal
         */
        $repVal = $query->repValues->getPos($i);
        if ($repVal->isWindowValue()){
            $vn = $repVal->value;
            $vnNeu = str_replace($von, $nach, $vn);
            if ($vnNeu != $vn && $query->getReportMetaRecord($vnNeu) != null){
                $repVal->value = $vnNeu;
            }            
        }else{
            $vn = $repVal->getName();
            $vnNeu = str_replace($von, $nach, $vn);
            if ($vnNeu != $vn && $query->getReportMetaRecord($vnNeu) != null){
                $query->repValues->rename($i,$vnNeu);
            }            
        }
   }
    for ($i=0; $i < $query->repFilter->size(); $i++){
        $vn = $query->repFilter->getPos($i)->getName();
        $vnNeu = str_replace($von, $nach, $vn);
        if ($vnNeu != $vn && $query->getReportMetaRecord($vnNeu) != null){
            $query->repFilter->rename($i,$vnNeu);
        }
    }
    for ($i=0; $i < $query->repColumns->size(); $i++){
        $v = $query->repColumns->getPos($i);
        if (false) $v = new RepColumn('');
        $vn = $v->val_name;
        $vnNeu = str_replace($von, $nach, $vn);
        if ($vnNeu != $vn && $query->getReportMetaRecord($vnNeu) != null){
            $v->val_name = $vnNeu;
            $mr = $query->getReportMetaRecord('WSR_WERTELISTE');
            $options = $mr->getSelectOptions();
            $vonLabel = null;
            $nachLabel = null;
            foreach ($options as $opt){
                if ($opt[0] == $von) $vonLabel = $opt[1];
                if ($opt[0] == $nach) $nachLabel = $opt[1];
            }
            if ($vonLabel != null && $nachLabel != null)    
            $v->label = str_replace($vonLabel, $nachLabel, $v->label);
        }
        $vn = $v->filter;
        $vnNeu = str_replace($von, $nach, $vn);
        if ($vnNeu != $vn && $query->getReportMetaRecord($vnNeu) != null){
            $v->filter = $vnNeu;
        }
    }
}

$meldungen = "";
/**
 * var $query Query
 */
$query = null;

if (array_key_exists('reportID', $_REQUEST)
    && !array_key_exists('action',$_REQUEST)){
    // Wenn die Report-ID als Parameter übergeben wird, immer neu eingelesenen Report ausgeben.
    require_once './includes/loadReportDef.inc';
    loadReportID($_REQUEST['reportID']);
    if ($query != null) $query->loadSaveAddFilter = false;
    else echo 'FEHLER Query ist NULL bei ReportID '.$_REQUEST['reportID'];
    leseRechteVerlangen();
    
}else{
    error_log('Suche Report: '.$_REQUEST['report']);
    if (session_status() == PHP_SESSION_NONE)  session_start();    
    if (array_key_exists('WSReportQuery', $_SESSION)){
    //DEBUG HIER WEITERMACHEN
    //    $meldungen .= "Benutze Session-Query<br>";
        $query = $_SESSION['WSReportQuery'];
    }
    session_write_close();
    if (array_key_exists('report', $_REQUEST)){
        if ($query == null || $query->title != $_REQUEST['report']){        
            require_once './includes/loadReportDef.inc';
error_log('Lade Report');            
            loadReport($_REQUEST['report']);
            //DEBUG
    //        $meldungen .= "Lade Report neu<br>";
            if ($query != null) $query->loadSaveAddFilter = false;
            else echo 'FEHLER Query ist NULL bei Report '.$_REQUEST['report'];
            
    //DEBUG        
    //        else $meldungen .= "übernehme AddFilter<br>";
        }else if ($query == null){
            error_log('Query bleibt null');
        }else{
            error_log('Query-Title = '.$query->title);
        }
    }
    error_log('Query-Title ist jetzt '.$query->title);
    // if (false) $query = new Query('');
    leseRechteVerlangen();
    if ($query == null && !array_key_exists('report', $_REQUEST)){
        echo WSRTranslate::translate('FEHLER').':<br>'.WSRTranslate::translate('Keine Report-Anforderung gefunden.');
        die();
    }
    if ($query == null || trim($query->title) != trim($_REQUEST['report'])){
        echo WSRTranslate::translate('FEHLER').':<br>'.WSRTranslate::translate('Die Report-Defintion')
        .' >'.$_REQUEST['report'].'< '.WSRTranslate::translate('konnte nicht geladen werden.')
        .'<br>'.WSRTranslate::translate('Query-Title ist')
        .' >'.$query->title.'<';
        die();
}
} // ENDE ohne Parameter reportID
/*****
 * Filter-Vektor aufbauen bzw. ergänzen.
 * Wenn outputReport aus dem editor aufgerufen wurde, is der Parameter 'action' gesetzt.
 * Dann und nur dann werden die addFilter aus der Query übernommen. 
 * @var Vector[RepFilter] $filterVector: enthält alle zusätzlich gesetzten Filter.
 */

if ($aktiverEditor)
    $filterVector = $query->addFilter;
else $filterVector = new Vector();
//   error_log(print_r($_REQUEST,true));

$werte = null;
if (array_key_exists('Werte', $_REQUEST)){
    $werte = $_REQUEST['Werte'];
}
if (array_key_exists('Werte2', $_REQUEST)){
    $werte = $_REQUEST['Werte2'];
}
if ($werte != null){
    $mrf = $query->getReportMetaRecord('WSR_WERTELISTE');
    if ($mrf != null){
        if (false) $mrf=new MetaRecord();
        $options = $mrf->getSelectOptions();
        for ($i=0; $i < count($options); $i++)
            if ($options[$i][1] == $werte) {
                $query->repWerteAuswahl = $options[$i][0] ;
                $i = count($options);
        }
        for ($i=0; $i < count($options); $i++)
            if ($options[$i][0] == $werte) {
                $query->repWerteAuswahl = $options[$i][0] ;
                $i = count($options);
            }
    }else $werte = null;
}

if (isValid($query->repWerteAuswahl) 
    && isValid($query->repWerteBasis)){
    if ($query->repWerteAuswahl != $query->repWerteBasis){
        werteErsetzen();
    }
    $mrf = $query->getReportMetaRecord('WSR_WERTELISTE');
    if (false) $mrf=new MetaRecord();
    $options = $mrf->getSelectOptions();
    for ($i=0; $i < count($options); $i++)
        if ($options[$i][0] == $query->repWerteAuswahl) {
            $werte = $options[$i][1] ;
            $i = count($options);
        }
}
else {
    $werte = null;
    $query->repWerteAuswahl = null;
}

if (array_key_exists('addFilter', $_REQUEST)){
    for ($i=0; $i < $_REQUEST['addFilter']; $i++){
        $fn = $_REQUEST['filter'.$i];
        $l = $fn;
        if ($query->repFilter->has($fn)){
            $l.=' gefunden ';
            $f = $query->repFilter->get($fn);
            if ($f->doNotTake) {
//                error_log($l.' nicht genommen');
                continue;
            }
            $l .= ' genommen ';
            if (array_key_exists('filterValue'.$i.'V1', $_REQUEST)){
                $f->fs_von = $_REQUEST['filterValue'.$i.'V1'];                
            }
            else $f->fs_von = null;
            if (array_key_exists('filterValue'.$i.'V2', $_REQUEST)){
                $f->fs_bis = $_REQUEST['filterValue'.$i.'V2'];
            }
            else $f->fs_bis = null;
            $filterVector->put($f);
            $mr = $query->getReportMetaRecord($fn);
            if ($mr != null && $mr->filterType == MetaRecord::LIST)
                $mr->filterType = MetaRecord::SELECT;
            if (false) error_log($l.print_r($f,true));
        }
//        else error_log($l.' nicht gefunden');
    }
}


if (array_key_exists('setFilterNames', $_REQUEST)){
    if (array_key_exists('setFilter', $_REQUEST) && strlen($_REQUEST['setFilter']) > 0){
            $f = $query->repFilter->get( $_REQUEST['setFilter']);
            if ($f == null) $f = new RepFilter($_REQUEST['setFilter']);
            if ($f != null){
                //        $f = $query->repFilter->get($fn);
                if (array_key_exists('setFilterValue1', $_REQUEST)){
                    $f->fs_von = $_REQUEST['setFilterValue1'];
                }
                else $f->fs_von = null;
                if (array_key_exists('setFilterValue2', $_REQUEST)){
                    $f->fs_bis = $_REQUEST['setFilterValue2'];
                }
                else $f->fs_bis = null;
                $filterVector->put($f);
            }
    }
    
    if (array_key_exists('setFilter2', $_REQUEST) && strlen($_REQUEST['setFilter2']) > 0){
            $f = $query->repFilter->get( $_REQUEST['setFilter2']);
            if ($f == null) $f = new RepFilter($_REQUEST['setFilter2']);
            if ($f != null){
                //        $f = $query->repFilter->get($fn);
                if (array_key_exists('setFilter2Value1', $_REQUEST)){
                    $f->fs_von = $_REQUEST['setFilter2Value1'];
                }
                else $f->fs_von = null;
                if (array_key_exists('setFilter2Value2', $_REQUEST)){
                    $f->fs_bis = $_REQUEST['setFilter2Value2'];
                }
                else $f->fs_bis = null;
                $filterVector->put($f);
            }
    }    
}else{
    if (array_key_exists('setFilter', $_REQUEST) && strlen($_REQUEST['setFilter']) > 0){
//        error_log("setFilter ohne setFilterNames");
        $f = $query->repFilter->getPos($_REQUEST['setFilter']);
            if ($f != null){
                //        $f = $query->repFilter->get($fn);
                if (array_key_exists('setFilterValue1', $_REQUEST)){
                    $f->fs_von = $_REQUEST['setFilterValue1'];
                }
                else $f->fs_von = null;
                if (array_key_exists('setFilterValue2', $_REQUEST)){
                    $f->fs_bis = $_REQUEST['setFilterValue2'];
                }
                else $f->fs_bis = null;
                $filterVector->put($f);
            }
    }
    
    if (array_key_exists('setFilter2', $_REQUEST) && strlen($_REQUEST['setFilter2']) > 0){        
//        error_log("setFilter2 ohne setFilterNames");
        $f = $query->repFilter->getPos( $_REQUEST['setFilter2']);
            if ($f != null){
                //        $f = $query->repFilter->get($fn);
                if (array_key_exists('setFilter2Value1', $_REQUEST)){
                    $f->fs_von = $_REQUEST['setFilter2Value1'];
                }
                else $f->fs_von = null;
                if (array_key_exists('setFilter2Value2', $_REQUEST)){
                    $f->fs_bis = $_REQUEST['setFilter2Value2'];
                }
                else $f->fs_bis = null;
                $filterVector->put($f);
            }
    }
    
}
/**
 * filterVector in repFilter eintragen:
 * Bei den addFilter wurde oben geprüft, ob sie übernommen
 * werden oder repFilter->doNotTake gesetzt ist.
 * setFilter werden erstmal grundsätzlich übernommen.
 * Wenn gleichzeitig doNotTake gesetzt ist, gehen sie aber beim
 * nächsten Aufruf wieder verloren.
 */

if ($filterVector->size() > 0){
    for ($i=0; $i < $filterVector->size(); $i++){
        $fix = $filterVector->getPos($i);
        if ($fix != null)
        if ($fix->isSet()) $query->repFilter->put($fix);
    }
}


/*
 * Erst hier ist der repFilter des Reports vollständig und kann geprüft werden
 */

// error_log("Rep-Filter ".print_r($query->repFilter,true));
// error_log("FilterVector ".print_r($filterVector,true));

//DEBUG
// echo'<pre>';var_dump($query->repFilter);echo'</pre>';

$moves->apply();

/****
 * Parameter für HTTP-Links "SQL" und "Tabellen" vorbereiten
 * @var String $addFilter: zusätzliche Filter-Parameter im Get-Modus
 */
$addFilter = makeGetFilter(true,null,null,null,null);

if (array_key_exists('sortDir', $_REQUEST))  $addFilter.='&sortDir='.$_REQUEST['sortDir'];
if (array_key_exists('sortVal', $_REQUEST)) $addFilter.='&sortVal='.$_REQUEST['sortVal'];
if (array_key_exists('sortCol', $_REQUEST)) $addFilter.='&sortCol='.$_REQUEST['sortCol'];
if (array_key_exists('sortColVal', $_REQUEST)) $addFilter.='&sortColVal='.$_REQUEST['sortColVal'];



$border = "";

if ($spreadsheet){
    header("Content-Type: application/vnd.oasis.opendocument.spreadsheet; charset=UTF-8");
    header("Content-disposition: attachment; filename=\"".$query->title.".html\"");
    $border = " border =\"1\" ";
    $bearbeitenButton = false;
}else{
    header("Content-Type: text/html; charset=UTF-8");
    if ($aktiverEditor || isset($_SESSION['EndUser'])) $bearbeitenButton = false;
    else $bearbeitenButton = true;    
}

if ($query->queryType > Query::DATEN){
    if (session_status() == PHP_SESSION_NONE)  session_start();
    $_SESSION['WSReportErfassung'] = $query;
    session_write_close();
}

/******
 * Beginn der Ausgabe
 * filterRequiredGroup NOCH IMPLEMENTIEREN ?
 * 
 * Jede Ausgabe mit einer gültigen reportID wird gezählt, damit die Verwendung
 * von Reports ausgewertet werden kann.
 */
// global $sqlErg;
// $sqlErg = 0;
if ($query->reportID > 0){
    $logSql = new SqlAbfrage('WSR');
    $logSql->sql = 'update wsrAdminReport set lastUse=current_date(), useCounter = useCounter+1';
    $logSql->addSel('reportID = ', $query->reportID);
    $erg = $logSql->executeE(false);
    if ($erg === false || $erg->rowCount() < 1 ){
        error_log('Fehler beim Update wsrAdminReport: '. $logSql->errorInfo()[2]);
    }
}

?>
<!DOCTYPE html>
<html>
<head>
<meta charset="UTF-8">
<?php if ($query->outputTitlePrefix == $query->title) {?>
<title><?=$query->outputTitlePrefix ?></title>
<?php }else{?>
<title><?=$query->outputTitlePrefix ?>: <?=$query->title?></title>
<?php }?>
<meta name="GENERATOR" content="WS-PHP: outputReport.php">
<meta name="viewport" content="width=device-width, initial-scale=1.0">
<link rel="stylesheet" href="./jscss/standard2.css">
<link rel="stylesheet" href="./jscss/tableFolder.css">
<link rel="stylesheet" href="<?=$query->reportCSS ?>" >
<script type="text/javascript" src="./jscss/tableHeader.js">
	</script>
<script type="text/javascript" src="./jscss/tableFolder.js">
	</script>
<script type="text/javascript">
	var chartSize = "<?=$query->chart_size?>";			
	</script>
	
<script type="text/javascript" src="./jscss/WSReport.js">
	</script>
<script type="text/javascript">
var call = "<?=$_SERVER['SCRIPT_NAME']?>?id=<?=$query->object_id?>&table=<?=$query->class_table?>";
var requestID = "<?=$query->object_id?>";
var requestTable = "<?=$query->class_table?>";
console.log("Call="+call);
window.addEventListener('load', function() {
	var th = new TableHeader("ReportTabelle");
 	th.activateHover(<?=$query->reportHoverMode ?>); 	
// 	var tf = new TableFolder("ReportTabelle",TableFolder.Summen);
<?php
if ($query->folderMode == Query::FOLDERERSTEZEILE){
    ?>
var tf = new TableFolder("ReportTabelle",TableFolder.Header,TableFolder.Eingeklappt,true,true);
// tf.restore();
<?php 
}else
if ($query->folderMode > Query::KEINE){
?>
var tf = new TableFolder("ReportTabelle",TableFolder.WSReporter,<?=$query->folderMode - 2?>,true,true);
// tf.restore();
<?php 
}
?>
 });

	</script>
</head> 
<?php
$selFilter = false;
$rechteFilter = rechteFilter();

if ($query->repWerteBasis != null
    && $query->repWerteAuswahlSelector) $selFilter = true;
else
for ($i=0; $i < $query->repFilter->size(); $i++){
    $filter =  $query->repFilter->getPos($i);
    if (false) $filter = new RepFilter('N');
    if (in_array($filter->getName(), $rechteFilter)) {
        //             error_log('Überspringe Rechte-Filter '.$filter->getName());
        continue;
    }else{
        if ($filter->selectorImOutput) $selFilter = true;
    }
}



if ($_SERVER["HTTP_HOST"] == "localhost:8898") echo '<body class="entwicklung">';
else echo '<body>';
if (! $spreadsheet){    
?>
<div id='kopfleiste' class='kopfleiste noprint' >
<img src='./icons/open-menu256.png' class='button' onclick='showReports()'>
<?php 
if ((array_key_exists('reports', $_SESSION) && count($_SESSION['reports']) > 0)
    || $query->repReports->size() > 0){    
?> 
<a onclick='showReports()' class='kopfleiste' title="<?=WSRTranslate::translate('Andere Reports aufrufen')?>"
	><?=WSRTranslate::translate('Reports')?></a>
<div style="display: inline-block">
<?php
}
if ($selFilter){
?>
<a onclick='showFilter()' class='kopfleiste' title="<?=WSRTranslate::translate('Filter anpassen')?>">
<img src='./icons/filter1.svg' class='icon' /> <?=WSRTranslate::translate('Filter')?></a>
<?php
}
if (dqAdminRechte() || $entwicklung){
    /****
     * Debug-Links für Administratoren der Datenquelle
     * $addFilter hat hier alle Parameter aus dem $filterVector
     */
    ?>
<a href="./showSQL.php?report=<?=$query->title.$addFilter?>" target="showSQL"   class='kopfleiste' >SQL</a>
<a href="./showSQL.php?report=<?=$query->title.$addFilter?>&showTable=X" target="showSQL"  class='kopfleiste' ><?=WSRTranslate::translate('Tabellen')?></a>

<?php
}
if ($bearbeitenButton){
    ?>
     <a class='kopfleiste'  onclick='document.getElementById("RepEditor").submit()'
     title="<?=WSRTranslate::translate('Report-Editor zum Bearbeiten' 
     ."\nder Report-Spezifikation aufrufen")?>"><img src='./icons/bearbeiten.png' class='icon' /> 
     <?=WSRTranslate::translateH('Report bearbeiten')?></a>
<?php
}
if (false) $query = new Query('');
if ($query->reportHasEnterValues()){
    ?>
     <a class='kopfleiste'  onclick='document.getElementById("Aktualisieren").submit()'
     title="<?=WSRTranslate::translate('Aktualisieren der Anzeige nach Eingaben')?>"
     ><img src='./icons/view-refresh.png' class='icon' /> <?=WSRTranslate::translate('Aktualisieren')?></a>
<?php
}
?>
     <a class='kopfleiste'  onclick='document.getElementById("Spread").submit()'
     title="<?=WSRTranslate::translate('Ausgabe der Tabelle als Spreadsheet')?>"
     ><img src='./icons/calc.png' class='icon' /> <?=WSRTranslate::translate('als Spreadsheet')?></a>
      <a class='kopfleiste' id= "Anleitung"  onclick="anleitung('12','outputReport')"
      ><img src='./icons/devhelp.png' class='icon' /> <?=WSRTranslate::translate('Anleitung')?></a>
<?php
}
?>
</div> 
</div>
<?php 
if(
    ($query->repReports->size() > 0 || array_key_exists('reports', $_SESSION))
    && ! $spreadsheet) {
        ?>
  <div class="inlineBox noprint" id='ShowReports'>
  <div class="xButton" onclick="showReports()" title="<?=WSRTranslate::translate('schließen')?>">&otimes;</div>
<h3><?=WSRTranslate::translate('Weitere Reports')?></h3>
<table>
<?php 
if (array_key_exists('startSeite', $_SESSION) && isValid($_SESSION['startSeite'])){
    ?>
<tr><th class="left"><a href="./outputReport.php?report=<?=$_SESSION['startSeite']?>"><?=$_SESSION['startSeite'] ?></a></th><td>Startseite</td></tr>
<?php 
}//// Startseite
for ($ri = 0; $ri < $query->repReports->size(); $ri++){
    $rep = $query->repReports->getPos($ri);
    $aufruf = $rep->getName();
    if ($rep->copyFilter){
        $aufruf .= makeGetFilter(true, null, null, null, null);
        $rep->kommentar .=' *';
    }
    ?>
<tr><th class="left"><a href="./outputReport.php?report=<?=$aufruf ?>"><?=$rep->getName() ?></a></th><td><?=quoteForHTML($rep->kommentar) ?></td></tr>
    <?php }
if (array_key_exists('reports', $_SESSION) && count($_SESSION['reports']) > 0) 
    foreach ($_SESSION['reports'] as $report){
        $rep = $query->repReports->get($report);
        if ($rep == null){
    ?>
<tr><th class="left"><a href="./outputReport.php?report=<?=$report ?>"><?=$report ?></a></th><td>&nbsp;</td></tr>
    <?php 
}
else if ($rep->copyFilter){
    ?>
<tr><th class="left"><a href="./outputReport.php?report=<?=$report ?>"><?=$report ?></a></th><td>-</td></tr>
    <?php 
}
}
?>    
</table>
<small>* <?=WSRTranslate::translate('übernimmt gesetzte Filter')?></small>
</div>
<?php 
} // Ende ReportVrz
?>
<form method="get" action = "./editReport.php" target="ReportEditor" id='RepEditor'>
<?php
makePostFilter();
?>     <input type="hidden" name="report" value="<?=$query->title?>">
     </form>
     <form method="put" action = "./outputReport.php" id='Spread' >
<?php 
makePostFilter();
$moves->outputParams();
?>
     <input type="hidden" name="report" value="<?=$query->title?>">
     <input type="hidden" name="spreadSheet" value="X">
     </form>

     <form method="put" action = "./outputReport.php" id='Aktualisieren' >
<?php 
makePostFilter();
$moves->outputParams();
if (array_key_exists('action', $_REQUEST)
    && array_key_exists('id', $_REQUEST)
    && array_key_exists('table', $_REQUEST)){
?>
     <input type="hidden" name="action" value="<?=$_REQUEST['action']?>">
     <input type="hidden" name="id" value="<?=$_REQUEST['id']?>">
     <input type="hidden" name="table" value="<?=$_REQUEST['table']?>">
<?php
}
?>
     <input type="hidden" name="report" value="<?=$query->title?>">
     </form>


<?php
/****
 * Zusätzliche Filter-Optionen, die der Benutzer aufrufen kann
 */

if ($selFilter && ! $spreadsheet){
    require_once './includes/enterFilterSpec.inc';
    ?>
  <div class="inlineBox noprint" id='ShowFilter'>
  <div class="xButton" onclick="showFilter()" title="<?=WSRTranslate::translate('schließen')?>">&otimes;</div>
    
<h3>Filter setzen oder ändern</h3>
    <form id="FilterForm" method="post" action="./outputReport.php" class="noprint">
    <input type="hidden" name="report" value="<?=$query->title ?>"> 
    <input type="hidden" name="setFilter" value=""> 
    <input type="hidden" name="setFilterValue1" value=""> 
    <input type="hidden" name="setFilterValue2" value=""> 
    <input type="hidden" name="setFilter2" value=""> 
    <input type="hidden" name="setFilter2Value1" value=""> 
    <input type="hidden" name="setFilter2Value2" value=""> 
<?php 
if ($aktiverEditor){
    ?>
	<input type="hidden" name="action" value="reportBearbeiten"/>
	<input type="hidden" name="id" value="<?=$_REQUEST['id']?>"/>
	<input type="hidden" name="table" value="<?=$_REQUEST['table']?>"/>
 <?php 
}else{
    $moves->outputParams();
}

/****
 * Existierende Filter-Optionen aus $filterVector
 */
makePostFilter();

/****
 * RepFilter-Elemente mit Option selectorImOutput
 * Werte2 überschreibt vorher gesetzten Werte-Parameter
 */
if ($query->repWerteBasis != null && $query->repWerteAuswahlSelector){
    $aw = $query->repWerteAuswahl;
    if ($aw == null) $aw = $query->repWerteBasis;
    $mrf = $query->getReportMetaRecord('WSR_WERTELISTE');    
?>
<b><?=WSRTranslate::translate('Werte')?>:</b> <select id="WSR_WERTELISTE" name="Werte2">
<?php 

    $options = $mrf->getSelectOptions();
    foreach ($options as $opt){
        ?>
<option value="<?=$opt[0] ?>" <?=checkSel($aw, $opt[0])?> ><?=$opt[1] ?></option>        
<?php 
    }
    ?>
</select>   <br> 
<?php 
    
}
    for ($i=0; $i < $query->repFilter->size(); $i++){
         $filter =  $query->repFilter->getPos($i);
         if (in_array($filter->getName(), $rechteFilter)) {
//             error_log('Überspringe Rechte-Filter '.$filter->getName());
             continue;
         }else{
//             error_log($filter->getName().' wird zur Auswahl angezeigt.');
         }
         if ($filter->selectorImOutput) {
             $rt = null;
             $pref = '';
//             $star = '';
             if (substr($filter->getName(),1,1)== ':'){
                 $rt = $query->repTables->get(substr($filter->getName(),0,1));
                 if ($rt != null){
                     if ($filter->isWindowFilter()) $metaRecord = $filter->getMetaRecord();
                     else $metaRecord = $rt->query->metaData[substr($filter->getName(),2)];
                     if ($filter->isValueFilter()) $pref= '* '.$rt->prefix.' ';
                     else $pref= $rt->prefix.' ';
                 }
             }else
                 if (substr($filter->getName(),1,1)== '='){
                     $rt = $query->repTables->get(substr($filter->getName(),0,1));
                     if ($rt != null){
                         if ($filter->isWindowFilter()) $metaRecord = $filter->getMetaRecord();
                         else $metaRecord = $rt->query->metaData[substr($filter->getName(),2)];
                         $pref= '* '.$rt->prefix.' ';                         
                     }
             }else{
                 if ($filter->isWindowFilter()) $metaRecord = $filter->getMetaRecord();
                 else $metaRecord = $query->getReportMetaRecord($filter->getName());
                 if ($filter->isValueFilter()) $pref = '* ';
             }
             if ($metaRecord == null) $label = "FEHLER: Keine Metadaten für ".$filter->getName();
             else $label = $metaRecord->label;
             ?>
            <div class="nowrap" >
            <?=$pref?><b <?=makeTitleClass(makeOutputFilterTitleDescription($metaRecord),'') ?>><?=$label?></b>
            <?php
            $vPos = $filterVector->findPos($filter->getName());
//            if ($vPos !== false && $filter->isSet()) enterFilterSpec('Form', $vPos, $filter->getName(), $filter->fs_von, $filter->fs_bis);
            if ($metaRecord != null)
//                if ($rt != null) $metaRecord->resetSelectOptions($rt->kennzeichen);
//            else $metaRecord->resetSelectOptions('base');
            if ($vPos !== false ) enterFilterSpec('Form', $vPos, $filter->getName(), $filter->fs_von, $filter->fs_bis);
            else enterFilterSpec('Output', $i,$filter->getName(), $filter->fs_von, $filter->fs_bis);
            ?>
            </div>
 <?php 
         }
     }
 
 ?>
 <input type="submit" value="<?=WSRTranslate::translate('Anzeigen')?>">
    </form>
    </div> 
<?php 
 }
/****
 * Ende Filter anzeigen und ergänzen.
 */

?>
<h1><?=$query->title?></h1>
<?php 
checkRequiredFilter();


/****
 * Fehlermeldungen anzeigen
 */
if (isValid($meldungen)) echo '<br>'.str_replace("\n", "<br>\n", $meldungen).'<br>';
$meldungen ='';
?>
<div class="article white">
<?php 
    
$startZeit = hrtime(true);
createHistory();
$sql = new SqlAbfrage('DWH');
createBasicQuery($sql,'base',-1, -1);
for ($i = 0; $i < $query->repTables->size(); $i++)
    if ($query->repTables->getPos($i)->joinType == JoinedTable::JOINSPALTEN){
        $k = $query->repTables->getPos($i)->kennzeichen;
        $sql->union();
        createBasicQuery($sql,$k,-1, -1);
        $sql->unionWeg();
    }
// if (isValid($meldungen)) echo '<br>'.str_replace("\n", "<br>\n", $meldungen).'<br>';
$meldungen ='';

// FEHLER Abfangen
if ($query->values_in_columns && $query->repColumns->size() < 1){
    echo '<br>'.WSRTranslate::translate('FEHLER').':</b> '.WSRTranslate::translate('Werte in Spalten ohne Spalten-Definition');
    echo"</body></html>";
    die();
}

/****
 * Beginn der DB-Abfrage
 */
if (!createTable('base', $sql))
    echo '<br><b>Base:</b>'.$meldungen.'<br>';
    else if (isValid($meldungen)){
        echo 'base:<br>'.str_replace("\n", "<br>\n", $meldungen);
        $meldungen = '';
    }
for ($i = 0; $i < $query->repTables->size(); $i++)
    if ($query->repTables->getPos($i)->joinType == JoinedTable::JOINOUTERWERTE
        || $query->repTables->getPos($i)->joinType == JoinedTable::JOINWERTE){
            if ($query->repTables->getPos($i)->query == NULL
                || $query->repTables->getPos($i)->query->phys_table == NULL){
                    $meldungen .="Fehler beim lesen der verknüpften Datenquelle ".$query->repTables->getPos($i)->query->class_table;
            }else{
    $k = $query->repTables->getPos($i)->kennzeichen;
    $meldungen ='';
    $sql = new SqlAbfrage('DWH');
    createBasicQuery($sql,$k,-1, -1);
    if (!createTable($k, $sql))
        echo '<br><b>'.$k.':</b><br>'.str_replace("\n", "<br>\n", $meldungen).'<br>';
    else if (isValid($meldungen)){
        echo $k.':<br>'.str_replace("\n", "<br>\n", $meldungen);
        $meldungen = '';
    }
            }
}

/****
 * Zeilen und Spalten aufbauen
 */
$meldungen ='';
$sql = new SqlAbfrage('DWH');
createLeadTable($sql);
if (! createTable('lead', $sql)){
    echo '<br><b>Output:</b>'. str_replace("\n", "<br>\n", $meldungen).'<br>'
                .'<pre>'.$sql->sql.'</pre><br>';
                echo '</body></html>';
                die();
}

$meldungen ='';
$sql = new SqlAbfrage('DWH');
createJoinedTable($sql);
if (!createTable('ergebnis', $sql))
    echo '<br><b>Ergebnis:</b><br>'.str_replace("\n", "<br>\n", $meldungen).'<br>';
else if (isValid($meldungen)){
    echo WSRTranslate::translate('Ergebnis').':<br>'.str_replace("\n", "<br>\n", $meldungen);
    $meldungen = '';
}

// Nur bei MySql-DB wird der Aufruf von createMysqlSorterTable benötigt
if ( array_key_exists('sortDir', $_REQUEST) 
    && $_REQUEST['sortDir'] != Sort::NONE
    && $query->sort_mode != SortOptions::NOSORTVAL
    && isset($mysql) ) {            
        if ($query->sort_mode == SortOptions::SORTSPECIAL || $query->sort_mode == SortOptions::SORTSPECIALSZ) {
            for ($s=0; $s < $query->repRows->size(); $s++) {
                if (!$query->repRows->getPos($s)->webLinkColumn){                    
                    $sql = new SqlAbfrage('DWH');
                    createMysqlSorterTable($sql,'ergebnis',$s);
                    if (! createTable('sorter'.$s, $sql))
                        echo '<br><b>Sorter-Table'.$s.':</b><br>'. str_replace("\n", "<br>\n", $meldungen).'<br>'
                                    .'<pre>'.$sql->sql.'</pre><br>';
                    else if (isValid($meldungen)){
                        echo 'Sorter Table'.$s.':<br>'.str_replace("\n", "<br>\n", $meldungen);
                        $meldungen = '';
                    }           
                }
            }
        }else{
            $sql = new SqlAbfrage('DWH');
            createMysqlSorterTable($sql,'ergebnis',0);
            if (! createTable('sorter', $sql))
                echo '<br><b>Sorter-Table:</b><br>'. str_replace("\n", "<br>\n", $meldungen).'<br>'
                        .'<pre>'.$sql->sql.'</pre><br>';
            else if (isValid($meldungen)){
                echo 'Sorter Table:<br>'.str_replace("\n", "<br>\n", $meldungen);
                $meldungen = '';
            }                        
        }
}
    
/****
 * Ergebnistabelle aufbauen
 */

$sql = new SqlAbfrage('DWH');
createSortedTable($sql,'ergebnis');
if (! createTable('output', $sql)){
    echo '<br><b>Output:</b>'. str_replace("\n", "<br>\n", $meldungen).'<br>'
                .'<pre>'.$sql->sql.'</pre><br>';
  echo '</body></html>';
  die();
}
else if (isValid($meldungen)){
    echo 'SortedTable:<br>'.str_replace("\n", "<br>\n", $meldungen);
    $meldungen = '';
}

                //    showTable('output');
if (false) $query = new Query("tab");
if (false) $mr = new MetaRecord();

/****
 * Struktur für TH-Elemente der Spalten ($repColumns)
 * @var array $colHeads
 */
$colHeads = array();
for ($c=0; $c < $query->repColumns->size(); $c++){
    $q = $query->repColumns->getPos($c);
//    echo "<pre>Col $c\n";
//    var_dump($q);
//    echo '</pre>';
    $mr = $query->getReportMetaRecord($q->column_name);
    $selVar = $mr->getSelName();
    $dispVar = $mr->getDispName();
    if (substr($q->column_name,1,1) == ':'){
        $selVar = substr($q->column_name,0,1).'__'.$selVar;
        $dispVar = substr($q->column_name,0,1).'__'.$dispVar;
    }
        
    $sql = "select distinct ".$dispVar.' as disp, '.$selVar.' as sel from WSR_output where colGroup = '.$c.' order by ';
    switch ($q->sort){
        case Sort::SORTINTERN: $sql .= $selVar;
        break;
        case Sort::SORTINTERNDESC: $sql .= $selVar.' desc';
        break;
        case Sort::SORTANZEIGE: $sql .= $dispVar;
        break;
        case Sort::SORTANZEIGEDESC: $sql .= $dispVar.' desc';
        break;
        default: $sql .= $selVar;
        break;
        
    }
    
    if ($dbDWH == null) setdbDWH();    
    if (false !== $stmt = $dbDWH->query($sql)){
        $colHeads[$c] = $stmt->fetchAll(PDO::FETCH_ASSOC);
        if (count($colHeads[$c]) == 0) {
            $colHeads[$c][0]['disp'] = '&nbsp;';
            $colHeads[$c][0]['sel'] = '';
        }
        $stmt->closeCursor();
    }
    else {
        echo "Bei Col-Liste $q->column_name:";
        var_dump($dbDWH->errorInfo());
        echo '<br>'.$sql;
    }
    for ($i=0; $i < count($colHeads[$c]);$i++) {
        $colHeads[$c][$i]['disp'] = formatWert($mr, $colHeads[$c][$i]['disp']);
    }
    unset($sql);
}
// echo "<pre>ColHeads\n";
// var_dump($colHeads);
// echo '</pre>';
if ($dbDWH == null) setdbDWH();
if (false !== $stmt = $dbDWH->query('select * from WSR_output'))
    $daten = $stmt->fetchAll(PDO::FETCH_ASSOC);
    else var_dump($dbDWH->errorInfo());
//    echo "<pre>Daten\n";
//    var_dump($daten);
//    echo '</pre>';

/****
 * Ausgabe in der HTML-Tabelle aufbauen
 */    
    
$startAusgabe = hrtime(true);
    
$rowSpans = array();  
// $lastRowSpan = array();
$chartValues = array();
?>
</div>
<?php 
/****
 * Überschriften
 * @var array[String] $filterTitel: Anzeige der Filter in den Überschriften
 */

$filterTitel = array();
for ($i=0; $i < $query->repFilter->size(); $i++){
    $filter =  $query->repFilter->getPos($i);
    if ($filter->inTitelZeile > 0){
        $titel = showFilter($filter,$filter->inTitelMitLabel);
        if (isValid($titel)) $filterTitel[$filter->inTitelZeile] = $titel.' '.showFilterValue($filter->getName());
    }
}

if ($query->repWerteAuswahl != NULL && 
    $query->repWerteAuswahlTitel > 0){
        if ($query->repWerteAuswahlLabel) $filterTitel[$query->repWerteAuswahlTitel] = 
            WSRTranslate::translate("Werte").": ".$werte;
        else  $filterTitel[$query->repWerteAuswahlTitel] = $werte;
}

if (array_key_exists(1, $filterTitel)) echo '<h2>'.quoteForHTML($filterTitel[1])."</h2>\n";
if (array_key_exists(2, $filterTitel)) echo '<h2>'.quoteForHTML($filterTitel[2])."</h2>\n";
if (array_key_exists(3, $filterTitel)) echo '<h3>'.quoteForHTML($filterTitel[3])."</h3>\n";
if (array_key_exists(4, $filterTitel)) echo '<h3>'.quoteForHTML($filterTitel[4])."</h3>\n";
if (array_key_exists(5, $filterTitel)) echo '<h3>'.quoteForHTML($filterTitel[5])."</h3>\n";

?>
<form name="DisplayForm"  method="PUT" action = "./outputReport.php" id="DisplayForm" >

<?php
/****
 * Request-Parameter übergeben.
 * Wird später für Sortierung gebraucht
 * $addFilter aus der Query müssen hier nicht wiederholt werden. 
 */

foreach ($_REQUEST as $p => $v){
  if ($p != 'sortVal'
      && $p != 'sortDir'
      && $p != 'sortCol'
      && $p != 'sortColVal'
      && substr($p, 0,4) != 'WSM_'
      ){
?>
	<input type="hidden" name="<?=$p?>" value="<?=$v?>">
 <?php 
      }
}
if ($aktiverEditor){
    ?>
	<input type="hidden" name="action" value="reportBearbeiten"/>
	<input type="hidden" name="id" value="<?=$_REQUEST['id']?>"/>
	<input type="hidden" name="table" value="<?=$_REQUEST['table']?>"/>
 <?php 
}else{
    $moves->outputParams();
}
?>
	<input type="hidden" name="sortVal" value="">
	<input type="hidden" name="sortDir" value="0">
	<input type="hidden" name="sortCol" value="">
	<input type="hidden" name="sortColVal" value="">
<table id="ReportTabelle" <?=$border?>> <!-- wegen potentieller Border für Spreadsheet -->
<thead><tr>
<?php 
// Erste Zeile in der Überschrift
for($c=0; $c < $query->repRows->size(); $c++){
    $class = '';
    if ($query->repRows->getPos($c)->fixedTableColumns || $query->fix_labelColumns){
        $class = 'class="fts"';
    }
    
    if ($query->repRows->getPos($c)->webLinkColumn){
        if (! $spreadsheet){
            $rr = $query->repRows->getPos($c);
            echo '<th rowspan="2" '.$class.' title="'.$rr->web_title.'">'.$rr->web_link_title."</th>\n";
        }
        $rowSpans[$c] = 0;
    }else{
        $mr = $query->getReportMetaRecord($query->repRows->getPos($c)->getName());
        if ($mr == null) echo '<th rowspan="2">Label für '.$query->repRows->getPos($c)->getName()." fehlt.</th>\n";
        else {
            if ($query->repRows->getPos($c)->fixedTableColumns || $query->fix_labelColumns){
                $class = 'class="fts draggable ';
            }else{
                $class = 'class="draggable ';
            }
            $wpos = $c+1;
            while ($wpos < $query->repRows->size() && $query->repRows->getPos($wpos)->webLinkColumn) $wpos++;
            if ($query->repRows->size() > $wpos &&
                $query->repRows->getPos($wpos)->summenZeilen) $class.=' WSFSummen';
                if ($query->repRows->size() == $wpos &&
                    $query->values_in_columns) $class.=' '.$query->valueColumnFolderMode;
            $class .= '" ';
            
            echo '<th rowspan="2"  '.$class.makeTitle($mr->description);
            if (!$spreadsheet){
                if ($aktiverEditor) dragNDdrop("'Zeilen'", $c, 'reportDrop');
                else dragNDdrop("'Zeilen'", $c, 'outputDrop');
            }
            echo '>'.$mr->label."</th>\n";
        }
        $rowSpans[$c] = 0;
    }
}

if ($query->values_in_columns) echo '<th rowspan="2" class="fts">'."&nbsp;</th>\n";
else   // Normale Ausgabe der Header mit Werten in einzelnen Spalten
// Erste Zeile Value-Labels    
for($c=0; $c < $query->repValues->size(); $c++){
    $hatTitel[$c] = false;
    /**
     * @var RepValue $v
     */
    $v = $query->repValues->getPos($c);
    $vn = $v->getValueName();
    $mr = $query->getReportMetaRecord($vn);
    if ($mr == null) echo '<th rowspan="2">Label für '.$vn." fehlt.</th>\n";
    else {
        if (substr($vn,1,1) ==':' || substr($vn,1,1) =='='
            || (isValid($v->label) && (strpos($v->label,':') !== false))){
                if (isValid($v->label) && (strpos($v->label,':') !== false)){
                    if (strpos($v->label,':') > 0){
                        $cTitel = substr($v->label,0,strpos($v->label,':'));
                        $hatTitel[$c] = true;
                        $colSpan = 0;
                        for ($cs = 1; $colSpan < 1; $cs++){
                            if (($c+$cs) >= $query->repValues->size()) $colSpan = $cs;
                            else {
                                $vls = $query->repValues->getPos($c+$cs)->label;
                                if (!isValid($vls)) $colSpan = $cs;
                                else if ($cTitel != substr($vls,0,strpos($vls,':'))) $colSpan = $cs;
                                else $hatTitel[$c + $cs] = true;                                
                            }
                        }
                        echo "<th colspan=\"$colSpan\" class=\"rightBorder\">".$cTitel."</th>\n";
                        $c += $cs-2;                   
                    } // if strpos > 0                   
                }else {
                    $jt = $query->repTables->get(substr($vn,0,1));
                    // Entweder Prefix aus $jt oder Titel ais repValue->label vor ':'
                    if ($jt != null && isValid($jt->prefix)){
                        $hatTitel[$c] = true;
                        $colSpan = 0;
                        for ($cs = 1; $colSpan < 1; $cs++){
                            if (($c+$cs) >= $query->repValues->size()) $colSpan = $cs;
                            else {
                                $vns = $query->repValues->getPos($c+$cs)->getName();
                                if (substr($vns,1,1) != ':' && substr($vns,1,1) != '=')  $colSpan = $cs;
                                if (substr($vns,0,1) != substr($vn,0,1))  $colSpan = $cs;
                                else $hatTitel[$c + $cs] = true;
                            }
                        }
                        echo "<th colspan=\"$colSpan\"  class=\"rightBorder\">".$jt->prefix."</th>\n";
                        $c += $cs-2;
                    } // if (isValid($jt->prefix)
            } // else jt
        } // if substring in ':','=' or : in $v->label
        
        if (! $hatTitel[$c]){
            if ($v->isWindowValue()) $label = $v->getFunctionText($v->function).'<br>'.$mr->label;
            else $label = $mr->label;    
            if ($query->repValues->getPos($c)->label != null) {
                $label = $query->repValues->getPos($c)->label;
                if (strpos($label,':') === 0) $label = substr($label,1);
            }
            if ($query->sort_mode > SortOptions::NOSORTVAL 
                && ($query->folderMode == 0 || $query->sort_mode != SortOptions::SORTTABLE)
                && ! $spreadsheet){
                $sortIcon = "sort.gif";
                $nextSort = Sort::AUF;
                if ($query->sort_direction != 0
                    && $query->sort_value == $vn
                    && $query->sort_colGroup == null){
                        if ($query->sort_direction == Sort::AB){
                            $nextSort = Sort::NONE;
                            $sortIcon = 'ab.gif';
                        }else{
                            $nextSort = Sort::AB;
                            $sortIcon = 'auf.gif';                            
                        }
                }
                
                if (! $v->isWindowValue())
                    $label.='&nbsp;<img src="./icons/'.$sortIcon.'" onclick="sortiere(\''.$vn."','".$nextSort
                    ."')\" class=\"sortIcon\"  />";
                //height=\"10px\" width=\"10px\" 
            }
            echo '<th rowspan="2" class="draggable" ';
            if ( !($v->backgroundColor == "white" || $v->backgroundColor == '#ffffff'))
                echo " style=\"background-color: ".$v->backgroundColor."\" ";
                
            if (!$spreadsheet){
                if ($aktiverEditor) dragNDdrop("'Werte'", $c, 'reportDrop');
                else dragNDdrop("'Werte'", $c, 'outputDrop');
            }
            
            if ($v->isWindowValue() && isValid(RepValue::getFunctionDesc($v->function))){
                if ($v->abSpalte > 0){
                    $ebene = ' in '.$query->getReportMetaRecord($query->repRows->getPos($v->abSpalte -1)->getName())->label;
                }
                else $ebene = '';
                if ($query->repValues->getPos($c)->label != null)
                    echo makeTitle($mr->label.":\n".RepValue::getFunctionDesc($v->function)
                        .$ebene
                        .":\n".$mr->description).'>'.$label."</th>\n";
                else
                    echo makeTitle(RepValue::getFunctionDesc($v->function)
                        .$ebene
                        .":\n".$mr->description).'>'.$label."</th>\n";
                        
            }else{
                if ($query->repValues->getPos($c)->label != null)
                    echo makeTitle($mr->label.":\n".$mr->description).'>'.$label."</th>\n";
                else
                    echo makeTitle($mr->description).'>'.$label."</th>\n";
                        
            }
        }
    }
} // ENDE Normale Ausgabe der Header mit Werten in einzelnen Spalten
// Ausgabe der Header für Spalten
for($c=0; $c < $query->repColumns->size(); $c++){
   /**
    * 
    * @var RepColumn $rc
    */
    $rc = $query->repColumns->getPos($c);
    if (isValid($rc->label)) {
        echo '<th class="draggable rightBorder" colspan="'.count($colHeads[$c]).'"';
        if (!$spreadsheet){
            if ($aktiverEditor) dragNDdrop("'Spalten'", $c, 'reportDrop');
            else dragNDdrop("'Spalten'", $c, 'outputDrop');
            if ($rc->function > 0) {
                $title = RepValue::getFunctionDesc($rc->function);
                if ($rc->abSpalte > 0) $title .= ' '.WSRTranslate::translate('in').' '
                    .$query->getReportMetaRecord($query->repRows->getPos($rc->abSpalte-1)->getName())->label;
                if ($rc->nachSpalten) $title .= ' '.WSRTranslate::translate('in Spalten');
                $title.=":\n";
            }else $title='';
            $mrv = $query->getReportMetaRecord($rc->val_name);
            if ($mrv != null){
                if (isValid($mrv->description)) $title.= $mrv->description;
                else $title .= $mrv->label;                
            }else $title .= 'Metadaten für '.$rc->val_name.' fehlen.';
            $title.="\nnach:\n";
            $mrv = $query->getReportMetaRecord($rc->getName());
            if ($mrv != null){
                if (isValid($mrv->description)) $title.= $mrv->description;
                else $title .= $mrv->label;
            }else $title .= 'Metadaten für '.$rc->getInsertArray().' fehlen.';
            echo makeTitle($title);
        }        
        echo '>'.$rc->label."</th>\n";
    }
    else {
        $mr = $query->getReportMetaRecord($rc->getName());
        if ($mr == null) echo '<th rowspan="2">Label für '.$rc->getName()." fehlt.</th>\n";
        else
            for($cv=0; $cv < count($colHeads[$c]); $cv++){
                $label =$colHeads[$c][$cv]['disp'];
                // Web-Link für normale Spalten
                if (isValid($rc->web_link) && ! $rc->reportLinkInValues){
                    if (substr($rc->web_link,0,1) == '#'){
                        // An dieser Stelle kann es keinen Link aus den Daten geben
                        // Die Angabe wird ignoriert
                        }
                        else{
                            $cell ='<a title="Aufruf des Reports '.quoteForText($rc->web_link).'" href="./outputReport.php?report='.$rc->web_link;
                            $cell .= makeGetFilter($rc->reportCopyFilter,null,null,$rc,$colHeads[$c][$cv]['sel']);
                            $cell.='">';
                            $label = $cell.$label.'</a>';
                        }
                }    
                // ENDE Web-Link für Spalten-Köpfe
                
                if ($query->sort_mode > SortOptions::NOSORTVAL
                    && isValid($rc->val_name)
                    && ($query->folderMode == 0 || $query->sort_mode != SortOptions::SORTTABLE)
                    && $rc->function <= 0
                    && ! $spreadsheet){
                    $sortIcon = "sort.gif";
                    $nextSort = Sort::AUF;
                    if ($query->sort_direction != 0
                        && $query->sort_colGroup != null
                        && $query->sort_colValue == $colHeads[$c][$cv]['sel']
                        ){
                            if ($query->sort_direction == Sort::AB){
                                $nextSort = Sort::NONE;
                                $sortIcon = 'ab.gif';
                            }else{
                                $nextSort = Sort::AB;
                                $sortIcon = 'auf.gif';
                            }
                    }
                    // FEHLER, weil hier der display-Wert statt dem Select-Wert steht!!
                    $label.='&nbsp;<img src="./icons/'.$sortIcon.'" onclick="sortiereColumn(\''.$c
                    ."','".$colHeads[$c][$cv]['sel']."','".$nextSort
                    ."')\"  class=\"sortIcon\"  />";
                }
                //height=\"10px\" width=\"10px\"
                
                echo '<th rowspan="2" ';
                //                if (!$spreadsheet && $aktiverEditor)  dragNDdrop("'Spalten'", $c, 'reportDrop');                
                echo '>'.$label."</th>\n";
        }
    }
}
echo "</tr><tr>\n";
// Zweite Zeile in der Überschrift für normale Spalten
if (! $query->values_in_columns) 
    for($c=0; $c < $query->repValues->size(); $c++){
        /**
         * @var RepValue v
         */
        $v = $query->repValues->getPos($c);
        if (false) $v = new RepValue('');
        $vn = $v->getValueName();
        $mr = $query->getReportMetaRecord($vn);
        if ($mr != null){
            if ($hatTitel[$c]){
                if ( isValid($v->label)){
                    $label = $v->label;
                    if (strpos($label,':') !== false){
                        $stpos = strpos($label, ':');
                        $label = substr($label, $stpos + 1);
                    }   
                }else{
                    if ($v->isWindowValue()) $label = $v->getFunctionText($v->function).'<br>'.$mr->label;
                    else     $label = $mr->label;
                }
                if ($query->sort_mode > SortOptions::NOSORTVAL
                    && ($query->folderMode == 0 || $query->sort_mode != SortOptions::SORTTABLE)
                    && ! $spreadsheet){
                    $sortIcon = "sort.gif";
                    $nextSort = Sort::AUF;
                    if ($query->sort_direction != 0
                        && $query->sort_value == $vn
                        && $query->sort_colGroup == null){
                            if ($query->sort_direction == Sort::AB){
                                $nextSort = Sort::NONE;
                                $sortIcon = 'ab.gif';
                            }else{
                                $nextSort = Sort::AB;
                                $sortIcon = 'auf.gif';
                            }
                    }
                    if (! $v->isWindowValue())
                        $label.='&nbsp;<img src="./icons/'.$sortIcon.'" onclick="sortiere(\''.$vn."','".$nextSort
                        ."')\"  class=\"sortIcon\"  />";
                    //height=\"10px\" width=\"10px\"
                }
                if (!isValid($label)) $label = "&nbsp;";
                echo "<th class=\"draggable\" ";
                if ( !($v->backgroundColor == "white" || $v->backgroundColor == '#ffffff'))
                    echo " style=\"background-color: ".$v->backgroundColor."\" ";
                    
                    
                // echo makeTitle($description);
                
                if (!$spreadsheet){
                    if ($aktiverEditor) dragNDdrop("'Werte'", $c, 'reportDrop');
                    else dragNDdrop("'Werte'", $c, 'outputDrop');
                    if ($v->isWindowValue() && isValid(RepValue::getFunctionDesc($v->function))){
                        if ($v->abSpalte > 0){
                            $ebene = ' in '.$query->getReportMetaRecord($query->repRows->getPos($v->abSpalte -1)->getName())->label;
                        }
                        else $ebene = '';
                        if ($query->repValues->getPos($c)->label != null)
                            echo makeTitle($mr->label.":\n".RepValue::getFunctionDesc($v->function)
                                .$ebene
                                .":\n".$mr->description);
                                else
                                    echo makeTitle(RepValue::getFunctionDesc($v->function)
                                        .$ebene
                                        .":\n".$mr->description);
                                        
                    }else{
                        if ($query->repValues->getPos($c)->label != null)
                            echo makeTitle($mr->label.":\n".$mr->description);
                            else
                                echo makeTitle($mr->description);
                                
                    }
                    
                }
                
                echo ">".$label."</th>\n";
            }  // if $hatTitel[$c]
        } // Ende if $mr != null        
} // Ende for
// ENDE Zweite Zeile in der Überschrift 
for($c=0; $c < $query->repColumns->size(); $c++){
    /**
     * @var RepColumn $rc
     */
    $rc = $query->repColumns->getPos($c);
    if (isValid($rc->label)) {
        $mr = $query->getReportMetaRecord($rc->getName());
            for($cv=0; $cv < count($colHeads[$c]); $cv++){
                $label =$colHeads[$c][$cv]['disp'];
//                echo $rc->web_link .' && '. (!$rc->reportLinkInValues).'<br>';
                if (isValid($rc->web_link) && ! $rc->reportLinkInValues){
                    if (substr($rc->web_link,0,1) == '#'){
                        // An dieser Stelle sind die Daten für den Link
                        // ebenfalls nicht vorhanden. Angabe wird ignoriert.
                    }
                else{ // Link auf Report
//$rc = new RepColumn();                                        
                    if (isValid($rc->web_title)) $wtt = ' title="'.$rc->web_title.'" ';
                    else $wtt = 'title="Aufruf des Reports '.$rc->web_link.'" ';
                    if (isValid($rc->web_target)) $wtt .= ' target ="'.quoteForText($rc->web_target).'" ';
                    $cell ='<a '.$wtt.' href="./outputReport.php?report='.$rc->web_link;
                    $zeile = null;
                    $cell .= makeGetFilter($rc->reportCopyFilter,$zeile,null,$rc,$colHeads[$c][$cv]['sel']);
                    $cell.='">';
                    $label = $cell.$label.'</a>';
                    }
                }
                if ($query->sort_mode > SortOptions::NOSORTVAL
                    && ($query->folderMode == 0 || $query->sort_mode != SortOptions::SORTTABLE)
                    && isValid($rc->val_name)
                    && ! $spreadsheet){
                        $sortIcon = "sort.gif";
                        $nextSort = Sort::AUF;
                        if ($query->sort_direction != 0
                            && $query->sort_colGroup != null
                            && $query->sort_colValue == $colHeads[$c][$cv]['sel']
                            ){
                                if ($query->sort_direction == Sort::AB){
                                    $nextSort = Sort::NONE;
                                    $sortIcon = 'ab.gif';
                                }else{
                                    $nextSort = Sort::AB;
                                    $sortIcon = 'auf.gif';
                                }
                        }
                        if ($rc->function <= 0)
                            $label.='&nbsp;<img src="./icons/'.$sortIcon.'" onclick="sortiereColumn(\''.$c
                            ."','".$colHeads[$c][$cv]['sel']."','".$nextSort
                            ."')\"  class=\"sortIcon\"  />";
                        // height=\"10px\" width=\"10px\"
                }
                if (!isValid($label)) $label = "&nbsp;";
                echo '<th>'.$label."</th>\n";                
            }
    }
}
?>
</tr></thead>
<tbody>
<?php
$hasTfoot = false;
/****
 * Beginn der Datenausgabe in der HTML-Tabelle
 */
// DROP Summen funktioniert nicht, wenn die Daten der Joined Query nur in der Summenzeile ausgegeben werden.
// Deshalb nur noch, wenn nur eine Datenquelle verwendet wird?
$canDropSummen = true;
for ($i=0; $i < $query->repFilter->size();$i++){
    $rf = $query->repFilter->getPos($i);
    if (false) $rf = new RepFilter('');
    if ($rf->isValueFilter()) $canDropSummen = false;
    if ($rf->isWindowFilter()) $canDropSummen = false;
    
}

if (!$query->dropSummen){
for ($i=0; $i < $query->repTables->size(); $i++)
    if ($query->class_table != $query->repTables->getPos($i)->query->class_table) $canDropSummen = false;
}
    
$colData = array();
for ($znr = 0; $znr < count($daten); $znr++){
    $zeile = $daten[$znr];
    
// Spalten der colGroups füllen    
    $zeileAusgeben = false;
    if ($zeile['colGroup'] == 99) $zeileAusgeben = true;
    // Vorbereitung für values_in_columns
    else if ($query->values_in_columns) {
        $rc = $query->repColumns->getPos($zeile['colGroup']);
        $vn = $rc->getName();
        $mr = $query->getReportMetaRecord($vn);
        if (! array_key_exists($zeile['colGroup'], $colData)) $colData[$zeile['colGroup']] = array();
        $vc = $mr->getSelName();
        if (substr($vn,1,1) == ':' || substr($vn,1,1) == '=' ) $vc = substr($vn,0,1).'__'.$vc;
        if (! array_key_exists($zeile[$vc], $colData[$zeile['colGroup']]))            
            $colData[$zeile['colGroup']][$zeile[$vc]] = array();
        
        for ($iv=0; $iv < $query->repValues->size(); $iv++){
            /**
             * @var RepValue $val
             */
            $val = $query->repValues->getPos($iv);
            $mr = $query->getReportMetaRecord($val->getValueName());
            if ($val->isWindowValue()) $vv = $val->getName();
            else $vv = $mr->getName();
            if (substr($val->getName(),1,1) == ':' || substr($val->getName(),1,1) == '=' ) 
                $vv = substr($val->getName(),0,1).'__'.$vv;
            
            $im = 1;
            while ($query->repRows->getPos($query->repRows->size()-$im) != null
                && $query->repRows->getPos($query->repRows->size()-$im)->webLinkColumn) {
                $im ++;
            }
                
            if ($zeile[$vv] == null) {
                $colData[$zeile['colGroup']][$zeile[$vc]][$val->getName()] = '&nbsp;';
            }else
            if (! $spreadsheet 
                && $val->dispVal != DisplayOptions::ZAHL
                &&( $query->display_charts ==  ChartOptions::IMMER
                    || ($query->repRows->size() == 0)
                    || ($query->display_charts ==  ChartOptions::NURSUMMEN
                        && $zeile['S'.($query->repRows->size()-$im)] == 1 )
                    || ($query->display_charts ==  ChartOptions::KEINESUMMEN
                        && $zeile['S'.($query->repRows->size()-$im)] == 0 )
                    )
                    ){
                $id = 'R'.$znr.'CG'.$zeile['colGroup'].'C'.$zeile[$vc].'V'.$val->getName();
                switch ($val->dispVal){
                    case DisplayOptions::ZAHLHBAR:
                        $id = 'H'.$id;
                        $colData[$zeile['colGroup']][$zeile[$vc]][$val->getName()] = '<div class="'.$mr->align.' nowrap">';
                        if ($val->isWindowValue() && $val->hasFunctionFormatDecs() && $zeile[$vv] != null)
                            $colData[$zeile['colGroup']][$zeile[$vc]][$val->getName()] .= 
                                    WSRFormats::formatDecimal($zeile[$vv],$val->getFunctionFormatDecs());
                        else
                            $colData[$zeile['colGroup']][$zeile[$vc]][$val->getName()] .= formatWert($mr, $zeile[$vv]);                                        
                        $colData[$zeile['colGroup']][$zeile[$vc]][$val->getName()] .= '&nbsp;<canvas id="'.$id.'"></canvas></div>';
                        break;
                    case DisplayOptions::ZAHLVBAR:
                        $id = 'V'.$id;
                        $colData[$zeile['colGroup']][$zeile[$vc]][$val->getName()] =
                            '<canvas class="center" id="'.$id.'"></canvas><br>';
                        if ($val->isWindowValue() && $val->hasFunctionFormatDecs() && $zeile[$vv] != null)
                            $colData[$zeile['colGroup']][$zeile[$vc]][$val->getName()] .=
                                    WSRFormats::formatDecimal($zeile[$vv],$val->getFunctionFormatDecs());
                        else
                            $colData[$zeile['colGroup']][$zeile[$vc]][$val->getName()] .= formatWert($mr, $zeile[$vv]);
                        break;
                    case DisplayOptions::HBAR:
                        $id = 'H'.$id;
                        $colData[$zeile['colGroup']][$zeile[$vc]][$val->getName()] = 
                            '<canvas class="'.$mr->align.'" id="'.$id.'"></canvas>';
                        break;
                    case DisplayOptions::VBAR:
                        $id = 'V'.$id;
                        $colData[$zeile['colGroup']][$zeile[$vc]][$val->getName()] = 
                            '<canvas class="center" id="'.$id.'"></canvas>';
                        break;
                }
                switch($val->skalierung){
                    case ChartOptions::SKALIERETABELLE:
                        setChart($val->getName(),$mr,$id,$zeile[$vv],'_TABELLE_');
                        break;
                    case ChartOptions::SKALIEREZEILE:
                        setChart($val->getName(),$mr,$id,$zeile[$vv],'_ZEILE_');
                        break;
                    case ChartOptions::SKALIEREWERT:
                        setChart($val->getName(),$mr,$id,$zeile[$vv],$val->getName());
                        break;
                    case ChartOptions::SKALIERESPALTE:
                        setChart($val->getName(),$mr,$id,$zeile[$vv],$vn);
                        break;
                    default:
                        setChart($val->getName(),$mr,$id,$zeile[$vv],$mr->chartGroup);
                        break;
                }
            }
            else {
                if ($val->isWindowValue() && $val->hasFunctionFormatDecs() && $zeile[$vv] != null)
                    $colData[$zeile['colGroup']][$zeile[$vc]][$val->getName()] = WSRFormats::formatDecimal($zeile[$vv],$val->getFunctionFormatDecs());
                else     
                    $colData[$zeile['colGroup']][$zeile[$vc]][$val->getName()] = formatWert($mr, $zeile[$vv]);
            }
        }
        if ($znr +1 >= count($daten)) $zeileAusgeben = true;
        else {
            $naechsteZeile = $daten[$znr+1];
            for ($r = 0; $r < $query->repRows->size(); $r ++){
                if (!$query->repRows->getPos($r)->webLinkColumn){
                    if ($zeile['S'.$r] != $naechsteZeile['S'.$r]) $zeileAusgeben = true;
                    else{
                        $vn = $query->repRows->getPos($r)->getName();
                        $vd = $query->getReportMetaRecord($vn)->getSelName();
                        if (substr($vn,1,1) == ':' || substr($vn,1,1) == '=' ) $vd = substr($vn,0,1).'__'.$vd;
                        if ($zeile[$vd] != $naechsteZeile[$vd]) $zeileAusgeben = true;
                    }
                }
            }
        }
//        if ($zeileAusgeben){
//            echo '<pre>';
//            var_dump($colHeads);
//            var_dump($colData);
//            var_dump($daten);
//            echo '</pre>';
//        }
    }
    else{
        // Spalten füllen
        $rc = $query->repColumns->getPos($zeile['colGroup']);
        $vn = $rc->getName();
        $mr = $query->getReportMetaRecord($vn);
        if (! array_key_exists($zeile['colGroup'], $colData)) $colData[$zeile['colGroup']] = array();
        $vc = $mr->getSelName();
        if (substr($vn,1,1) == ':' || substr($vn,1,1) == '=' ) $vc = substr($vn,0,1).'__'.$vc;        
        $mr = $query->getReportMetaRecord($rc->val_name); 
        if ($mr == null){
            error_log('Fehler bei Spalte '.$vn.'  valName='.$rc->val_name.'  vc = '.$vc);
            $colData[$zeile['colGroup']][$zeile[$vc]] = '&nbsp; MR ist NULL'.$rc->val_name;
        }else{
            if ($rc->function > 0) $vv = 'WSR_WINDOWCOLVALUE'.$zeile['colGroup'];
            else{
                $vv = $mr->getDispName();
                if (substr($rc->val_name, 1,1) == ':') $vv = substr($rc->val_name, 0,1).'__'.$vv;
                else
                if (substr($vn,1,1) == ':' || substr($vn,1,1) == '=' ) $vv = substr($vn,0,1).'__'.$vv;
            }
            if ($zeile[$vv] == null) {
                $colData[$zeile['colGroup']][$zeile[$vc]] = '&nbsp;';
            }else
                if (! $spreadsheet
                    && $rc->displayOption != DisplayOptions::ZAHL
                    &&( $query->display_charts ==  ChartOptions::IMMER
                        || ($query->display_charts ==  ChartOptions::NURSUMMEN
                            && $zeile['S'.($query->repRows->size()-1)] == 1 )
                        || ($query->display_charts ==  ChartOptions::KEINESUMMEN
                            && $zeile['S'.($query->repRows->size()-1)] == 0 )
                        )
                    ){
                $id = 'R'.$znr.'CG'.$zeile['colGroup'].'C'.$zeile[$vc];
                switch ($rc->displayOption){
                    case DisplayOptions::ZAHLHBAR:
                        $id = 'H'.$id;
                        $colData[$zeile['colGroup']][$zeile[$vc]] = '<div class="'.$mr->align.' nowrap">';
                        if ($rc->function > 0 && $rc->hasFunctionFormatDecs() && $zeile[$vv] != null)
                            $colData[$zeile['colGroup']][$zeile[$vc]] .= WSRFormats::formatDecimal($zeile[$vv],$rc->getFunctionFormatDecs());
                        else
                            $colData[$zeile['colGroup']][$zeile[$vc]] .= formatWert($mr, $zeile[$vv]);
                        $colData[$zeile['colGroup']][$zeile[$vc]] .= '&nbsp;<canvas id="'.$id.'"></canvas></div>';
                        break;
                    case DisplayOptions::ZAHLVBAR:
                        $id = 'V'.$id;
                        $colData[$zeile['colGroup']][$zeile[$vc]] =
                        '<canvas class="center" id="'.$id.'"></canvas><br>';
                        if ($rc->function > 0 && $rc->hasFunctionFormatDecs() && $zeile[$vv] != null)
                            $colData[$zeile['colGroup']][$zeile[$vc]] .= WSRFormats::formatDecimal($zeile[$vv],$rc->getFunctionFormatDecs());
                        else 
                            $colData[$zeile['colGroup']][$zeile[$vc]] .= formatWert($mr, $zeile[$vv]);
                        break;
                    case DisplayOptions::HBAR:
                        $id = 'H'.$id;
                        $colData[$zeile['colGroup']][$zeile[$vc]] =
                        '<canvas class="'.$mr->align.'" id="'.$id.'"></canvas>';
                        break;
                    case DisplayOptions::VBAR:
                        $id = 'V'.$id;
                        $colData[$zeile['colGroup']][$zeile[$vc]] =
                        '<canvas class="center" id="'.$id.'"></canvas>';
                        break;
                }
                switch($rc->skalierung){
                    case ChartOptions::SKALIERETABELLE:
                        setChart($rc->val_name,$mr,$id,$zeile[$vv],'_TABELLE_');
                        break;
                    case ChartOptions::SKALIEREZEILE:
                        setChart($rc->val_name,$mr,$id,$zeile[$vv],'_ZEILE_');
                        break;
                    case ChartOptions::SKALIEREWERT:
                        setChart($rc->val_name,$mr,$id,$zeile[$vv],$rc->val_name);
                        break;
                    case ChartOptions::SKALIERESPALTE:
                        setChart($rc->val_name,$mr,$id,$zeile[$vv],$vn);
                        break;
                    default:
                        setChart($rc->val_name,$mr,$id,$zeile[$vv],$mr->chartGroup);
                        break;
                }
            }
            else 
                if ($rc->function > 0 && $rc->hasFunctionFormatDecs() && $zeile[$vv] != null)
                    $colData[$zeile['colGroup']][$zeile[$vc]] = WSRFormats::formatDecimal($zeile[$vv],$rc->getFunctionFormatDecs());
                    else
                        
                $colData[$zeile['colGroup']][$zeile[$vc]] = formatWert($mr, $zeile[$vv]);
        }
        if ($znr +1 >= count($daten)) $zeileAusgeben = true;
        else {
            $naechsteZeile = $daten[$znr+1];
            for ($r = 0; $r < $query->repRows->size(); $r ++){
                if (! $query->repRows->getPos($r)->webLinkColumn){
                    if ($zeile['S'.$r] != $naechsteZeile['S'.$r]) $zeileAusgeben = true;
                    else{
                        $vn = $query->repRows->getPos($r)->getName();
                        $vd = $query->getReportMetaRecord($vn)->getSelName();
                        if (substr($vn,1,1) == ':' || substr($vn,1,1) == '=' ) $vd = substr($vn,0,1).'__'.$vd;
                        if ($zeile[$vd] != $naechsteZeile[$vd]) $zeileAusgeben = true;                    
                    }
                }
            }            
        }
    }
/**
 * Report-Zeile ausgeben
 */    
    if ($zeileAusgeben ){
        /**
         * ggf. Chart-Values aus _ZEILE_ in ZNRzeilennummer kopieren und
         * _ZEILE_ freigeben
         */
        if (array_key_exists('_ZEILE_', $chartValues)){
            $chartValues['ZNR'.$znr] = $chartValues['_ZEILE_'];
            unset($chartValues['_ZEILE_']);
        }
    /**
     * Behandlung von Summenzeilen
     * Zu Beginn feststellen, ob die Summenzeile angezeigt werden soll.
     */    
    if ($query->values_in_columns || $zeile['colGroup']== 99){
        $dropSummen = false;
        if ($canDropSummen && $znr > 0){
            $cocount = 0;
            for($c=0; $c < $query->repRows->size();  $c++){
                if (array_key_exists('S'.$c, $zeile) 
                    && $zeile['S'.$c] == 1
                    ){
                    $dropSummen = true;                 
                    $vn = $query->getReportMetaRecord($query->repRows->getPos($c)->getName())->getDispName();
//                    $vs = $zeile['Stufe'];
                    $vz = $znr -1;
                    
                    if ($vz >= 0 && $daten[$vz]['Stufe'] == $zeile['Stufe']
                        && $daten[$vz]['S'.$c] == '1') {
                            $dropSummen = false;
                    }else
                    while ($vz > 0 && $daten[$vz]['Stufe'] == $zeile['Stufe']) $vz--;
                    $vval = $daten[$vz][$vn];                           
                    for ($z = $vz; $z >= 0; $z--){
                        $vzeile = $daten[$z];
                        if (array_key_exists('S'.$c, $vzeile) && $vzeile['S'.$c] == 1) {
                            if ($c == 0){
                               $dropSummen = false;
                            }else      
                            // Hier wird die Stufe der Summenzeile kontrolliert.
                            // Wenn aber Stufe+1 existiert und keine Summenzeile hat, sollte auch (!!!) 
                            // diese Gruppe kontrolliert werden.
                            if ($c+1 < $query->repRows->size()
                                && ! $query->repRows->getPos($c+1)->summenZeilen
                                && $z < $vz-2){
                                $dropSummen = false;
                            }else{
                                $z = -1;
                            }
                        }
                        else{
                            if ($vzeile[$vn] != $vval){
                                $dropSummen = false;
                                $z = -1;
                            }else if ($vzeile['colGroup']== 99) $cocount ++;
                            
                        }
                    }
                    if ($z < 0 && $c+1 < $query->repRows->size()
                        && ! $query->repRows->getPos($c+1)->summenZeilen
                        ){
                            $dropSummen = false;
                    }
                    
                    /**
                     * Summenzeile anzeigen, wenn ein Filter ab dieser Stufe wirkt.
                     */
                    if ($dropSummen){
                        for ($f = 0; $f < $query->repFilter->size() && $dropSummen; $f++){
                            if ($query->repFilter->getPos($f)->abSpalte-1 == $c) $dropSummen = false;
                        }                        
                    }
                    $c = $query->repRows->size();
                }
            }
        } // Ende canDropSummen
        $summenzeile = true;
        
        $vz = $znr -1;
        if ($vz < 0) $vz = 0;
        
        if ($dropSummen)  echo '<tr class="drop">';        
        else if ($zeile['S0'] == 1 && $query->repRows->size() > 0) {            
            if ($query->fix_labelColumns && ! $spreadsheet){
                $hasTfoot = true;
                echo'</tbody><tfoot>';
            }
            echo '<tr class="summen0">'; 
        }
        else if (array_key_exists('S1', $zeile) && $zeile['S1'] == 1
            && ! ($daten[$vz]['S1'] == 1 && $daten[$vz]['Stufe'] == $zeile['Stufe'] && $daten[$vz]['colGroup'] == 99) ) echo '<tr class="summen1">';
        else if (array_key_exists('S2', $zeile) && $zeile['S2'] == 1
            && ! ($daten[$vz]['S2'] == 1 && $daten[$vz]['Stufe'] == $daten[$znr]['Stufe'] && $daten[$vz]['colGroup'] == 99) ) echo '<tr class="summen2">';            
        else if (array_key_exists('S3', $zeile) && $zeile['S3'] == 1
            && ! ($daten[$vz]['S3'] == 1 && $daten[$vz]['Stufe'] == $zeile['Stufe']  && $daten[$vz]['colGroup'] == 99) ) echo '<tr class="summen3">';
        else {
            echo '<tr>';
            $summenzeile = false;
        }
        
    // Zeilen ausgeben    
    for($c=0; $c < $query->repRows->size(); $c++){           
        if ($query->repRows->getPos($c)->webLinkColumn){ // Web-Link-Spalte
            $cc = $c;
            while ($cc > 0 && $query->repRows->getPos($cc)->webLinkColumn) $cc--;
            
            if ($zeile['S'.$cc] == 0){
                if ($rowSpans[$c] == 0){
                $rowSpans[$c] = $rowSpans[$cc]+1; 
                if ($query->values_in_columns)
                { 
                       echo createRowCell($zeile, $query->repRows->getPos($c),
                        $rowSpans[$c] * $query->repValues->size());
                }
                 
                    else
                        echo createRowCell($zeile, $query->repRows->getPos($c), $rowSpans[$c]);
                }  
                $rowSpans[$c] --;
            }
        }else  // ENDE webLinkColumn
        if ($zeile['S'.$c] == 0){ // Normale Zeile
            if ($rowSpans[$c] > 0) $rowSpans[$c] --;
            else{
                if ($zeile['S'.$c] == 1) $rowSpans[$c] = 1;
                else {
                    $r = 0;
                    $mr = $query->getReportMetaRecord($query->repRows->getPos($c)->getName());
                    $vn = $mr->getSelName();
                    $val = $zeile[$vn];
                    if ($c == 0) $maxR = count($daten) - $znr;
                    else $maxR = $rowSpans[$c-1] +1;
                    $rs = 1;
                    if ($query->values_in_columns) {
                        $im = 1;
                        while ($query->repRows->getPos($query->repRows->size()-$im)->webLinkColumn) {
                            $im ++;
                        }
                        if ($c < $query->repRows->size()-$im){
                            $mrv = $query->getReportMetaRecord($query->repRows->getPos($query->repRows->size()-$im)->getName());
                            $vnv = $mrv->getSelName();
                            $valv = $zeile[$vnv];
                            
                            for ($r = 1; $rs < $maxR
                                && ($r + $znr) < count($daten)
                                && $val == $daten[$znr+$r][$vn]
                                && $daten[$znr+$r]['S'.$c] == 0;
                                $r++ )
                                if ($valv != $daten[$znr+$r][$vnv]
                                    || $daten[$znr+$r]['Stufe'] != $daten[$znr+$r-1]['Stufe']) {
                                        $rs ++;
                                        $valv = $daten[$znr+$r][$vnv];
                            }
                        }
                    }else{
                        for ($r = 1; $rs < $maxR 
                                     && ($r + $znr) < count($daten)
                                     && $val == $daten[$znr+$r][$vn]
                                     && $daten[$znr+$r]['S'.$c] == 0;
                                     $r++ )
                            if ($daten[$znr+$r]['colGroup'] == 99) $rs ++; 
                    }
                    $rowSpans[$c] = $rs; 
//                    $lastRowSpan[$c] = $rs;
                }
            
                if ($query->values_in_columns){
                    echo createRowCell($zeile, $query->repRows->getPos($c), 
                        $rowSpans[$c] * $query->repValues->size());
                }
                    else
                        echo createRowCell($zeile, $query->repRows->getPos($c), $rowSpans[$c]);
                $rowSpans[$c] --;
            }
        }
        else{
            $class = 'class="summe"';
            if ($query->fix_labelColumns){
                $class = 'class="fts summe"';
            }
            $cc = $c - 1;
            $cs = $c;
            while ($cc > 0 && $query->repRows->getPos($cc)->webLinkColumn) $cc--;
            if ($spreadsheet) {
                for ($ci=0; $ci < $query->repRows->size(); $ci++){
                    if ($query->repRows->getPos($ci)->webLinkColumn) $cs++;
                }
            }
            if (! $query->repRows->getPos($c)->webLinkColumn && ($c == 0 || $zeile['S'.($cc)] == 0))
                if ($query->values_in_columns){
                    if ($query->repValues->size() > 1) $class = substr($class, 0,strlen($class)-1).' '.$query->valueColumnFolderMode.'"';
                    echo '<th colspan="'.($query->repRows->size()-$cs).'" '.$class.' rowspan="'.$query->repValues->size().'">'.WSRTranslate::translate('Insgesamt').'</th>';
            }else
                        echo '<th colspan="'.($query->repRows->size()-$cs).'" '.$class.' >'.WSRTranslate::translate('Insgesamt').'</th>';
                         
            $rowSpans[$c] = 0;
        }
    } // Ende FOR-Schleife repRows
    
    if ($query->values_in_columns) {
        for ($iv = 0; $iv < $query->repValues->size(); $iv++){
            if ($iv > 0) echo '</tr><tr>';
            /**
             * @var RepValue $v
             */
            $v = $query->repValues->getPos($iv);
            $vn = $v->getName();
            $mr = $query->getReportMetaRecord($v->getValueName());
            if ($v->isWindowValue()) $label = RepValue::getFunctionText($v->function).': '.$mr->label;
            else $label = $mr->label;
            if (substr($vn,1,1) == ':' || substr($vn,1,1) == '=' )
                $label = $query->repTables->get( substr($vn,0,1))->prefix.' '.$label;
            if ( ($v->backgroundColor == "white" || $v->backgroundColor == '#ffffff')) echo "\n<th ";
            else echo "\n<th style=\"background-color: ".$v->backgroundColor."\" ";
            if ($v->isWindowValue() && isValid(RepValue::getFunctionDesc($v->function))){
                if ($v->abSpalte > 0){
                    $ebene = ' '.WSRTranslate::translate('in').' '
                    .$query->getReportMetaRecord($query->repRows->getPos($v->abSpalte -1)->getName())->label;
                }
                else $ebene = '';
                if ($v->nachSpalten) $ebene .= ' '.WSRTranslate::translate('nach Spalten');
                if ($v->label != null)
                    echo makeTitle(RepValue::getFunctionDesc($v->function).$ebene.":\n".$label.":\n".$mr->description)." class =\"left fts draggable\"";
                    else echo makeTitle(RepValue::getFunctionDesc($v->function).$ebene.":\n".$mr->description)." class =\"left fts draggable\"";
                    
            }else{
                if ($v->label != null)
                    echo makeTitle($label.":\n".$mr->description)." class =\"left fts draggable\"";
                else echo makeTitle($mr->description)." class =\"left fts draggable\"";
            }
           if (!$spreadsheet){
               if ($aktiverEditor) dragNDdrop("'Werte'", $iv, 'reportDrop');
               else dragNDdrop("'Werte'", $iv, 'outputDrop');
           }
           
           if ($v->label != null) echo ">".$v->label.'</th>';
           else echo ">".$label.'</th>';
           
            for($c = 0; $c < count($colHeads); $c++){
                for ($is=0; $is < count($colHeads[$c]); $is++){
                if ($mr->enterValue 
                    && ! $spreadsheet
                    && substr($vn,1,1) != ':' 
                    && substr($vn,1,1) != '='
                    ){
                        $rowvars ='\'';
                        for ($zr = 0; $zr < $query->repRows->size(); $zr++){
                            $zvr = $query->repRows->getPos($zr);
                            if (false)$zvr = new RepRow('L');
                            if ($zvr->webLinkColumn) continue;
                            $mrn = $query->getReportMetaRecord($zvr->getName());
                            if (false) $mrn=new MetaRecord();
                            if ($mrn == null) continue;
                            $zvn = $mrn->getSelName();
                            $zv = $zeile[$zvn];
                            if ($zr > 0) $rowvars .= ';';
                            if (isValid($zv))$rowvars .= $zv;
                        }                        
                        $rowvars .= '\'';
                        echo "\n<td  ";
                        if (!$summenzeile){
                            if ( !($v->backgroundColor == "white" || $v->backgroundColor == '#ffffff'))
                                echo " style=\"background-color: ".$v->backgroundColor."\" ";
                        }
                        
                       if ( ($v->backgroundColor == "white" || $v->backgroundColor == '#ffffff')) echo "\n<td ";
                            else echo "\n<td  style=\"background-color: ".$v->backgroundColor."\" ";
                        if (array_key_exists($c, $colData)
                            && array_key_exists($colHeads[$c][$is]['sel'], $colData[$c])){
                                $cVal = $colHeads[$c][$is]['sel'];
                            if (array_key_exists($vn, $colData[$c][$cVal]))
                                $value = trim($colData[$c][$cVal][$vn]);
                                else $value = '';                        
                        }  else echo '>&nbsp;</td>';   
                        
                        echo ' class="'.$mr->align.'"><input type="text" size="'.$mr->enterValueSize.'" value="'.$value
                        .'" onchange="updateColValue(\''.$vn.'\','.$rowvars.',this.value,'.$c.',\''.trim($colHeads[$c][$is]['sel']).'\')"/>'."</td>\n";
                    }
                    else{
                        if ( ($v->backgroundColor == "white" || $v->backgroundColor == '#ffffff')) echo "\n<td ";
                            else echo "\n<td  style=\"background-color: ".$v->backgroundColor."\" ";
                        if (array_key_exists($c, $colData)
                            && array_key_exists($colHeads[$c][$is]['sel'], $colData[$c])){
                                $cVal = $colHeads[$c][$is]['sel'];
                            if (array_key_exists($vn, $colData[$c][$cVal]))
                                echo 'align="'.$mr->align.'">'.$colData[$c][$cVal][$vn].'</td>';
                                else echo '>&nbsp;</td>';                        
                        }  else echo '>&nbsp;</td>';   
                    }
                }
            }
        }
    }
    else {
        $lastValidRepRow = $query->repRows->size()-1;
        while ($lastValidRepRow > 0 
            && $query->repRows->getPos($lastValidRepRow)->webLinkColumn) $lastValidRepRow --;
        for($c=0; $c < $query->repValues->size(); $c++){
            $v = $query->repValues->getPos($c);
            $vn = $v->getValueName();
            $mr = $query->getReportMetaRecord($vn);
            if ($mr == null){
                echo "<td >Keine Metadaten für $vn</td>\n";
                continue;
            }
            if ($v->isWindowValue()) $vd = $v->getName();
            else {
                $vd = $mr->getName();
                if (substr($vn,1,1) == ':' || substr($vn,1,1) == '=' ) $vd = substr($vn,0,1).'__'.$vd;
            }
            if (!array_key_exists($vd, $zeile)) echo '<td>'.$vd.' fehlt</td>';
            else if ($mr->enterValue 
                && ! $spreadsheet
                && substr($vn,1,1) != ':' 
                && substr($vn,1,1) != '='
                ){
                $rowvars ='\'';
                for ($zr = 0; $zr < $query->repRows->size(); $zr++){
                    $zvr = $query->repRows->getPos($zr);
                    if (false)$zvr = new RepRow('L');
                    if ($zvr->webLinkColumn) continue;
                    $mrn = $query->getReportMetaRecord($zvr->getName());
                    if (false) $mrn=new MetaRecord();
                    if ($mrn == null) continue;
                    $zvn = $mrn->getSelName();
                    $zv = $zeile[$zvn];
                    if ($zr > 0) $rowvars .= ';';
                    if (isValid($zv))$rowvars .= $zv;
                }
                $rowvars .= '\'';
                echo "\n<td  ";
                if (!$summenzeile){
                    if ( !($v->backgroundColor == "white" || $v->backgroundColor == '#ffffff')) 
                        echo " style=\"background-color: ".$v->backgroundColor."\" ";
                }
                echo ' class="'.$mr->align.'"><input type="text" size="'.$mr->enterValueSize.'" value="'.formatWert($mr, $zeile[$vd])
                     .'" onchange="updateValue(\''.$vn.'\','.$rowvars.',this.value)"/>'."</td>\n";
            }
            else  if (! $spreadsheet 
                && $v->dispVal != DisplayOptions::ZAHL
                &&( $query->display_charts ==  ChartOptions::IMMER
                    || ($query->display_charts ==  ChartOptions::NURSUMMEN
                        && $zeile['S'.$lastValidRepRow] == 1 )
                    || ($query->display_charts ==  ChartOptions::KEINESUMMEN
                        && $zeile['S'.$lastValidRepRow] == 0 )
                    )
                ){
                $id = 'R'.$znr.'S'.$c;
                echo "\n<td  ";
                if (!$summenzeile)
                    if ( !($v->backgroundColor == "white" || $v->backgroundColor == '#ffffff'))
                        echo " style=\"background-color: ".$v->backgroundColor."\" ";
                    
                switch ($v->dispVal){
                    case DisplayOptions::ZAHLHBAR:
                        $id = 'H'.$id;
                        echo ' class="'.$mr->align.'"><div class="right nowrap">';
                        if ($v->isWindowValue() && $v->hasFunctionFormatDecs() && $zeile[$vd] != null)
                            echo WSRFormats::formatDecimal($zeile[$vd],$v->getFunctionFormatDecs());                            
                        else echo formatWert($mr, $zeile[$vd]);
                        echo '&nbsp;<canvas id="'.$id.'"></canvas></div>'."</td>\n";
                        break;
                    case DisplayOptions::ZAHLVBAR:
                        $id = 'V'.$id;
                        echo ' class="'.$mr->align.'"><canvas class="center" id="'.$id.'"></canvas><br>';
                        if ($v->isWindowValue() && $v->hasFunctionFormatDecs() && $zeile[$vd] != null)
                            echo WSRFormats::formatDecimal($zeile[$vd],$v->getFunctionFormatDecs());                            
                        else echo formatWert($mr, $zeile[$vd]);
                        echo "</td>\n";
                        break;
                    case DisplayOptions::HBAR:
                        $id = 'H'.$id;
                        echo ' class="'.$mr->align.'"><canvas class="right" id="'.$id.'"></canvas>'."</td>\n";
                        break;
                    case DisplayOptions::VBAR:
                        $id = 'V'.$id;
                        echo ' class="'.$mr->align.'"><canvas class="center" id="'.$id.'"></canvas>'."</td>\n";
                        break;
                }
                switch($v->skalierung){
                    case ChartOptions::SKALIERETABELLE:
                        setChart($vn,$mr,$id,$zeile[$vd],'_TABELLE');
                        break;
                    case ChartOptions::SKALIEREZEILE:
                        setChart($vn,$mr,$id,$zeile[$vd],'ZNR'.$znr);
                        break;
                    case ChartOptions::SKALIEREWERT:
                        setChart($vn,$mr,$id,$zeile[$vd],$vn);
                        break;
                    case ChartOptions::SKALIERESPALTE:
                        setChart($vn,$mr,$id,$zeile[$vd],$vn);
                        break;
                    default:
                        setChart($vn,$mr,$id,$zeile[$vd],$mr->chartGroup);
                        break;
                }
            }
            else {
                echo "\n<td ";
                if (!$summenzeile) 
                    if ( !($v->backgroundColor == "white" || $v->backgroundColor == '#ffffff'))
                        echo " style=\"background-color: ".$v->backgroundColor."\" ";
                        echo ' class="'.$mr->align.'">';
                        if ($v->isWindowValue() && $v->hasFunctionFormatDecs() && $zeile[$vd] != null)
                            echo WSRFormats::formatDecimal($zeile[$vd],$v->getFunctionFormatDecs());
                        else
                            echo formatWert($mr, $zeile[$vd])."</td>\n";
                
            }
        } // ENDE for($c=0; $c < $query->repValues->size(); $c++){
        for ($c = 0; $c < count($colHeads); $c++){
            for ($i = 0; $i < count($colHeads[$c]);$i++){
                if (array_key_exists($c, $colData) && array_key_exists($colHeads[$c][$i]['sel'], $colData[$c])){
                    $rc = $query->repColumns->getPos($c);
                    if (false) $rc = new RepColumn('');
                    if (! isValid($rc->web_link) || ! $rc->reportLinkInValues){
                        $mr = $query->getReportMetaRecord($rc->val_name);
                        if ($mr->enterValue && ! $spreadsheet){
                            $rowvars ='\'';
                            for ($zr = 0; $zr < $query->repRows->size(); $zr++){
                                $zvr = $query->repRows->getPos($zr);
                                if (false)$zvr = new RepRow('L');
                                if ($zvr->webLinkColumn) continue;
                                $mrn = $query->getReportMetaRecord($zvr->getName());
                                if (false) $mrn=new MetaRecord();
                                if ($mrn == null) continue;
                                $zvn = $mrn->getSelName();
                                $zv = $zeile[$zvn];
                                if ($zr > 0) $rowvars .= ';';
                                if (isValid($zv))$rowvars .= $zv;
                            }                        
                            $rowvars .= '\'';
                            echo "\n<td  ";
    /*                        
                            if (!$summenzeile){
                                if ( !($v->backgroundColor == "white" || $v->backgroundColor == '#ffffff'))
                                    echo " style=\"background-color: ".$v->backgroundColor."\" ";
                            }
    */                        
                            echo ' class="'.$mr->align.'"><input type="text" size="'.$mr->enterValueSize.'" value="'.$colData[$c][$colHeads[$c][$i]['sel']]
                            .'" onchange="updateColValue(\''.$rc->val_name.'\','.$rowvars.',this.value,'.$c.',\''.trim($colHeads[$c][$i]['sel']).'\')"/>'."</td>\n";
                        }
                        else
                        echo '<td align="right">'.$colData[$c][$colHeads[$c][$i]['sel']].'</td>';
                    }
                        else{
                            if (substr($rc->web_link,0,1) == '#'){
                                $mrl = $query->getReportMetaRecord(trim(substr($rc->web_link,1)));
                                if ($mrl != null)
                                    $link = $zeile[$mrl->getName()];
                                    $web_title = "";
                                    if (isValid($mrl->displayVar))  $web_title = quoteForText($zeile[$mrl->getDispName()]);
                                    else
                                        if (isValid($mrl->description))  $web_title = quoteForText($mrl->description);
                                        else
                                            if (isValid($mrl->label))  $web_title = quoteForText($mrl->label);
                                            //                            else if (isValid($row->web_title)) $web_title = quoteForText($row->web_title);
                                            
                                            if (isValid($web_title)) $web_title = ' title="'.$web_title.'" ';
                                    if (isValid($link)){
                                        if (isValid($rc->web_target)) $wtt =' target="'.quoteForText($rc->web_target).'" ';
                                        else $wtt = "";
                                        if (substr($link,0,1) == '.')
                                            $cell .='<a href="'. $link.$wtt.$web_title.'>';
                                        else $cell .='<a href="http://'.$link .$wtt.$web_title.'>';
                                    }
                            
                            }else{ // Link auf Report
                                if (isValid($rc->web_title)) $wtt = ' title="'.$rc->web_title.'" ';
                                    else $wtt = 'title="Aufruf des Reports '.$rc->web_link.'" ';
                                    if (isValid($rc->web_target)) $wtt .= ' target ="'.quoteForText($rc->web_target).'" ';
                                $web_link ='<a '.$wtt.' href="./outputReport.php?report='.$rc->web_link;
                                $web_link .= makeGetFilter($rc->reportCopyFilter,$zeile,null,$rc,$colHeads[$c][$i]['sel']);
                                $web_link.='">';
                            }
                            echo '<td align="right">'.$web_link.$colData[$c][$colHeads[$c][$i]['sel']].'</a></td>';
                        }// Ende Link auf Report
                }
                else echo '<td>&nbsp;</td>';
            } // ENDE for ($i = 0; $i < count($colHeads[$c]);$i++){
        }// ENDE for ($c = 0; $c < count($colHeads); $c++){
    }// ENDE if not values_in_columns
    
    echo "</tr>\n";
    }
    $colData = array();
    }
 
}
if ($hasTfoot) echo'</tfoot>';
else echo'</tbody>';
$fertig = hrtime(true);

?>



</table>
<?php 
/****
 * Ende der HTML-Tabelle
 * Anzeige der Geld-Einheiten
 */

switch ($query->repGeldFaktor){
    case 0.001: echo '<div>'.WSRTranslate::translate('Geldangaben in Tsd.€')."</div>\n";
    break;
    case 0.000001: echo '<div>'.WSRTranslate::translate('Geldangaben in Mio.€')."</div>\n";
    break;
    default: break;
}

/****
 * Ende des Forms für die Sortierung
 */
?>
</form>
<?php
/****
 * Kommentar ausgeben
 */
 if (isValid($query->kommentar)) echo'<br><div class="kommentar">'.quoteForHTML($query->kommentar).'</div><br>';
 
 /****
  * Ausgabe aller Filter für diese Abfrage
  */
 ?> 
<div class="inlineBox neueSeite">
<?php 
if (array_key_exists('User', $_SESSION)){
    if ($spreadsheet) echo "<br>";
    echo '<b>'.WSRTranslate::translate('Anmeldung').':</b> '.$_SESSION['Name']."\n";
}
if ($spreadsheet) echo '<br><b>'.WSRTranslate::translate('Auswahl').'</b><br>';
else echo '<h3>'.WSRTranslate::translate('Auswahl').'</h3>';
//  (<?=number_format($fertig,0,',','.'))
 // $selFilters = false;
 $rechteFilter = rechteFilter();
 if ($werte != null)
     echo "<b>".WSRTranslate::translate('Werte')."</b>: ".$werte."<br><br>\n";    
 for ($i=0; $i < $query->repFilter->size(); $i++){
     $filter =  $query->repFilter->getPos($i);
     if (false !== in_array($filter->getName(), $rechteFilter)) {
         continue;
     }
     $titel = showFilter($filter,true);
     if (isValid($titel)){
//         $mr = $query->getReportMetaRecord($filter->getName());
         if ($spreadsheet) echo "<div class=\"nowrap\"><b>".$titel;
         else echo "<div class=\"nowrap\">".$titel.' '.showFilterValue($filter->getName());            
         if ($filter->abSpalte > 0) {
             if ($filter->isWindowFilter()){
                 if (! $filter->isSortFilter()){
                     echo ' Ebene: '.$query->getReportMetaRecord($query->repRows->getPos($filter->abSpalte-1)->getName())->label;
                 }
             }else
             echo ' ab '.$query->getReportMetaRecord($query->repRows->getPos($filter->abSpalte-1)->getName())->label;
             
         }
            if ($spreadsheet) 
             echo '<br>';
         echo "</div>\n";
     }
 }
 
?> 
 </div>
<?php 
 
/**
 * Historisierung anzeigen.
 * Zunächst Prüfung ob:
 */ 
if (!isset($_SESSION['EndUser'])){
$showHist = false;
$showQueryHist = array();
if ($query->histType > 0 && $query->historyOption > 0){
    $showHist = true;
    $showQueryHist[-1] = 1;
}
foreach ($query->dimensions as $dim)
//    $dim = new Dimension(null);
    if ($dim->histType > 0
        && $dim->historyOption > 0
        && ($dim->historyOption != $query->historyOption
            || ($dim->historyOption == HistoryOptions::HSDATUM
                && $dim->historyDate != $query->historyDate)
            )
        ){
            $showHist = true;
            $showQueryHist[-1] = 2;            
    }
 
for ($it = 0; $it < $query->repTables->size(); $it ++){
    $jt = $query->repTables->getPos($it);
    if (false) $jt = new JoinedTable(null);
    if (count($jt->dimHistoryOption) > 0
        && $jt->dimHistoryOption[0] != 0
        && ($jt->dimHistoryOption[0] != $query->historyOption
            || ($jt->dimHistoryOption[0] == HistoryOptions::HSDATUM
                && $jt->dimHistoryDate[0] != $query->historyDate)
            )
        ){
        $showHist = true;
        $showQueryHist[$it] = 1;
    }
    foreach ($jt->dimHistoryOption as $nr => $o)
        if ($nr > 0
            && $o != 0
            && ($o != $jt->dimHistoryOption[0]
            || ($o == HistoryOptions::HSDATUM
            && $jt->dimHistoryDate[0] != $jt->dimHistoryDate[$nr])
            )
        ){
            $showHist = true;
            $showQueryHist[$it] = 2;
    }
}
/**
 * Historisierung anzeigen
 */
if ($showHist){
?>    
    <div class="inlineBox">
    <?php
    if ($spreadsheet) echo '<br><b>'.WSRTranslate::translate('Stand (Historisierung)').':</b><br>';
    else echo '<h3>'.WSRTranslate::translate('Stand (Historisierung)').':</h3>';
    echo "<div class=\"nowrap\">";
    if ($query->histType > 0){
        echo "<b>".WSRTranslate::translateH('Allgemein').":</b> ";
        if ($query->historyOption == HistoryOptions::HSUEBERNEHMEN) echo WSRTranslate::translateH('Aktuell');    
        else if ($query->historyOption == HistoryOptions::HSDATUM) 
            echo WSRFormats::formatDate(DateTime::createFromFormat('Y-m-d',$query->historyDate));
        else echo HistoryOptions::getKlartext($query->historyOption);
        echo "<br>\n";
    }
    if (array_key_exists(-1, $showQueryHist) && $showQueryHist[-1] > 1){
        echo '&nbsp;<b>'.WSRTranslate::translateH('Dimensionen').':</b>';
        foreach ($query->dimensions as $dim)
//            $dim = new Dimension(null);
            if ($dim->histType > 0
                && $dim->historyOption > 0){
                    if ($dim->historyOption == HistoryOptions::HSDATUM
                        && $dim->historyDate != $query->historyDate){
                            echo "\n<br>&nbsp;&nbsp;".$dim->dimName.': ';
                            echo WSRFormats::formatDate(DateTime::createFromFormat('Y-m-d',$dim->historyDate));
                    }else if ($dim->historyOption != $query->historyOption){
                        echo "\n<br>&nbsp;&nbsp;".$dim->dimName.': ';
                        echo HistoryOptions::getKlartext($dim->historyOption);
                    }
                    echo "<br>\n";                    
        }
         
    } // Ende if ($showQueryHist[-1] > 1)
        echo "</div>\n";
    foreach ($showQueryHist as $snr => $so)
        if ($snr >= 0 && $so > 0){
            $jt = $query->repTables->getPos($snr);
            if (false) $jt = new JoinedTable(null);
            echo "\n<br><div class=\"nowrap\"><b>{$jt->prefix}:</b> ";
            if ($jt->dimHistoryOption[0] == HistoryOptions::HSUEBERNEHMEN) echo '';
            else if ($jt->dimHistoryOption[0] == HistoryOptions::HSDATUM){
                $datum = DateTime::createFromFormat('Y-m-d',$jt->dimHistoryDate[0]);
                if ($datum) echo WSRFormats::formatDate($datum);
                else echo '<b>'.WSRTranslate::translate('Fehler').':</b> '
                    .WSRTranslate::translate('expliziter Stichtag fehlt').' '.$jt->dimHistoryDate[0];
            }
            else echo HistoryOptions::getKlartext($jt->dimHistoryOption[0]);
            if ($showQueryHist[$snr] > 1){
                echo '<br>&nbsp;<b>Dimensionen:</b>';
                foreach ($jt->dimHistoryOption as $nr => $o)
                    if ($nr > 0 && $o > 0){
                        $dim = $jt->query->dimensions[$nr];
                        if ($o == HistoryOptions::HSDATUM
                            && $jt->dimHistoryDate[$nr] != $jt->dimHistoryDate[0]){
                                echo "\n<br>&nbsp;&nbsp;".$dim->dimName.': ';
                                echo WSRFormats::formatDate(DateTime::createFromFormat('Y-m-d',$jt->dimHistoryDate[$nr]));
                        }else if ($o != $jt->dimHistoryOption[0]){
                            echo "\n<br>&nbsp;&nbsp;".$dim->dimName.': ';
                            echo HistoryOptions::getKlartext($o);
                        }
                } // ENDE foreach dim
            }
            echo "</div>\n";
    }// Ende foreach $showQueryHist
    echo "</div>\n";
    if ($spreadsheet) echo '<br><br>';
} // Ende if $showHist
} // if ! endUser
?>    
 <?php
 
/****
  <div class="inlineBox noprint">

 * Forms für Aufruf des Report-Editors und Spreadsheet stehen testweise oben
 
 if ($bearbeitenButton){
?>     
     <form method="get" action = "./editReport.php" target="ReportEditor" id='RepEditor'>
<?php 
makePostFilter();
?>     <input type="hidden" name="report" value="<?=$query->title?>">
     </form>
<?php 
 }
/****
 * Ende Aufruf des Report-Editors
 * 
 * Beginn Spreadsheet
 
 
 if (! $spreadsheet){
?>     
     <form method="put" action = "./outputReport.php" id='Spread' >
<?php 
makePostFilter();
$moves->outputParams();
?>
     <input type="hidden" name="report" value="<?=$query->title?>">
     <input type="hidden" name="spreadSheet" value="X">
     </form>
     
<?php 
 }
/****
 * Ende Spreadsheet
 echo '</div>';
*/
if (dqAdminRechte()){
?>     
<div ><?=WSRTranslate::translate('Zeit für Abfrage').' '.number_format(($startAusgabe - $startZeit) / 1000000,0)?><br>
<?=WSRTranslate::translate('Zeit für Ausgabe').' '.number_format(($fertig - $startAusgabe) / 1000000,0) ?><br>
<?=WSRTranslate::translate('Insgesamt').' '.number_format(($fertig - $aufrufZeit) / 1000000,0) ?><br></div>
<?php 
}
?>
</body>
<?php 
/****
 * Übergabe der Chartwerte als JavaScript-Funktionsaufrufe
 * ACHTUNG: Die Chart-Werte werden erst beim Aufbau der Tabelle erzeugt!!!
 */

	if (! $spreadsheet && count($chartValues) > 0 ){
?>
 <script>	
	function drawCharts(){
<?php 
    foreach ($chartValues as $chv){
        if ($chv->min > 0) $chv->min = 0;
        if ($chv->max < 0) $chv->max = 0;
        foreach ($chv->values as $id => $val){
            if ($val < 0) $color = $chv->negColor;
            else $color = $chv->color;
            if (substr($id,0,1) == "H")
                echo 'drawHBarChart("'.$id.'",'.$val.','.$chv->min.','.$chv->max.',"'.$color.'");'."\n";
            else
                echo 'drawVBarChart("'.$id.'",'.$val.','.$chv->min.','.$chv->max.',"'.$color.'");'."\n";
        }
    }
?>
	}
document.addEventListener('DOMContentLoaded', drawCharts()); 
</script>

<?php 
}
// echo '<pre>'; var_dump($chartValues); echo '</pre>';
// echo '<pre>'; var_dump($daten); echo '</pre>';
$stmt->closeCursor();
?>	

</html>
