<?php
/**
 * Projekt: WS-Reporter
 *          Installation mit Anlegen der WS-Reporter-Tabellen
 *
 * Autor:    Wilfried Schollenberger
 *           WS Unternehmensberatung und Controlling-Systeme GmbH
 * Kontakt:  wisch@ws-unternehmensberatung.de
 * Lizenz:   GPL 3.0
 *
 * Stand:    26.2.2023 (0.142)
 *
 */

global $meldung, $dbh, $fertig;
$meldung ='';
$fertig = true;

$sql = <<<EOS
CREATE TABLE `queryAttribute` (
 `table_name` varchar(64) DEFAULT NULL COMMENT 'Name der Tabelle',
 `SatzTyp` smallint(6) DEFAULT NULL COMMENT 'Typ des Eintrags',
 `attribute` varchar(250) DEFAULT NULL COMMENT 'Attribut für Datenquelle',
 PRIMARY KEY (`table_name`,`SatzTyp`)
) ENGINE=InnoDB  
EOS;

if ($dbh->exec($sql)  !== false){
    $meldung .= "<br>Tabelle queryAttribute wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei Tabelle queryAttribute:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}

$sql = <<<EOS
  CREATE TABLE queryComment 
    (
        table_name varchar(64)  comment "Name der Tabelle" 
      , qLanguage char(2)  comment "Sprache der Datenquelle" 
      , SortNr Smallint  comment "Sortierfolge" 
      , kommentar varchar(255)  comment "Kommentar" 

      , PRIMARY KEY ( table_name, qLanguage, SortNr )
    )
    ;   

EOS;

if ($dbh->exec($sql) !== false){
    $meldung .= "<br>Tabelle queryComment wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei Tabelle queryComment:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}

$sql = <<<EOS
CREATE TABLE `queryColumnMetadata` (
 `table_name` varchar(64)  NOT NULL COMMENT 'Name der Tabelle',
 `column_name` varchar(64) NOT NULL COMMENT 'Name des Attributs',
 `SpaltenTyp` smallint(6) NOT NULL COMMENT 'Typ der Spalte',
 `dimNr` Smallint  comment "Dimensions-Nummer", 
 `SortNr` smallint(6) NOT NULL COMMENT 'Sortierfolge',
 `column_label` varchar(255) DEFAULT NULL COMMENT 'Label zum Attribut',
 `column_desc` varchar(1024) COLLATE utf8_german2_ci DEFAULT '' COMMENT 'Beschreibung zur Spalte',
 `column_hints` varchar(1024)   comment "Technische Hinweise für Spalten"
 `qColType` smallint(5) unsigned DEFAULT NULL COMMENT 'Typ der Spalte',
 `qDecimals` smallint(5) unsigned DEFAULT NULL COMMENT 'Anzahl Nachkommastellen in der Ausgabe',
 `qFilterTyp` smallint(5) unsigned DEFAULT NULL COMMENT 'Filter-Typ',
 `qSelectVar` varchar(1024) DEFAULT NULL COMMENT 'Name oder Ausdruck zur Abfrage des Wertes',
 `qDisplayVar` varchar(1024) COLLATE utf8_german2_ci DEFAULT NULL COMMENT 'Name der anzuzeigenden Spalte, bei Werten name der Ergebnis-Spalte',
 `qSelectFilterTable` varchar(255) COLLATE utf8_german2_ci DEFAULT NULL,
 `qSelectFilterVar` varchar(255) COLLATE utf8_german2_ci DEFAULT NULL COMMENT 'Name der Spalte zur Einschränkung des Wertebereichs',
 `qSelectFilterTable2` varchar(255) COLLATE utf8_german2_ci DEFAULT NULL,
 `qSelectFilterVar2` varchar(255) COLLATE utf8_german2_ci DEFAULT NULL,
 `qHtmlData` char(15) DEFAULT NULL COMMENT 'HTML-Tag für Daten',
 `qAlign` char(15) DEFAULT NULL COMMENT 'Ausrichtung der Anzeige (left, center, right)',
 `qChartGruppe` varchar(64) DEFAULT NULL COMMENT 'Name der Chartgruppe zur Skalierung bei mehreren Werten mit gemeinsamer Skalierung',
 `qChartColor` char(15) DEFAULT NULL COMMENT 'Farbe der Grafik',
 `qChartNegColor` char(15) DEFAULT NULL COMMENT 'Farbe der Grafik bei negativen Werten',
 `qChartMin` double  DEFAULT NULL comment "Optionale Untergrenze für Skalierung der Charts", 
 `qChartMax` double  DEFAULT NULL comment "Optionale Obergrenze für Skalierung der Charts",
  enterValue boolean default false  comment "Eingabefeld", 
  enterValueSize INT DEFAULT '15' comment "Größe des Eingabefelds", 
  qSetVarname varchar(64) DEFAULT NULL comment "Name der Variablen im Set-Statement bei der Daten-Eingabe",
  filterRequiredGroup SMALLINT NULL DEFAULT '0' COMMENT 'Erforderlicher Filter - Gruppe',   
  filterSort  SMALLINT  DEFAULT '1' COMMENT 'Sortierung des Auswahl-Filters', 
  hidden boolean  comment "Kennzeichen: Verborgen", 
  reqDimNr1 Smallint  comment "Erforderliche Dimension", 
  reqDimNr2 Smallint  comment "Erforderliche Dimension",
  reqDimNr3 Smallint  comment "Erforderliche Dimension", 
  reqDimNr4 Smallint  comment "Erforderliche Dimension", 
  reqDimNr5 Smallint  comment "Erforderliche Dimension", 
   PRIMARY KEY (`table_name`,`column_name`)
) ENGINE=InnoDB 
EOS;

if ($dbh->exec($sql) !== false){
    $meldung .= "<br>Tabelle queryColumnMetadata wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei Tabelle queryColumnMetadata:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}


$sql = <<<EOS
CREATE TABLE `queryColumnMetadataSelectOptions` (
 `table_name` varchar(64) NOT NULL COMMENT 'Name der Tabelle',
 `column_name` varchar(64) NOT NULL COMMENT 'Name des Attributs',
 `SortNr` smallint(6) NOT NULL COMMENT 'Sortierfolge',
 `value` varchar(250) DEFAULT NULL COMMENT 'Value-Wert in der Auswahl',
 `label` varchar(1024) DEFAULT NULL COMMENT 'Angezeigter Text in der Auswahl',
 PRIMARY KEY `qSelections` (`table_name`,`column_name`,`SortNr`),
 UNIQUE KEY `qSelectionsValue` (`table_name`,`column_name`,`value`),
 UNIQUE KEY `qSelectionsLabel` (`table_name`,`column_name`,`label`)
) ENGINE=InnoDB 
EOS;

if ($dbh->exec($sql) !== false){
    $meldung .= "<br>Tabelle queryColumnMetadataSelectOptions wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei Tabelle queryColumnMetadataSelectOptions:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}


$sql = <<<EOS
CREATE TABLE queryDefaultFilter
(
    table_name varchar(64)  comment "Name der Tabelle"
    , column_name varchar(64)  comment "Name des Filters"
    , qAddFilterVon varchar(255)  comment "Erster Wert des Filters"
    , qAddFilterBis varchar(255)  comment "Zweiter Wert des Filters"
    
    , primary key ( table_name, column_name )
    )
    ;
EOS;
if ($dbh->exec($sql) !== false){
    $meldung .= "<br>Tabelle queryDefaultFilter wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei Tabelle queryDefaultFilter:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}


$sql = <<<EOS
CREATE TABLE queryDimensions
(
    table_name varchar(64)  comment "Name der Tabelle"
    , SortNr Smallint  comment "Sortierfolge"
    , qDimName varchar(64)  comment "Name der Dimension"
    , baseDimNr Smallint  comment "Dimensions-Nummer bei mehrstufiger Verknüpfung"
    , qDimTable varchar(64)  comment "Name der physischen Tabelle zu einer Dimension"
    , qDimType Smallint  comment "Art der Verknüpfung einer Dimension"
    , qDimKey varchar(64)  comment "Name der Zeile in der Dimension zum Verknüpfen"
    , qBaseKey varchar(64)  comment "Name der Zeile in der Basisquelle"
    , qSelectVar varchar(255)  comment "Name oder Ausdruck zur Abfrage des Wertes"
      , qDimIsFilter boolean  comment "Kennzeichen, dass die Dimension immer als Filter verwendet wird" 
      , qHistType Smallint  comment "Typ der Historisierung" 
      , qHistoryOption Smallint  comment "Kennzeichen, wie Stichtag für Historisierung ermittelt wird" 
      , qHistoryDate date  comment "Manuell gesetzter Stichtag" 
      , qGueltigAb varchar(64)  comment "Variablenname für gueltig_ab in historisierten Tabellen" 
      , qGueltigBis varchar(64)  comment "Variablenname für gueltig_bis in historisierten Tabellen" 
      , qStichtag varchar(64)  comment "Variablenname für Stichtag in historisierten Tabellen" 
    
    , primary key ( table_name, SortNr )
    )
    ;   
EOS;

if ($dbh->exec($sql) !== false){
    $meldung .= "<br>Tabelle queryDimensions wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei Tabelle queryDimensions:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}

$sql = <<<EOS
  CREATE TABLE queryHints
    (
        table_name varchar(64)  comment "Name der Tabelle"
      , qLanguage char(2)  comment "Sprache der Datenquelle"
      , SortNr Smallint  comment "Sortierfolge"
      , kommentar varchar(255)  comment "Kommentar"
      
      , PRIMARY KEY ( table_name, qLanguage, SortNr )
    )
    ;
    
EOS;

if ($dbh->exec($sql) !== false){
    $meldung .= "<br>Tabelle queryHints wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei Tabelle queryHints:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}




$sql = <<<EOS
   CREATE TABLE queryTranslations 
    (
        qLanguageType smallint unsigned  comment "Typ der Text-Eintrags für Übersetzungen" 
      , de varchar(1024) CHARACTER SET utf8 COLLATE utf8_bin NULL DEFAULT NULL  comment "Deutscher Text" 
      , en varchar(1024) CHARACTER SET utf8 COLLATE utf8_bin NULL DEFAULT NULL  comment "Englischer Text" 
      , fr varchar(1024) CHARACTER SET utf8 COLLATE utf8_bin NULL DEFAULT NULL  comment "Französischer Text" 
      , UNIQUE INDEX de ( de)
      , UNIQUE INDEX en ( en)
      , UNIQUE INDEX fr ( fr)
      , INDEX qlType ( qLanguageType )

    ) ENGINE=InnoDB
    ;   
EOS;
if ($dbh->exec($sql) !== false){
    $meldung .= "<br>Tabelle queryTranslations wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei Tabelle queryTranslations:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}


/** Tabellen für Report-Definitionen   */
$sql = <<<EOS
 CREATE TABLE reportAddFilterLists 
    (
        reportID bigint unsigned  comment "Report-ID" 
      , column_name varchar(64)  comment "Name des Attributs" 
      , fs_von varchar(255)  comment "Filterwert von" 

      , PRIMARY KEY ( reportID, column_name, fs_von )
) ENGINE=InnoDB
EOS;

if ($dbh->exec($sql) !== false){
    $meldung .= "<br>Tabelle reportAddFilterLists wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei Tabelle reportAddFilterLists:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}

/** Tabelle   */
$sql = <<<EOS
 CREATE TABLE reportAddFilters 
    (
        reportID bigint unsigned  comment "Report-ID" 
      , SortNr Smallint  comment "Sortierfolge" 
      , column_name varchar(64)  comment "Name des Attributs" 
      , fs_von varchar(255)  comment "Filterwert von" 
      , fs_bis varchar(255)  comment "Filterwert bis" 
      , ausschlussListe boolean  comment "Liste enthält auszuschließende Werte" 
      , abSpalte TINYINT  comment "Spalte, ab der der Filter wirken soll" 
      , selectorImOutput boolean  comment "Selector im Output anbieten" 
      , inTitelZeile TINYINT  comment "Titelzeile für Anzeige des Filters" 
      , inTitelMitLabel boolean  comment "Filter soll in der Titel-Zeile mit Label angezeigt werden" 
      , reportCopyFilter boolean  comment "Filter soll in Link mit übergeben werden" 
      , doNotTake boolean  comment "Filter soll aus dem Aufruf nicht übernommen werden" 

      , PRIMARY KEY ( reportID, column_name )
) ENGINE=InnoDB
EOS;

if ($dbh->exec($sql) !== false){
    $meldung .= "<br>Tabelle reportAddFilters wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei Tabelle reportAddFilters:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}

/** Tabelle   */
$sql = <<<EOS
 CREATE TABLE reportAttributes 
    (
        reportID bigint unsigned PRIMARY KEY comment "Report-ID" 
      , table_name varchar(64)  comment "Name der Tabelle" 
      , sort_mode TINYINT  comment "Art der Sortierung im Output" 
      , fix_labelColumns boolean  comment "Label-Spalten fixieren" 
      , values_in_columns boolean  comment "Werte in einer Spalte anzeigen" 
      , folderMode TINYINT  comment "Aufklapp-Zustand der Tabelle bei der Ausgabe" 
      , valueColumnFolderMode char(15)  comment "Einklapp-Mode für Werte in Spalten" 
      , chart_size char(3)  comment "Größe der Carts im Output" 
      , display_charts TINYINT  comment "Chart-Ausgabe" 
      , dropSummen boolean  comment "Redundante Summen-Zeilen unterdrücken" 
      , historyOptions varchar(255)  comment "Optionen zur Historisierung" 
      , rLanguage char(2)  comment "Sprache des Reports" 
      , rTranslations char(15)  comment "Report-Übersetzungen" 
      , repWerteBasis varchar(64)  comment "Basis für Werte-Auswahl" 
      , repWerteAuswahlTitel TINYINT  comment "Titelzeile zur Anzeige der Werte-Auswahl" 
      , repWerteAuswahlLabel boolean  comment "Wertefilter in Titel mit 'Werte:'" 
      , repWerteAuswahlSelector boolean  comment "Selektor für Werte-Filter im Output" 
      , repGeldFaktor decimal(20,10) default 1 not null  comment "Faktor zur Umrechnung von Geld-Beträgen" 
) ENGINE=InnoDB
EOS;

if ($dbh->exec($sql) !== false){
    $meldung .= "<br>Tabelle reportAttributes wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei Tabelle reportAttributes:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}

/** Tabelle   */
$sql = <<<EOS
 CREATE TABLE reportColumns 
    (
        reportID bigint unsigned  comment "Report-ID" 
      , SortNr Smallint  comment "Sortierfolge" 
      , column_name varchar(64)  comment "Name des Attributs" 
      , column_label varchar(255)  comment "Label zum Attribut" 
      , sort Smallint  comment "Sortierung der Ausgabe" 
      , val_name varchar(64)  comment "Name des Werts, der in den Spalten angezeigt wird" 
      , filter varchar(64)  comment "Name eines zusätzlichen Filters für diese Spalten" 
      , fs_von varchar(255)  comment "Filterwert von" 
      , fs_bis varchar(255)  comment "Filterwert bis" 
      , web_link varchar(255)  comment "Web-Link " 
      , web_title varchar(255)  comment "Title-Anzeige über Web-Link" 
      , web_target varchar(255)  comment "Fenster-Name für Web-Link" 
      , reportCopyFilter boolean  comment "Filter soll in Link mit übergeben werden" 
      , reportLinkInValues boolean  comment "Web-Link bei den Werten" 
      , keineSummen boolean  comment "Keine Summen ausgeben" 
      , dispval TINYINT  comment "Kennzeichen der Darstellung mit oder ohne Balken" 
      , skalierung TINYINT  comment "Kennzeichen der Gruppe zur Skalierung der Charts" 
      , wfFunction TINYINT DEFAULT 0  comment "Statistische Funktion für kumulierten Wert" 
      , abSpalte TINYINT DEFAULT 0  comment "Spalte, ab der die Funktion wirken soll" 
      , nachSpalten boolean DEFAULT false comment "Window-Funktion spaltenweise anwenden"
      , PRIMARY KEY ( reportID, column_name )
) ENGINE=InnoDB
EOS;

if ($dbh->exec($sql) !== false){
    $meldung .= "<br>Tabelle reportColumns wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei Tabelle reportColumns:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}

/** Tabelle   */
$sql = <<<EOS
 CREATE TABLE reportComment 
    (
        reportID bigint unsigned  comment "Report-ID" 
      , rLanguage char(2)  comment "Sprache des Reports" 
      , SortNr Smallint  comment "Sortierfolge" 
      , kommentar varchar(255)  comment "Zusätzlicher Kommentar" 

      , PRIMARY KEY ( reportID, rLanguage, SortNr )
) ENGINE=InnoDB
EOS;

if ($dbh->exec($sql) !== false){
    $meldung .= "<br>Tabelle reportComment wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei Tabelle reportComment:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}

/** Tabelle   */
$sql = <<<EOS
 CREATE TABLE reportFilters 
    (
        reportID bigint unsigned  comment "Report-ID" 
      , SortNr Smallint  comment "Sortierfolge" 
      , column_name varchar(64)  comment "Name des Attributs" 
      , fs_von varchar(255)  comment "Filterwert von" 
      , fs_bis varchar(255)  comment "Filterwert bis" 
      , ausschlussListe boolean  comment "Liste enthält auszuschließende Werte" 
      , abSpalte TINYINT  comment "Spalte, ab der der Filter wirken soll" 
      , selectorImOutput boolean  comment "Selector im Output anbieten" 
      , inTitelZeile TINYINT  comment "Titelzeile für Anzeige des Filters" 
      , inTitelMitLabel boolean  comment "Filter soll in der Titel-Zeile mit Label angezeigt werden" 
      , reportCopyFilter boolean  comment "Filter soll in Link mit übergeben werden" 
      , doNotTake boolean  comment "Filter soll aus dem Aufruf nicht übernommen werden" 

      , PRIMARY KEY ( reportID, column_name )
) ENGINE=InnoDB
EOS;

if ($dbh->exec($sql) !== false){
    $meldung .= "<br>Tabelle reportFilters wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei Tabelle reportFilters:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}

/** Tabelle   */
$sql = <<<EOS
 CREATE TABLE reportFilterLists 
    (
        reportID bigint unsigned  comment "Report-ID" 
      , column_name varchar(64)  comment "Name des Attributs" 
      , fs_von varchar(255)  comment "Filterwert von" 

      , PRIMARY KEY ( reportID, column_name, fs_von )

) ENGINE=InnoDB
EOS;

if ($dbh->exec($sql) !== false){
    $meldung .= "<br>Tabelle reportFilterLists wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei Tabelle reportFilterLists:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}

/** Tabelle   */
$sql = <<<EOS
 CREATE TABLE reportHints
    (
        reportID bigint unsigned  comment "Report-ID"
      , rLanguage char(2)  comment "Sprache des Reports"
      , SortNr Smallint  comment "Sortierfolge"
      , kommentar varchar(255)  comment "Zusätzlicher Kommentar"
      
      , PRIMARY KEY ( reportID, rLanguage, SortNr )
) ENGINE=InnoDB
EOS;

if ($dbh->exec($sql) !== false){
    $meldung .= "<br>Tabelle reportHints wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei Tabelle reportHints:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}


/** Tabelle   */
$sql = <<<EOS
 CREATE TABLE reportJoinedTables 
    (
        reportID bigint unsigned  comment "Report-ID" 
      , table_name varchar(64)  comment "Name der Tabelle" 
      , kennzeichen char(1)  comment "Kennzeichen der verknüpften Tabelle" 
      , prefix varchar(255)  comment "Präfix der verknüpften Tabelle" 
      , rJoinType TINYINT  comment "Art des Joins im WS-Reporter" 
      , historyOptions varchar(255)  comment "Optionen zur Historisierung" 

      , PRIMARY KEY ( reportID, kennzeichen )

) ENGINE=InnoDB
EOS;

if ($dbh->exec($sql) !== false){
    $meldung .= "<br>Tabelle reportJoinedTables wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei Tabelle reportJoinedTables:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}

/** Report-Verweise */

$sql = <<<EOS
 CREATE TABLE reportReports
 (
    reportID bigint unsigned  comment "Report-ID"
    , SortNr Smallint  comment "Sortierfolge"
    , reportTitle varchar(255)  comment "Titel des Reports"
    , copyFilter boolean  comment "Filter soll im Report-Aufruf mit übergeben werden"
    , kommentar varchar(255)  comment "Zusätzlicher Kommentar"
    
    , UNIQUE INDEX reportReports ( reportID, reportTitle )
    ) ENGINE=InnoDB
EOS;

if ($dbh->exec($sql) !== false){
    $meldung .= "<br>Tabelle reportReports wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei Tabelle reportReports:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}


/** Tabelle   */
$sql = <<<EOS
 CREATE TABLE reportRows 
    (
        reportID bigint unsigned  comment "Report-ID" 
      , SortNr Smallint  comment "Sortierfolge" 
      , column_name varchar(64)  comment "Name des Attributs" 
      , sort Smallint  comment "Sortierung der Ausgabe" 
      , webLinkColumn boolean  comment "Web-Link-Spalte" 
      , web_link_title varchar(255)  comment "Überschrift in Web-Link-Spalte" 
      , web_link_value varchar(255)  comment "Text in Web-Link-Spalte" 
      , web_link varchar(255)  comment "Web-Link " 
      , webCopyFilter boolean  comment "Filter in Web-Link übergeben" 
      , web_title varchar(255)  comment "Title-Anzeige über Web-Link" 
      , web_target varchar(255)  comment "Fenster-Name für Web-Link" 
      , fixedTableColumns boolean  comment "Spalte fixieren" 
      , summenZeilen boolean  comment "Summen-Zeilen ausgeben" 

      , PRIMARY KEY ( reportID, column_name )

) ENGINE=InnoDB
EOS;

if ($dbh->exec($sql) !== false){
    $meldung .= "<br>Tabelle reportRows wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei Tabelle reportRows:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}

/** Tabelle   */
$sql = <<<EOS
CREATE TABLE `reportTitle` (
 `reportID` bigint(20) unsigned NOT NULL AUTO_INCREMENT COMMENT 'Report-ID',
 `reportVersion` tinyint(4) DEFAULT 2 COMMENT 'Version der Report-Speicherung',
 `reportTitle_de` varchar(255) DEFAULT NULL COMMENT 'Titel des Reports in Deutsch',
 `reportTitle_en` varchar(255) DEFAULT NULL COMMENT 'Titel des Reports in Englisch',
 `reportTitle_fr` varchar(255) DEFAULT NULL COMMENT 'Titel des Reports in Französisch',
 PRIMARY KEY (`reportID`),
 UNIQUE KEY `reportTitle_de` (`reportTitle_de`),
 UNIQUE KEY `reportTitle_en` (`reportTitle_en`),
 UNIQUE KEY `reportTitle_fr` (`reportTitle_fr`)

) ENGINE=InnoDB
EOS;

if ($dbh->exec($sql) !== false){
    $meldung .= "<br>Tabelle reportTitle wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei Tabelle reportTitle:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}

/** Tabelle   */
$sql = <<<EOS
 CREATE TABLE reportValues 
    (
        column_name varchar(64)  comment "Name des Attributs" 
      , reportID bigint unsigned  comment "Report-ID" 
      , SortNr Smallint  comment "Sortierfolge" 
      , keineSummen boolean  comment "Keine Summen ausgeben" 
      , label varchar(1024)   comment "Angezeigter Text in der Auswahl" 
      , backgroundColor char(7)  comment "Hintergrundfarbe der Spalte" 
      , base char(1)  comment "Kennzeichen der Basis-Datenquelle für Berechnungen" 
      , dispval TINYINT  comment "Kennzeichen der Darstellung mit oder ohne Balken" 
      , skalierung TINYINT  comment "Kennzeichen der Gruppe zur Skalierung der Charts" 
      , wfFunction TINYINT DEFAULT 0  comment "Statistische Funktion für kumulierten Wert" 
      , wfValue varchar(64)  comment "Werte-Name für kumulierten Wert" 
      , abSpalte TINYINT DEFAULT 0  comment "Spalte, ab der der Filter wirken soll" 
      , nachSpalten boolean DEFAULT false comment "Window-Funktion spaltenweise anwenden"
 
      , PRIMARY KEY ( reportID, column_name )
) ENGINE=InnoDB
EOS;

if ($dbh->exec($sql) !== false){
    $meldung .= "<br>Tabelle reportValues wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei Tabelle reportValues:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}


/** ENDE Tabellen für Report-Definitionen   */


$sql = <<<EOS
CREATE TABLE `wsrUser` (
 `user` varchar(64) NOT NULL COMMENT 'Benutzerkennung',
 `password` varchar(64) DEFAULT NULL COMMENT 'Benutzerkennwort',
 `userLongName` varchar(250) DEFAULT NULL COMMENT 'Benutzername ',
 `wsrUebersetzer` BOOLEAN NOT NULL DEFAULT FALSE COMMENT 'Übersetzer im WS-Reporter',
 `reportTitle` varchar(250)  DEFAULT NULL COMMENT "Titel des Reports",
 `adminRecht` tinyint(1) DEFAULT NULL COMMENT 'Administrator-Rechte',
 `uLanguage` CHAR(2) NOT NULL DEFAULT 'de' COMMENT 'Sprache des Benutzers',
  endUser boolean  comment "Kennzeichen End-User", 
  rolle varchar(64) comment "Rolle in der Benutzerverwaltung",
  qAddFilter varchar(64)  comment "Name des benutzerspezischen Filters", 
  qAddFilterVon varchar(255) CHARACTER SET utf8  comment "Erster Wert des benutzerspezischen Filters", 
  qAddFilterBis varchar(255) CHARACTER SET utf8  comment "Zweiter Wert des benutzerspezifischen Filters", 
  qAddFilter2 varchar(64)  comment "Name des 2. benutzerspezischen Filters", 
  qAddFilter2Von varchar(255) CHARACTER SET utf8  comment "Erster Wert des 2. benutzerspezischen Filters", 
  qAddFilter2Bis varchar(255) CHARACTER SET utf8  comment "Zweiter Wert des 2. benutzerspezifischen Filters", 
 `error_count` smallint(5) unsigned DEFAULT NULL COMMENT 'Fehler-Zähler',
 PRIMARY KEY `user` (`user`)
) ENGINE=InnoDB 
EOS;

if ($dbh->exec($sql) !== false){
    $meldung .= "<br>Tabelle wsrUser wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei Tabelle wsrUser:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}


$sql = <<<EOS
 CREATE TABLE wsrRollen 
    (
        rolle varchar(64) PRIMARY KEY comment "Rolle in der Benutzerverwaltung" 
    )
    ;   
EOS;

if ($dbh->exec($sql) !== false){
    $meldung .= "<br>Tabelle wsrRollen wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei Tabelle wsrRollen:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}


$sql = <<<EOS
  CREATE TABLE wsrReports 
    (
        rolle varchar(64) not null  comment "Rolle in der Benutzerverwaltung" 
      , SortNr Smallint  comment "Sortierfolge" 
      , reportTitle varchar(255) not null  comment "Titel des Reports" 

      , primary key ( rolle, reportTitle )
    )
    ;  
EOS;

if ($dbh->exec($sql) !== false){
    $meldung .= "<br>Tabelle wsrReports wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei Tabelle wsrReports:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}


$sql = <<<EOS
CREATE TABLE `wsrRechte` (
 `user` varchar(64) NOT NULL COMMENT 'Benutzerkennung',
 `table_name` varchar(64) NOT NULL COMMENT 'Name der Tabelle',
 `adminRecht` tinyint(1) DEFAULT NULL COMMENT 'Administrator-Rechte',
 `qAddFilter` varchar(64) DEFAULT NULL COMMENT 'Name des benutzerspezischen Filters',
 `qAddFilterVon` VARCHAR(255) DEFAULT NULL COMMENT 'AddFilter von-Wert',
  qAddFilterBis varchar(255) CHARACTER SET utf8  comment "Zweiter Wert des benutzerspezifischen Filters", 
  qAddFilter2 varchar(64)  comment "Name des 2. benutzerspezischen Filters",
  qAddFilter2Von varchar(255) CHARACTER SET utf8  comment "Erster Wert des 2. benutzerspezischen Filters", 
  qAddFilter2Bis varchar(255) CHARACTER SET utf8  comment "Zweiter Wert des 2. benutzerspezifischen Filters",
 PRIMARY KEY `table_user` (`table_name`,`user`),
 KEY `user` (`user`)
) ENGINE=InnoDB 
EOS;

if ($dbh->exec($sql) !== false){
    $meldung .= "<br>Tabelle wsrRechte wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei Tabelle wsrRechte:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}



$sql = <<<EOS
CREATE TABLE wsrDefender
(
    IP_adresse char(40) PRIMARY KEY comment "IP-Adresse des Requests"
    , error_count smallint unsigned  comment "Fehler-Zähler"
    , datumUhrzeit datetime  comment "Datum und Uhrzeit des Eintrags"
    , error_type char(15)  comment "Fehler-Art"
    , webRequest varchar(255) CHARACTER SET utf8  comment "Web-Aufruf"    
    )
    ;   
EOS;

if ($dbh->exec($sql) !== false){
    $meldung .= "<br>Tabelle wsrDefender wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei Tabelle wsrDefender:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}

// Admin-Tabellen
$sql = <<<EOS
 CREATE TABLE wsrAdminQuery 
    (
        table_name varchar(64) PRIMARY KEY comment "Name der Tabelle" 
      , userCreated varchar(250)   comment "Benutzername des Erstellers" 
      , timeCreated datetime  comment "Datum und Uhrzeit der Erstellung" 
      , userStored varchar(250)   comment "Benutzername, der das Objekt zuletzt gespeichert hat" 
      , timeStored datetime  comment "Datum und Uhrzeit der letzten Speicherung" 
      , userReleased varchar(250)   comment "Benutzername, der das Objekt freigegeben hat" 
      , timeReleased datetime  comment "Datum und Uhrzeit der Freigabe" 
      , userResponsible varchar(250)   comment "Benutzername, der für das Objekt zuständig ist" 
      , lastUse date  comment "Datum der letzten Benutzung" 
      , useCounter bigint unsigned  default 0 comment "Zähler für die Benutzung eines Objekts" 
 
    )
    ;   
EOS;

if ($dbh->exec($sql) !== false){
    $meldung .= "<br>Tabelle wsrAdminQuery wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei Tabelle wsrAdminQuery:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}

$sql = <<<EOS
 CREATE TABLE wsrAdminReport 
    (
        reportID bigint unsigned PRIMARY KEY comment "Report-ID" 
      , userCreated varchar(250)   comment "Benutzername des Erstellers" 
      , timeCreated datetime  comment "Datum und Uhrzeit der Erstellung" 
      , userStored varchar(250)   comment "Benutzername, der das Objekt zuletzt gespeichert hat" 
      , timeStored datetime  comment "Datum und Uhrzeit der letzten Speicherung" 
      , userReleased varchar(250)   comment "Benutzername, der das Objekt freigegeben hat" 
      , timeReleased datetime  comment "Datum und Uhrzeit der Freigabe" 
      , userResponsible varchar(250)   comment "Benutzername, der für das Objekt zuständig ist" 
      , lastUse date  comment "Datum der letzten Benutzung" 
      , useCounter bigint unsigned  default 0 comment "Zähler für die Benutzung eines Objekts" 
 
    )
    ;   
EOS;

if ($dbh->exec($sql) !== false){
    $meldung .= "<br>Tabelle wsrAdminReport wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei Tabelle wsrAdminReport:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}


/* Log Tabelle zur Benutzung der Reports ist ersetzt durch wsrAdminReport

$sql = <<<EOS
CREATE TABLE wsrUsage
(
    reportID bigint unsigned PRIMARY KEY comment "Report-ID"
    , datum date  comment "Datum des Eintrags"
    , zaehler int unsigned  comment "Zähler"
    
    )
    ;   
EOS;

if ($dbh->exec($sql) !== false){
    $meldung .= "<br>Tabelle wsrUsage wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei Tabelle wsrUsage:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}
*/

// Views für Datenquellen erstellen
$sql = <<<EOS
   create view table_used as
    SELECT table_name, attribute as table_used FROM `queryAttribute` WHERE SatzTyp >= 100
    UNION select table_name, qDimName from queryDimensions
    ;
EOS;
if ($dbh->exec($sql) !== false){
    $meldung .= "<br>View table_used wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei View table_used:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}

$sql = <<<EOS
   create view table_label as
    SELECT table_name, attribute as label FROM `queryAttribute` WHERE SatzTyp = 2
    ;
EOS;
if ($dbh->exec($sql) !== false){
    $meldung .= "<br>View table_label wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei View table_label:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}



$sql = <<<EOS
 CREATE TABLE wsrTranslations 
    (
        zaehler int(10) unsigned NOT NULL DEFAULT 0
      , de varchar(1024) CHARACTER SET utf8 COLLATE utf8_bin NULL DEFAULT NULL  comment "Deutscher Text" 
      , en varchar(1024) CHARACTER SET utf8 COLLATE utf8_bin NULL DEFAULT NULL  comment "Englischer Text" 
      , fr varchar(1024) CHARACTER SET utf8 COLLATE utf8_bin NULL DEFAULT NULL  comment "Französischer Text" 
      , qLanguageType smallint unsigned  comment "Typ der Text-Eintrags für Übersetzungen" 

      , UNIQUE INDEX de ( de)
      , UNIQUE INDEX en ( en)
      , UNIQUE INDEX fr ( fr)
      , INDEX qlType ( qLanguageType )
    )
    ;
EOS;

if ($dbh->exec($sql) !== false){
    $meldung .= "<br>Tabelle wsrTranslations wurde erstellt.";
}else {
    $meldung .= "<br>Fehler bei Tabelle wsrTranslations:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}

$filename = './phpsec/wsrTranslations.sql';
$handle = fopen($filename, 'r');
$sql = fread($handle, filesize($filename));
fclose($handle);
if ($dbh->exec($sql) !== false){
    $meldung .= "<br>Tabelle wsrTranslations wurde geladen.";
}else {
    $meldung .= "<br>Fehler bei beim Laden der Tabelle wsrTranslations:<br>";
    $meldung .= $dbh->errorInfo()[2];
    $fertig = false;
}

?>